@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: compat.c,v 1.77 2005/11/15 03:05:14 nickm Exp $ */
-const char compat_c_id[] = "$Id: compat.c,v 1.77 2005/11/15 03:05:14 nickm Exp $";
+/* $Id: compat.c,v 1.78 2005/12/10 08:27:01 arma Exp $ */
+const char compat_c_id[] = "$Id: compat.c,v 1.78 2005/12/10 08:27:01 arma Exp $";
 
 /**
  * \file compat.c
@@ -463,7 +463,8 @@ set_max_file_descriptors(unsigned long limit, unsigned long cap)
     return -1;
   }
   if (rlim.rlim_max < limit) {
-    warn(LD_CONFIG,"We need %lu file descriptors available, and we're limited to %lu. Please change your ulimit -n.", limit, (unsigned long)rlim.rlim_max);
+    warn(LD_CONFIG,"We need %lu file descriptors available, and we're limited to %lu. Please change your ulimit -n.",
+         limit, (unsigned long)rlim.rlim_max);
     return -1;
   }
   most = (rlim.rlim_max > cap) ? cap : (unsigned) rlim.rlim_max;
