@@ -1,11 +1,11 @@
 /* Copyright 2003-2004 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: container.h,v 1.23 2005/11/01 03:47:55 arma Exp $ */
+/* $Id: container.h,v 1.27 2005/12/03 02:00:51 nickm Exp $ */
 
 #ifndef __CONTAINER_H
 #define __CONTAINER_H
-#define CONTAINER_H_ID "$Id: container.h,v 1.23 2005/11/01 03:47:55 arma Exp $"
+#define CONTAINER_H_ID "$Id: container.h,v 1.27 2005/12/03 02:00:51 nickm Exp $"
 
 #include "compat.h"
 #include "util.h"
@@ -25,12 +25,11 @@ smartlist_t *smartlist_create(void);
 void smartlist_free(smartlist_t *sl);
 void smartlist_set_capacity(smartlist_t *sl, int n);
 void smartlist_clear(smartlist_t *sl);
-void smartlist_truncate(smartlist_t *sl, int n);
 void smartlist_add(smartlist_t *sl, void *element);
 void smartlist_add_all(smartlist_t *sl, const smartlist_t *s2);
-void smartlist_remove(smartlist_t *sl, void *element);
+void smartlist_remove(smartlist_t *sl, const void *element);
 void smartlist_string_remove(smartlist_t *sl, const char *element);
-int smartlist_isin(const smartlist_t *sl, void *element);
+int smartlist_isin(const smartlist_t *sl, const void *element);
 int smartlist_string_isin(const smartlist_t *sl, const char *element);
 int smartlist_string_num_isin(const smartlist_t *sl, int num);
 int smartlist_overlap(const smartlist_t *sl1, const smartlist_t *sl2);
@@ -110,22 +109,21 @@ char *smartlist_join_strings2(smartlist_t *sl, const char *join,
 
 #define DECLARE_MAP_FNS(maptype, keytype, prefix)                       \
   typedef struct maptype maptype;                                       \
-  typedef struct prefix##entry_t prefix##iter_t;                        \
+  typedef struct prefix##entry_t *prefix##iter_t;                       \
   maptype* prefix##new(void);                                           \
   void* prefix##set(maptype *map, keytype key, void *val);              \
   void* prefix##get(maptype *map, keytype key);                         \
   void* prefix##remove(maptype *map, keytype key);                      \
-  typedef void* (*prefix##foreach_fn)(keytype key, void *val, void *data); \
-  void prefix##foreach(maptype *map, prefix##foreach_fn fn, void *data); \
   void prefix##free(maptype *map, void (*free_val)(void*));             \
   int prefix##isempty(maptype *map);                                    \
+  int prefix##size(maptype *map);                                       \
   prefix##iter_t *prefix##iter_init(maptype *map);                      \
   prefix##iter_t *prefix##iter_next(maptype *map, prefix##iter_t *iter); \
   prefix##iter_t *prefix##iter_next_rmv(maptype *map, prefix##iter_t *iter); \
   void prefix##iter_get(prefix##iter_t *iter, keytype *keyp, void **valp); \
   int prefix##iter_done(prefix##iter_t *iter);
 
-/* Map from const char * to void *. Implemented with a splay tree. */
+/* Map from const char * to void *. Implemented with a hash table. */
 DECLARE_MAP_FNS(strmap_t, const char *, strmap_);
 DECLARE_MAP_FNS(digestmap_t, const char *, digestmap_);
 
