@@ -1,8 +1,8 @@
 /* Copyright 2003 Roger Dingledine
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: util.c,v 1.233 2005/11/15 03:16:32 nickm Exp $ */
-const char util_c_id[] = "$Id: util.c,v 1.233 2005/11/15 03:16:32 nickm Exp $";
+/* $Id: util.c,v 1.239 2005/12/09 02:45:33 arma Exp $ */
+const char util_c_id[] = "$Id: util.c,v 1.239 2005/12/09 02:45:33 arma Exp $";
 
 /**
  * \file util.c
@@ -26,6 +26,7 @@ const char util_c_id[] = "$Id: util.c,v 1.233 2005/11/15 03:16:32 nickm Exp $";
 #include <direct.h>
 #else
 #include <dirent.h>
+#include <pwd.h>
 #endif
 
 #ifdef HAVE_CTYPE_H
@@ -417,6 +418,15 @@ find_whitespace(const char *s)
   return s;
 }
 
+/** Return true iff the DIGEST_LEN bytes in digest are all zero. */
+int
+tor_digest_is_zero(const char *digest)
+{
+  static char ZERO_DIGEST[] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
+
+  return !memcmp(digest, ZERO_DIGEST, DIGEST_LEN);
+}
+
 #define CHECK_STRTOX_RESULT()                           \
   /* Was at least one character converted? */           \
   if (endptr == s)                                      \
@@ -903,7 +913,19 @@ check_private_dir(const char *dirname, cpd_check_t check)
   }
 #ifndef MS_WINDOWS
   if (st.st_uid != getuid()) {
-    log(LOG_WARN, LD_FS, "%s is not owned by this UID (%d). You must fix this to proceed.", dirname, (int)getuid());
+    struct passwd *pw = NULL;
+    char *process_ownername = NULL;
+
+    pw = getpwuid(getuid());
+    process_ownername = pw ? tor_strdup(pw->pw_name) : tor_strdup("<unknown>");
+
+    pw = getpwuid(st.st_uid);
+
+    log(LOG_WARN, LD_FS, "%s is not owned by this user (%s, %d) but by %s (%d). Perhaps you are running Tor as the wrong user?",
+                         dirname, process_ownername, (int)getuid(),
+                         pw ? pw->pw_name : "<unknown>", (int)st.st_uid);
+
+    tor_free(process_ownername);
     return -1;
   }
   if (st.st_mode & 0077) {
@@ -1282,8 +1304,10 @@ tor_listdir(const char *dirname)
  * or reserved for local networks by RFC 1918.
  */
 int
-is_internal_IP(uint32_t ip)
+is_internal_IP(uint32_t ip, int for_listening)
 {
+  if (for_listening && !ip) /* special case for binding to 0.0.0.0 */
+    return 0;
   if (((ip & 0xff000000) == 0x0a000000) || /*       10/8 */
       ((ip & 0xff000000) == 0x00000000) || /*        0/8 */
       ((ip & 0xff000000) == 0x7f000000) || /*      127/8 */
@@ -1302,7 +1326,7 @@ is_internal_IP(uint32_t ip)
 int
 is_local_IP(uint32_t ip)
 {
-  return is_internal_IP(ip);
+  return is_internal_IP(ip, 0);
 }
 
 /** Parse a string of the form "host[:port]" from <b>addrport</b>.  If
