@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: buffers.c,v 1.178 2005/10/29 18:19:37 arma Exp $ */
-const char buffers_c_id[] = "$Id: buffers.c,v 1.178 2005/10/29 18:19:37 arma Exp $";
+/* $Id: buffers.c,v 1.182 2005/12/10 09:36:25 arma Exp $ */
+const char buffers_c_id[] = "$Id: buffers.c,v 1.182 2005/12/10 09:36:25 arma Exp $";
 
 /**
  * \file buffers.c
@@ -285,7 +285,7 @@ buf_shrink(buf_t *buf)
     return;
 
   debug(LD_MM,"Shrinking buffer from %d to %d bytes.",
-         (int)buf->len, (int)new_len);
+        (int)buf->len, (int)new_len);
   buf_resize(buf, new_len);
 }
 
@@ -413,7 +413,7 @@ read_to_buf_impl(int s, size_t at_most, buf_t *buf,
     if (buf->datalen > buf->highwater)
       buf->highwater = buf->datalen;
     debug(LD_NET,"Read %d bytes. %d on inbuf.",read_result,
-           (int)buf->datalen);
+          (int)buf->datalen);
     return read_result;
   }
 }
@@ -431,7 +431,7 @@ read_to_buf(int s, size_t at_most, buf_t *buf, int *reached_eof)
   char *next;
   size_t at_start;
 
-  assert_buf_ok(buf);
+  /* assert_buf_ok(buf); */
   tor_assert(reached_eof);
   tor_assert(s>=0);
 
@@ -479,8 +479,8 @@ read_to_buf_tls_impl(tor_tls_t *tls, size_t at_most, buf_t *buf, char *next)
   int r;
 
   debug(LD_NET,"before: %d on buf, %d pending, at_most %d.",
-         (int)buf_datalen(buf), (int)tor_tls_get_pending_bytes(tls),
-         (int)at_most);
+        (int)buf_datalen(buf), (int)tor_tls_get_pending_bytes(tls),
+        (int)at_most);
   r = tor_tls_read(tls, next, at_most);
   if (r<0)
     return r;
@@ -489,7 +489,7 @@ read_to_buf_tls_impl(tor_tls_t *tls, size_t at_most, buf_t *buf, char *next)
   if (buf->datalen > buf->highwater)
     buf->highwater = buf->datalen;
   debug(LD_NET,"Read %d bytes. %d on inbuf; %d pending",r,
-         (int)buf->datalen,(int)tor_tls_get_pending_bytes(tls));
+        (int)buf->datalen,(int)tor_tls_get_pending_bytes(tls));
   return r;
 }
 
@@ -523,8 +523,8 @@ read_to_buf_tls(tor_tls_t *tls, size_t at_most, buf_t *buf)
   assert_buf_ok(buf);
 
   debug(LD_NET,"start: %d on buf, %d pending, at_most %d.",
-         (int)buf_datalen(buf), (int)tor_tls_get_pending_bytes(tls),
-         (int)at_most);
+        (int)buf_datalen(buf), (int)tor_tls_get_pending_bytes(tls),
+        (int)at_most);
 
   if (buf_ensure_capacity(buf, at_most+buf->datalen))
     return TOR_TLS_ERROR;
@@ -594,7 +594,7 @@ flush_buf(int s, buf_t *buf, size_t sz, size_t *buf_flushlen)
   size_t flushed = 0;
   size_t flushlen0, flushlen1;
 
-  assert_buf_ok(buf);
+  /* assert_buf_ok(buf); */
   tor_assert(buf_flushlen);
   tor_assert(s>=0);
   tor_assert(*buf_flushlen <= buf->datalen);
@@ -610,7 +610,7 @@ flush_buf(int s, buf_t *buf, size_t sz, size_t *buf_flushlen)
   check();
 
   debug(LD_NET,"%d: flushed %d bytes, %d ready to flush, %d remain.",
-           s,r,(int)*buf_flushlen,(int)buf->datalen);
+        s,r,(int)*buf_flushlen,(int)buf->datalen);
   if (r < 0 || (size_t)r < flushlen0)
     return r; /* Error, or can't flush any more now. */
   flushed = r;
@@ -620,7 +620,7 @@ flush_buf(int s, buf_t *buf, size_t sz, size_t *buf_flushlen)
     r = flush_buf_impl(s, buf, flushlen1, buf_flushlen);
     check();
     debug(LD_NET,"%d: flushed %d bytes, %d ready to flush, %d remain.",
-           s,r,(int)*buf_flushlen,(int)buf->datalen);
+          s,r,(int)*buf_flushlen,(int)buf->datalen);
     if (r<0)
       return r;
     flushed += r;
@@ -645,7 +645,7 @@ flush_buf_tls_impl(tor_tls_t *tls, buf_t *buf, size_t sz, size_t *buf_flushlen)
   *buf_flushlen -= r;
   buf_remove_from_front(buf, r);
   debug(LD_NET,"flushed %d bytes, %d ready to flush, %d remain.",
-         r,(int)*buf_flushlen,(int)buf->datalen);
+        r,(int)*buf_flushlen,(int)buf->datalen);
   return r;
 }
 
@@ -657,7 +657,7 @@ flush_buf_tls(tor_tls_t *tls, buf_t *buf, size_t sz, size_t *buf_flushlen)
   int r;
   size_t flushed=0;
   size_t flushlen0, flushlen1;
-  assert_buf_ok(buf);
+  /* assert_buf_ok(buf); */
   tor_assert(tls);
   tor_assert(buf_flushlen);
   tor_assert(*buf_flushlen <= buf->datalen);
@@ -703,10 +703,11 @@ write_to_buf(const char *string, size_t string_len, buf_t *buf)
    */
 
   tor_assert(string);
-  assert_buf_ok(buf);
+  /* assert_buf_ok(buf); */
 
   if (buf_ensure_capacity(buf, buf->datalen+string_len)) {
-    warn(LD_MM, "buflen too small, can't hold %d bytes.", (int)(buf->datalen+string_len));
+    warn(LD_MM, "buflen too small, can't hold %d bytes.",
+         (int)(buf->datalen+string_len));
     return -1;
   }
 
@@ -726,7 +727,7 @@ write_to_buf(const char *string, size_t string_len, buf_t *buf)
   if (buf->datalen > buf->highwater)
     buf->highwater = buf->datalen;
   debug(LD_NET,"added %d bytes to buf (now %d total).",
-         (int)string_len, (int)buf->datalen);
+        (int)string_len, (int)buf->datalen);
   check();
   return buf->datalen;
 }
@@ -746,7 +747,7 @@ peek_from_buf(char *string, size_t string_len, buf_t *buf)
 
   tor_assert(string);
   tor_assert(string_len <= buf->datalen); /* make sure we don't ask for too much */
-  assert_buf_ok(buf);
+  /* assert_buf_ok(buf); */
 
   _split_range(buf, buf->cur, &string_len, &len2);
 
@@ -803,7 +804,7 @@ fetch_from_buf_http(buf_t *buf,
   char *headers, *body, *p;
   size_t headerlen, bodylen, contentlen;
 
-  assert_buf_ok(buf);
+  /* assert_buf_ok(buf); */
   buf_normalize(buf);
 
   if (buf_nul_terminate(buf)<0) {
@@ -822,12 +823,13 @@ fetch_from_buf_http(buf_t *buf,
   debug(LD_HTTP,"headerlen %d, bodylen %d.", (int)headerlen, (int)bodylen);
 
   if (max_headerlen <= headerlen) {
-    warn(LD_HTTP,"headerlen %d larger than %d. Failing.", (int)headerlen,
-           (int)max_headerlen-1);
+    warn(LD_HTTP,"headerlen %d larger than %d. Failing.",
+         (int)headerlen, (int)max_headerlen-1);
     return -1;
   }
   if (max_bodylen <= bodylen) {
-    warn(LD_HTTP,"bodylen %d larger than %d. Failing.", (int)bodylen, (int)max_bodylen-1);
+    warn(LD_HTTP,"bodylen %d larger than %d. Failing.",
+         (int)bodylen, (int)max_bodylen-1);
     return -1;
   }
 
@@ -888,10 +890,13 @@ fetch_from_buf_http(buf_t *buf,
  * If you want to specify the socks reply, write it into <b>req->reply</b>
  *   and set <b>req->replylen</b>, else leave <b>req->replylen</b> alone.
  *
+ * If <b>log_sockstype</b> is non-zero, then do a notice-level log of whether
+ * the connection is possibly leaking DNS requests locally or not.
+ *
  * If returning 0 or -1, <b>req->address</b> and <b>req->port</b> are undefined.
  */
 int
-fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
+fetch_from_buf_socks(buf_t *buf, socks_request_t *req, int log_sockstype)
 {
   unsigned char len;
   char tmpbuf[INET_NTOA_BUF_LEN];
@@ -924,7 +929,7 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
           req->reply[1] = '\xFF'; /* reject all methods */
           return -1;
         }
-        buf_remove_from_front(buf,2+nummethods);/* remove packet from buf */
+        buf_remove_from_front(buf,2+nummethods); /* remove packet from buf */
 
         req->replylen = 2; /* 2 bytes of response */
         req->reply[0] = 5; /* socks5 reply */
@@ -942,7 +947,7 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
           req->command != SOCKS_COMMAND_RESOLVE) {
         /* not a connect or resolve? we don't support it. */
         warn(LD_APP,"socks5: command %d not recognized. Rejecting.",
-               req->command);
+             req->command);
         return -1;
       }
       switch (*(buf->cur+3)) { /* address type */
@@ -956,7 +961,7 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
           tor_inet_ntoa(&in,tmpbuf,sizeof(tmpbuf));
           if (strlen(tmpbuf)+1 > MAX_SOCKS_ADDR_LEN) {
             warn(LD_APP,"socks5 IP takes %d bytes, which doesn't fit in %d. Rejecting.",
-                   (int)strlen(tmpbuf)+1,(int)MAX_SOCKS_ADDR_LEN);
+                 (int)strlen(tmpbuf)+1,(int)MAX_SOCKS_ADDR_LEN);
             return -1;
           }
           strlcpy(req->address,tmpbuf,sizeof(req->address));
@@ -964,7 +969,12 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
           buf_remove_from_front(buf, 10);
           if (!address_is_in_virtual_range(req->address) &&
               !have_warned_about_unsafe_socks) {
-            warn(LD_APP,"Your application (using socks5 on port %d) is giving Tor only an IP address. Applications that do DNS resolves themselves may leak information. Consider using Socks4A (e.g. via privoxy or socat) instead.  For more information, please see http://wiki.noreply.org/noreply/TheOnionRouter/TorFAQ#SOCKSAndDNS", req->port);
+            warn(LD_APP,"Your application (using socks5 on port %d) is giving "
+                 "Tor only an IP address. Applications that do DNS resolves "
+                 "themselves may leak information. Consider using Socks4A "
+                 "(e.g. via privoxy or socat) instead.  For more information, "
+                 "please see http://wiki.noreply.org/noreply/TheOnionRouter/"
+                 "TorFAQ#SOCKSAndDNS", req->port);
 //            have_warned_about_unsafe_socks = 1; // (for now, warn every time)
           }
           return 1;
@@ -975,13 +985,17 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
             return 0; /* not yet */
           if (len+1 > MAX_SOCKS_ADDR_LEN) {
             warn(LD_APP,"socks5 hostname is %d bytes, which doesn't fit in %d. Rejecting.",
-                   len+1,MAX_SOCKS_ADDR_LEN);
+                 len+1,MAX_SOCKS_ADDR_LEN);
             return -1;
           }
           memcpy(req->address,buf->cur+5,len);
           req->address[len] = 0;
           req->port = ntohs(get_uint16(buf->cur+5+len));
           buf_remove_from_front(buf, 5+len+2);
+          if (log_sockstype)
+            notice(LD_APP, "Your application (using socks5 on port %d) gave "
+                   "Tor a hostname, which means Tor will do the DNS resolve "
+                   "for you. This is good.", req->port);
           return 1;
         default: /* unsupported */
           warn(LD_APP,"socks5: unsupported address type %d. Rejecting.",*(buf->cur+3));
@@ -1001,7 +1015,7 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
           req->command != SOCKS_COMMAND_RESOLVE) {
         /* not a connect or resolve? we don't support it. */
         warn(LD_APP,"socks4: command %d not recognized. Rejecting.",
-               req->command);
+             req->command);
         return -1;
       }
 
@@ -1017,7 +1031,7 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
         tor_inet_ntoa(&in,tmpbuf,sizeof(tmpbuf));
         if (strlen(tmpbuf)+1 > MAX_SOCKS_ADDR_LEN) {
           debug(LD_APP,"socks4 addr (%d bytes) too long. Rejecting.",
-                 (int)strlen(tmpbuf));
+                (int)strlen(tmpbuf));
           return -1;
         }
         debug(LD_APP,"socks4: successfully read destip (%s)", safe_str(tmpbuf));
@@ -1036,7 +1050,10 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
       if (socks4_prot != socks4a &&
           !address_is_in_virtual_range(tmpbuf) &&
           !have_warned_about_unsafe_socks) {
-        warn(LD_APP,"Your application (using socks4 on port %d) is giving Tor only an IP address. Applications that do DNS resolves themselves may leak information. Consider using Socks4A (e.g. via privoxy or socat) instead.", req->port);
+        warn(LD_APP,"Your application (using socks4 on port %d) is giving Tor "
+             "only an IP address. Applications that do DNS resolves "
+             "themselves may leak information. Consider using Socks4A (e.g. "
+             "via privoxy or socat) instead.", req->port);
 //      have_warned_about_unsafe_socks = 1; // (for now, warn every time)
       }
       if (socks4_prot == socks4a) {
@@ -1055,6 +1072,10 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
           return -1;
         }
         tor_assert(next < buf->cur+buf->datalen);
+        if (log_sockstype)
+          notice(LD_APP, "Your application (using socks4a on port %d) gave "
+                 "Tor a hostname, which means Tor will do the DNS resolve "
+                 "for you. This is good.", req->port);
       }
       debug(LD_APP,"socks4: Everything is here. Success.");
       strlcpy(req->address, startaddr ? startaddr : tmpbuf,
@@ -1082,7 +1103,9 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
 "</p>\n"
 "<p>\n"
 "See <a href=\"http://tor.eff.org/documentation.html\">http://tor.eff.org/documentation.html</a> for more information.\n"
-"<!-- Plus this comment, to make the body response more than 512 bytes, so IE will be willing to display it. Comment comment comment comment comment comment comment comment comment comment comment comment.-->\n"
+"<!-- Plus this comment, to make the body response more than 512 bytes, so "
+"     IE will be willing to display it. Comment comment comment comment "
+"     comment comment comment comment comment comment comment comment.-->\n"
 "</p>\n"
 "</body>\n"
 "</html>\n"
@@ -1091,7 +1114,7 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req)
       /* fall through */
     default: /* version is not socks4 or socks5 */
       warn(LD_APP,"Socks version %d not recognized. (Tor is not an http proxy.)",
-             *(buf->cur));
+           *(buf->cur));
       return -1;
   }
 }
