@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitbuild.c,v 1.158 2005/11/05 20:15:26 nickm Exp $ */
-const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.158 2005/11/05 20:15:26 nickm Exp $";
+/* $Id: circuitbuild.c,v 1.174 2005/12/10 21:07:21 arma Exp $ */
+const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.174 2005/12/10 21:07:21 arma Exp $";
 
 /**
  * \file circuitbuild.c
@@ -279,7 +279,7 @@ circuit_t *
 circuit_init(uint8_t purpose, int need_uptime, int need_capacity, int internal)
 {
   circuit_t *circ = circuit_new(0, NULL); /* sets circ->p_circ_id and circ->p_conn */
-  circ->state = CIRCUIT_STATE_OR_WAIT;
+  circuit_set_state(circ, CIRCUIT_STATE_OR_WAIT);
   circ->build_state = tor_malloc_zero(sizeof(cpath_build_state_t));
   circ->build_state->need_uptime = need_uptime;
   circ->build_state->need_capacity = need_capacity;
@@ -338,17 +338,24 @@ circuit_handle_first_hop(circuit_t *circ)
   in.s_addr = htonl(firsthop->extend_info->addr);
   tor_inet_ntoa(&in, tmpbuf, sizeof(tmpbuf));
   debug(LD_CIRC,"Looking for firsthop '%s:%u'",tmpbuf,
-         firsthop->extend_info->port);
+        firsthop->extend_info->port);
   /* imprint the circuit with its future n_conn->id */
   memcpy(circ->n_conn_id_digest, firsthop->extend_info->identity_digest,
          DIGEST_LEN);
-  n_conn = connection_get_by_identity_digest(
-         firsthop->extend_info->identity_digest, CONN_TYPE_OR);
-  if (!n_conn || n_conn->state != OR_CONN_STATE_OPEN) { /* not currently connected */
+  n_conn = connection_or_get_by_identity_digest(
+         firsthop->extend_info->identity_digest);
+  /* If we don't have an open conn, or the conn we have is obsolete
+   * (i.e. old or broken) and the other side will let us make a second
+   * connection without dropping it immediately... */
+  if (!n_conn || n_conn->state != OR_CONN_STATE_OPEN ||
+      (n_conn->is_obsolete &&
+       router_digest_version_as_new_as(firsthop->extend_info->identity_digest,
+                                       "0.1.1.9-alpha-cvs"))) {
+    /* not currently connected */
     circ->n_addr = firsthop->extend_info->addr;
     circ->n_port = firsthop->extend_info->port;
 
-    if (!n_conn) { /* launch the connection */
+    if (!n_conn || n_conn->is_obsolete) { /* launch the connection */
       n_conn = connection_or_connect(firsthop->extend_info->addr,
                                      firsthop->extend_info->port,
                                      firsthop->extend_info->identity_digest);
@@ -384,16 +391,20 @@ circuit_handle_first_hop(circuit_t *circ)
 void
 circuit_n_conn_done(connection_t *or_conn, int status)
 {
-  circuit_t *circ;
+  extern smartlist_t *circuits_pending_or_conns;
 
   debug(LD_CIRC,"or_conn to %s, status=%d",
-         or_conn->nickname ? or_conn->nickname : "NULL", status);
+        or_conn->nickname ? or_conn->nickname : "NULL", status);
 
-  for (circ=global_circuitlist;circ;circ = circ->next) {
+  if (!circuits_pending_or_conns)
+    return;
+
+  SMARTLIST_FOREACH(circuits_pending_or_conns, circuit_t *, circ,
+  {
     if (circ->marked_for_close)
       continue;
-    if (circ->state == CIRCUIT_STATE_OR_WAIT &&
-        !circ->n_conn &&
+    tor_assert(circ->state == CIRCUIT_STATE_OR_WAIT);
+    if (!circ->n_conn &&
         circ->n_addr == or_conn->addr &&
         circ->n_port == or_conn->port &&
         !memcmp(or_conn->identity_digest, circ->n_conn_id_digest, DIGEST_LEN)) {
@@ -403,8 +414,10 @@ circuit_n_conn_done(connection_t *or_conn, int status)
         continue;
       }
       debug(LD_CIRC,"Found circ %d, sending create cell.", circ->n_circ_id);
+      /* circuit_deliver_create_cell will set n_circ_id and add us to
+       * orconn_circuid_circuit_map, so we don't need to call
+       * set_circid_orconn here. */
       circ->n_conn = or_conn;
-      memcpy(circ->n_conn_id_digest, or_conn->identity_digest, DIGEST_LEN);
       if (CIRCUIT_IS_ORIGIN(circ)) {
         if (circuit_send_next_onion_skin(circ) < 0) {
           info(LD_CIRC,"send_next_onion_skin failed; circuit marked for closing.");
@@ -420,12 +433,14 @@ circuit_n_conn_done(connection_t *or_conn, int status)
           continue;
         }
         tor_free(circ->onionskin);
+        circuit_set_state(circ, CIRCUIT_STATE_OPEN);
       }
     }
-  }
+  });
 }
 
-/** Find a new circid that isn't currently in use by the outgoing
+/** Find a new circid that isn't currently in use on the circ->n_conn
+ * for the outgoing
  * circuit <b>circ</b>, and deliver a cell of type <b>cell_type</b>
  * (either CELL_CREATE or CELL_CREATE_FAST) with payload <b>payload</b>
  * to this circuit.
@@ -473,13 +488,33 @@ inform_testing_reachability(void)
   if (me->dir_port)
     tor_snprintf(dirbuf, sizeof(dirbuf), " and DirPort %s:%d",
                  me->address, me->dir_port);
-  log(LOG_NOTICE, LD_OR, "Now checking whether ORPort %s:%d%s %s reachable... (this may take several minutes)",
+  log(LOG_NOTICE, LD_OR, "Now checking whether ORPort %s:%d%s %s reachable... "
+                         "(this may take up to %d minutes -- look for log "
+                         "messages indicating success)",
       me->address, me->or_port,
       me->dir_port ? dirbuf : "",
-      me->dir_port ? "are" : "is");
+      me->dir_port ? "are" : "is",
+      TIMEOUT_UNTIL_UNREACHABILITY_COMPLAINT/60);
   return 1;
 }
 
+/** Return true iff we should send a create_fast cell to build a circuit
+ * starting at <b>router</b>.  (If <b>router</b> is NULL, we don't have
+ * information on the router. */
+static INLINE int
+should_use_create_fast_for_router(routerinfo_t *router)
+{
+  or_options_t *options = get_options();
+
+  if (!options->FastFirstHopPK || options->ORPort)
+    return 0;
+  else if (!router || !router->platform ||
+           !tor_version_as_new_as(router->platform, "0.1.0.6-rc"))
+    return 0;
+  else
+    return 1;
+}
+
 extern int has_completed_circuit;
 
 /** This is the backbone function for building circuits.
@@ -505,14 +540,13 @@ circuit_send_next_onion_skin(circuit_t *circ)
   tor_assert(CIRCUIT_IS_ORIGIN(circ));
 
   if (circ->cpath->state == CPATH_STATE_CLOSED) {
+    int fast;
     uint8_t cell_type;
     debug(LD_CIRC,"First skin; sending create cell.");
 
     router = router_get_by_digest(circ->n_conn->identity_digest);
-
-    if (1 || /* Disable this '1' once we believe CREATE_FAST works. XXXX */
-        (get_options()->ORPort || !router || !router->platform ||
-         !tor_version_as_new_as(router->platform, "0.1.0.6-rc"))) {
+    fast = should_use_create_fast_for_router(router);
+    if (! fast) {
       /* We are an OR, or we are connecting to an old Tor: we should
        * send an old slow create cell.
        */
@@ -524,8 +558,9 @@ circuit_send_next_onion_skin(circuit_t *circ)
         return -1;
       }
     } else {
-      /* We are not an OR, and we're building the first hop of a circuit to
-       * a new OR: we can be speedy. */
+      /* We are not an OR, and we're building the first hop of a circuit to a
+       * new OR: we can be speedy and use CREATE_FAST to save an RSA operation
+       * and a DH operation. */
       cell_type = CELL_CREATE_FAST;
       memset(payload, 0, sizeof(payload));
       crypto_rand(circ->cpath->fast_handshake_state,
@@ -538,8 +573,9 @@ circuit_send_next_onion_skin(circuit_t *circ)
       return -1;
 
     circ->cpath->state = CPATH_STATE_AWAITING_KEYS;
-    circ->state = CIRCUIT_STATE_BUILDING;
-    debug(LD_CIRC,"first skin; finished sending create cell.");
+    circuit_set_state(circ, CIRCUIT_STATE_BUILDING);
+    info(LD_CIRC,"First hop: finished sending %s cell to '%s'",
+         fast ? "CREATE_FAST" : "CREATE", router->nickname);
   } else {
     tor_assert(circ->cpath->state == CPATH_STATE_OPEN);
     tor_assert(circ->state == CIRCUIT_STATE_BUILDING);
@@ -547,7 +583,7 @@ circuit_send_next_onion_skin(circuit_t *circ)
     hop = onion_next_hop_in_cpath(circ->cpath);
     if (!hop) {
       /* done building the circuit. whew. */
-      circ->state = CIRCUIT_STATE_OPEN;
+      circuit_set_state(circ, CIRCUIT_STATE_OPEN);
       info(LD_CIRC,"circuit built!");
       circuit_reset_failure_count(0);
       if (!has_completed_circuit) {
@@ -623,7 +659,9 @@ circuit_extend(cell_t *cell, circuit_t *circ)
   relay_header_unpack(&rh, cell->payload);
 
   if (rh.length < 4+2+ONIONSKIN_CHALLENGE_LEN+DIGEST_LEN) {
-    log_fn(LOG_PROTOCOL_WARN, LD_PROTOCOL,"Wrong length %d on extend cell. Closing circuit.", rh.length);
+    log_fn(LOG_PROTOCOL_WARN, LD_PROTOCOL,
+           "Wrong length %d on extend cell. Closing circuit.",
+           rh.length);
     return -1;
   }
 
@@ -632,9 +670,14 @@ circuit_extend(cell_t *cell, circuit_t *circ)
 
   onionskin = cell->payload+RELAY_HEADER_SIZE+4+2;
   id_digest = cell->payload+RELAY_HEADER_SIZE+4+2+ONIONSKIN_CHALLENGE_LEN;
-  n_conn = connection_get_by_identity_digest(id_digest, CONN_TYPE_OR);
+  n_conn = connection_or_get_by_identity_digest(id_digest);
 
-  if (!n_conn || n_conn->state != OR_CONN_STATE_OPEN) {
+  /* If we don't have an open conn, or the conn we have is obsolete
+   * (i.e. old or broken) and the other side will let us make a second
+   * connection without dropping it immediately... */
+  if (!n_conn || n_conn->state != OR_CONN_STATE_OPEN ||
+    (n_conn->is_obsolete &&
+     router_digest_version_as_new_as(id_digest,"0.1.1.9-alpha-cvs"))) {
      /* Note that this will close circuits where the onion has the same
      * router twice in a row in the path. I think that's ok.
      */
@@ -643,16 +686,16 @@ circuit_extend(cell_t *cell, circuit_t *circ)
     in.s_addr = htonl(circ->n_addr);
     tor_inet_ntoa(&in,tmpbuf,sizeof(tmpbuf));
     info(LD_CIRC|LD_OR,"Next router (%s:%d) not connected. Connecting.",
-           tmpbuf, circ->n_port);
+         tmpbuf, circ->n_port);
 
     circ->onionskin = tor_malloc(ONIONSKIN_CHALLENGE_LEN);
     memcpy(circ->onionskin, onionskin, ONIONSKIN_CHALLENGE_LEN);
-    circ->state = CIRCUIT_STATE_OR_WAIT;
+    circuit_set_state(circ, CIRCUIT_STATE_OR_WAIT);
 
     /* imprint the circuit with its future n_conn->id */
     memcpy(circ->n_conn_id_digest, id_digest, DIGEST_LEN);
 
-    if (n_conn) {
+    if (n_conn && !n_conn->is_obsolete) {
       circ->n_addr = n_conn->addr;
       circ->n_port = n_conn->port;
     } else {
@@ -737,9 +780,10 @@ circuit_init_cpath_crypto(crypt_path_t *cpath, char *key_data, int reverse)
   return 0;
 }
 
-/** A created or extended cell came back to us on the circuit,
- * and it included <b>reply</b> (the second DH key, plus KH).
- * DOCDOC reply_type.
+/** A created or extended cell came back to us on the circuit, and it included
+ * <b>reply</b> as its body.  (If <b>reply_type</b> is CELL_CREATED, the body
+ * contains (the second DH key, plus KH).  If <b>reply_type</b> is
+ * CELL_CREATED_FAST, the body contains a secret y and a hash H(x|y).)
  *
  * Calculate the appropriate keys and digests, make sure KH is
  * correct, and initialize this hop of the cpath.
@@ -795,7 +839,8 @@ circuit_finish_handshake(circuit_t *circ, uint8_t reply_type, char *reply)
   }
 
   hop->state = CPATH_STATE_OPEN;
-  info(LD_CIRC,"Finished building circuit hop:");
+  info(LD_CIRC,"Finished building %scircuit hop:",
+       (reply_type == CELL_CREATED_FAST) ? "fast " : "");
   circuit_log_path(LOG_INFO,LD_CIRC,circ);
   control_event_circuit_status(circ, CIRC_EVENT_EXTENDED);
 
@@ -867,13 +912,13 @@ onionskin_answer(circuit_t *circ, uint8_t cell_type, char *payload, char *keys)
   cell.command = cell_type;
   cell.circ_id = circ->p_circ_id;
 
-  circ->state = CIRCUIT_STATE_OPEN;
+  circuit_set_state(circ, CIRCUIT_STATE_OPEN);
 
   memcpy(cell.payload, payload,
          cell_type == CELL_CREATED ? ONIONSKIN_REPLY_LEN : DIGEST_LEN*2);
 
   debug(LD_CIRC,"init digest forward 0x%.8x, backward 0x%.8x.",
-         (unsigned int)*(uint32_t*)(keys), (unsigned int)*(uint32_t*)(keys+20));
+        (unsigned int)*(uint32_t*)(keys), (unsigned int)*(uint32_t*)(keys+20));
   if (circuit_init_cpath_crypto(tmp_cpath, keys, 0)<0) {
     warn(LD_BUG,"Circuit initialization failed");
     tor_free(tmp_cpath);
@@ -932,19 +977,19 @@ new_route_len(double cw, uint8_t purpose, extend_info_t *exit,
     routelen++;
 #endif
   debug(LD_CIRC,"Chosen route length %d (%d routers available).",routelen,
-         smartlist_len(routers));
+        smartlist_len(routers));
 
   num_acceptable_routers = count_acceptable_routers(routers);
 
   if (num_acceptable_routers < 2) {
     info(LD_CIRC,"Not enough acceptable routers (%d). Discarding this circuit.",
-           num_acceptable_routers);
+         num_acceptable_routers);
     return -1;
   }
 
   if (num_acceptable_routers < routelen) {
     info(LD_CIRC,"Not enough routers: cutting routelen from %d to %d.",
-           routelen, num_acceptable_routers);
+         routelen, num_acceptable_routers);
     routelen = num_acceptable_routers;
   }
 
@@ -1147,7 +1192,7 @@ choose_good_exit_server_general(routerlist_t *dir, int need_uptime,
     }
   }
   info(LD_CIRC, "Found %d servers that might support %d/%d pending connections.",
-         n_best_support, best_support, n_pending_connections);
+       n_best_support, best_support, n_pending_connections);
 
   preferredexits = smartlist_create();
   add_nickname_list_to_smartlist(preferredexits,options->ExitNodes,1,1);
@@ -1179,8 +1224,8 @@ choose_good_exit_server_general(routerlist_t *dir, int need_uptime,
     if (best_support == -1) {
       if (need_uptime || need_capacity) {
         info(LD_CIRC, "We couldn't find any live%s%s routers; falling back to list of all routers.",
-               need_capacity?", fast":"",
-               need_uptime?", stable":"");
+             need_capacity?", fast":"",
+             need_uptime?", stable":"");
         return choose_good_exit_server_general(dir, 0, 0);
       }
       notice(LD_CIRC, "All routers are down or middleman -- choosing a doomed exit at random.");
@@ -1237,18 +1282,21 @@ choose_good_exit_server_general(routerlist_t *dir, int need_uptime,
  */
 static routerinfo_t *
 choose_good_exit_server(uint8_t purpose, routerlist_t *dir,
-                        int need_uptime, int need_capacity)
+                        int need_uptime, int need_capacity, int is_internal)
 {
-  routerinfo_t *r;
   or_options_t *options = get_options();
   switch (purpose) {
     case CIRCUIT_PURPOSE_C_GENERAL:
-      return choose_good_exit_server_general(dir, need_uptime, need_capacity);
+      if (is_internal) /* pick it like a middle hop */
+        return router_choose_random_node(NULL, get_options()->ExcludeNodes,
+               NULL, need_uptime, need_capacity,
+               get_options()->_AllowUnverified & ALLOW_UNVERIFIED_MIDDLE, 0);
+      else
+        return choose_good_exit_server_general(dir, need_uptime, need_capacity);
     case CIRCUIT_PURPOSE_C_ESTABLISH_REND:
-      r = router_choose_random_node(options->RendNodes, options->RendExcludeNodes,
-          NULL, need_uptime, need_capacity,
-          options->_AllowUnverified & ALLOW_UNVERIFIED_RENDEZVOUS, 0);
-      return r;
+      return router_choose_random_node(options->RendNodes, options->RendExcludeNodes,
+             NULL, need_uptime, need_capacity,
+             options->_AllowUnverified & ALLOW_UNVERIFIED_RENDEZVOUS, 0);
   }
   warn(LD_BUG,"Bug: unhandled purpose %d", purpose);
   tor_fragile_assert();
@@ -1276,8 +1324,8 @@ onion_pick_cpath_exit(circuit_t *circ, extend_info_t *exit)
     exit = extend_info_dup(exit);
   } else { /* we have to decide one */
     routerinfo_t *router =
-           choose_good_exit_server(circ->purpose, rl,
-                                   state->need_uptime, state->need_capacity);
+           choose_good_exit_server(circ->purpose, rl, state->need_uptime,
+                                   state->need_capacity, state->is_internal);
     if (!router) {
       warn(LD_CIRC,"failed to choose an exit server");
       return -1;
@@ -1315,10 +1363,10 @@ int
 circuit_extend_to_new_exit(circuit_t *circ, extend_info_t *info)
 {
   circuit_append_new_exit(circ, info);
-  circ->state = CIRCUIT_STATE_BUILDING;
+  circuit_set_state(circ, CIRCUIT_STATE_BUILDING);
   if (circuit_send_next_onion_skin(circ)<0) {
     warn(LD_CIRC, "Couldn't extend circuit to new point '%s'.",
-           info->nickname);
+         info->nickname);
     circuit_mark_for_close(circ);
     return -1;
   }
@@ -1414,9 +1462,9 @@ choose_good_middle_server(uint8_t purpose,
 }
 
 /** Pick a good entry server for the circuit to be built according to
- * <b>state</b>.  Don't reuse a chosen exit (if any), don't use this router
- * (if we're an OR), and respect firewall settings; if we're using helper nodes,
- * return one.
+ * <b>state</b>.  Don't reuse a chosen exit (if any), don't use this
+ * router (if we're an OR), and respect firewall settings; if we're
+ * using helper nodes, return one.
  *
  * If <b>state</b> is NULL, we're choosing entries to serve as helper nodes,
  * not for any particular circuit.
@@ -1451,6 +1499,8 @@ choose_good_entry_server(cpath_build_state_t *state)
         smartlist_add(excluded, r);
     }
   }
+  // XXX we should exclude busy exit nodes here, too,
+  // but only if there are enough other nodes available.
   choice = router_choose_random_node(options->EntryNodes, options->ExcludeNodes,
            excluded, state ? state->need_uptime : 1,
            state ? state->need_capacity : 0,
@@ -1484,7 +1534,6 @@ onion_extend_cpath(uint8_t purpose, crypt_path_t **head_ptr,
   int cur_len;
   crypt_path_t *cpath;
   extend_info_t *info = NULL;
-  smartlist_t *excludednodes;
 
   tor_assert(head_ptr);
 
@@ -1499,15 +1548,12 @@ onion_extend_cpath(uint8_t purpose, crypt_path_t **head_ptr,
 
   if (cur_len >= state->desired_path_len) {
     debug(LD_CIRC, "Path is complete: %d steps long",
-           state->desired_path_len);
+          state->desired_path_len);
     return 1;
   }
 
   debug(LD_CIRC, "Path is %d long; we want %d", cur_len,
-         state->desired_path_len);
-
-  excludednodes = smartlist_create();
-  add_nickname_list_to_smartlist(excludednodes,get_options()->ExcludeNodes,0,1);
+        state->desired_path_len);
 
   if (cur_len == state->desired_path_len - 1) { /* Picking last node */
     info = extend_info_dup(state->chosen_exit);
@@ -1522,14 +1568,13 @@ onion_extend_cpath(uint8_t purpose, crypt_path_t **head_ptr,
       info = extend_info_from_router(r);
   }
 
-  smartlist_free(excludednodes);
   if (!info) {
     warn(LD_CIRC,"Failed to find node for hop %d of our path. Discarding this circuit.", cur_len);
     return -1;
   }
 
   debug(LD_CIRC,"Chose router %s for hop %d (exit is %s)",
-         info->nickname, cur_len+1, build_state_get_exit_nickname(state));
+        info->nickname, cur_len+1, build_state_get_exit_nickname(state));
 
   onion_append_hop(head_ptr, info);
   extend_info_free(info);
@@ -1717,7 +1762,7 @@ remove_dead_helpers(void)
       base16_encode(dbuf, sizeof(dbuf), helper->identity, DIGEST_LEN);
       format_local_iso_time(tbuf, since);
       warn(LD_CIRC, "Helper node '%s' (%s) has been %s since %s; removing.",
-             helper->nickname, dbuf, why, tbuf);
+           helper->nickname, dbuf, why, tbuf);
       tor_free(helper);
       smartlist_del(helper_nodes, i);
       helper_nodes_changed();
@@ -1759,13 +1804,13 @@ helper_nodes_set_status_from_directory(void)
           helper->unlisted_since = time(NULL);
           ++changed;
           warn(LD_CIRC,"Helper node '%s' is not listed by directories",
-                 helper->nickname);
+               helper->nickname);
           severity = LOG_WARN;
         }
       } else {
         if (helper->unlisted_since) {
           warn(LD_CIRC,"Helper node '%s' is listed again by directories",
-                 helper->nickname);
+               helper->nickname);
           ++changed;
           severity = LOG_WARN;
         }
@@ -1815,17 +1860,17 @@ helper_node_set_status(const char *digest, int succeeded)
           if (helper->down_since) {
             /*XXXX shouldn't warn. NM */
             warn(LD_CIRC,
-                   "Connection to formerly down helper node '%s' succeeded. "
-                   "%d/%d helpers usable.", helper->nickname,
-                   num_live_helpers(), smartlist_len(helper_nodes));
+                 "Connection to formerly down helper node '%s' succeeded. "
+                 "%d/%d helpers usable.", helper->nickname,
+                 num_live_helpers(), smartlist_len(helper_nodes));
             helper_nodes_changed();
           }
           helper->down_since = 0;
         } else if (!helper->down_since) {
           helper->down_since = time(NULL);
           warn(LD_CIRC,
-                 "Connection to helper node '%s' failed. %d/%d helpers usable.",
-                 helper->nickname, num_live_helpers(), smartlist_len(helper_nodes));
+               "Connection to helper node '%s' failed. %d/%d helpers usable.",
+               helper->nickname, num_live_helpers(), smartlist_len(helper_nodes));
           helper_nodes_changed();
         }
       }
