@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitlist.c,v 1.70 2005/11/11 19:28:32 arma Exp $ */
-const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.70 2005/11/11 19:28:32 arma Exp $";
+/* $Id: circuitlist.c,v 1.85 2005/12/10 09:36:25 arma Exp $ */
+const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.85 2005/12/10 09:36:25 arma Exp $";
 
 /**
  * \file circuitlist.c
@@ -12,16 +12,16 @@ const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.70 2005/11/11 19:28:32 a
 
 #include "or.h"
 
-/* Define RB_AUGMENT to avoid warnings about if statements with emtpy bodies.
- */
-#define RB_AUGMENT(x) do{}while(0)
-#include "tree.h"
+#include "../common/ht.h"
 
 /********* START VARIABLES **********/
 
 /** A global list of all circuits at this hop. */
 circuit_t *global_circuitlist=NULL;
 
+/** A list of all the circuits in CIRCUIT_STATE_OR_WAIT. */
+smartlist_t *circuits_pending_or_conns=NULL;
+
 static void circuit_free(circuit_t *circ);
 static void circuit_free_cpath(crypt_path_t *cpath);
 static void circuit_free_cpath_node(crypt_path_t *victim);
@@ -31,37 +31,39 @@ static void circuit_free_cpath_node(crypt_path_t *victim);
 /** A map from OR connection and circuit ID to circuit.  (Lookup performance is
  * very important here, since we need to do it every time a cell arrives.) */
 typedef struct orconn_circid_circuit_map_t {
-  RB_ENTRY(orconn_circid_circuit_map_t) node;
+  HT_ENTRY(orconn_circid_circuit_map_t) node;
   connection_t *or_conn;
   uint16_t circ_id;
   circuit_t *circuit;
 } orconn_circid_circuit_map_t;
 
-/** Helper for RB tree: compare the OR connection and circuit ID for a and b,
+/** Helper for hash tables: compare the OR connection and circuit ID for a and b,
  * and return less than, equal to, or greater than zero appropriately.
  */
 static INLINE int
-compare_orconn_circid_entries(orconn_circid_circuit_map_t *a,
-                              orconn_circid_circuit_map_t *b)
+_orconn_circid_entries_eq(orconn_circid_circuit_map_t *a,
+                          orconn_circid_circuit_map_t *b)
 {
-  if (a->or_conn < b->or_conn)
-    return -1;
-  else if (a->or_conn > b->or_conn)
-    return 1;
-  else
-    return ((int)b->circ_id) - ((int)a->circ_id);
-};
+  return a->or_conn == b->or_conn && a->circ_id == b->circ_id;
+}
 
-static RB_HEAD(orconn_circid_tree, orconn_circid_circuit_map_t) orconn_circid_circuit_map = RB_INITIALIZER(orconn_circid_circuit_map);
-RB_PROTOTYPE(orconn_circid_tree, orconn_circid_circuit_map_t, node, compare_orconn_circid_entries);
-RB_GENERATE(orconn_circid_tree, orconn_circid_circuit_map_t, node, compare_orconn_circid_entries);
+static INLINE unsigned int
+_orconn_circid_entry_hash(orconn_circid_circuit_map_t *a)
+{
+  return (((unsigned)a->circ_id)<<16) ^ (unsigned)(uintptr_t)(a->or_conn);
+}
+
+static HT_HEAD(orconn_circid_map, orconn_circid_circuit_map_t) orconn_circid_circuit_map = HT_INITIALIZER();
+HT_PROTOTYPE(orconn_circid_map, orconn_circid_circuit_map_t, node,
+             _orconn_circid_entry_hash, _orconn_circid_entries_eq);
+HT_GENERATE(orconn_circid_map, orconn_circid_circuit_map_t, node,
+            _orconn_circid_entry_hash, _orconn_circid_entries_eq, 0.6,
+            malloc, realloc, free);
 
 /** The most recently returned entry from circuit_get_by_circid_orconn;
  * used to improve performance when many cells arrive in a row from the
  * same circuit.
  */
-/* (We tried using splay trees, but round-robin turned out to make them
- * suck.) */
 orconn_circid_circuit_map_t *_last_circid_orconn_ent = NULL;
 
 /** Set the p_conn or n_conn field of a circuit <b>circ</b>, along
@@ -90,6 +92,8 @@ circuit_set_circid_orconn(circuit_t *circ, uint16_t id,
     circ->n_circ_id = id;
     circ->n_conn = conn;
   }
+  if (conn == old_conn && old_id == id)
+    return;
 
   if (_last_circid_orconn_ent &&
       ((old_id == _last_circid_orconn_ent->circ_id &&
@@ -100,13 +104,14 @@ circuit_set_circid_orconn(circuit_t *circ, uint16_t id,
   }
 
   if (old_conn) { /* we may need to remove it from the conn-circid map */
+    tor_assert(old_conn->magic == CONNECTION_MAGIC);
     search.circ_id = old_id;
     search.or_conn = old_conn;
-    found = RB_FIND(orconn_circid_tree, &orconn_circid_circuit_map, &search);
+    found = HT_REMOVE(orconn_circid_map, &orconn_circid_circuit_map, &search);
     if (found) {
-      RB_REMOVE(orconn_circid_tree, &orconn_circid_circuit_map, found);
+      tor_free(found);
     }
-    tor_free(found);
+    --old_conn->n_circuits;
   }
 
   if (conn == NULL)
@@ -115,7 +120,7 @@ circuit_set_circid_orconn(circuit_t *circ, uint16_t id,
   /* now add the new one to the conn-circid map */
   search.circ_id = id;
   search.or_conn = conn;
-  found = RB_FIND(orconn_circid_tree, &orconn_circid_circuit_map, &search);
+  found = HT_FIND(orconn_circid_map, &orconn_circid_circuit_map, &search);
   if (found) {
     found->circuit = circ;
   } else {
@@ -123,8 +128,31 @@ circuit_set_circid_orconn(circuit_t *circ, uint16_t id,
     found->circ_id = id;
     found->or_conn = conn;
     found->circuit = circ;
-    RB_INSERT(orconn_circid_tree, &orconn_circid_circuit_map, found);
+    HT_INSERT(orconn_circid_map, &orconn_circid_circuit_map, found);
   }
+  ++conn->n_circuits;
+}
+
+/** Change the state of <b>circ</b> to <b>state</b>, adding it to or removing
+ * it from lists as appropriate. */
+void
+circuit_set_state(circuit_t *circ, int state)
+{
+  tor_assert(circ);
+  if (state == circ->state)
+    return;
+  if (circ->state == CIRCUIT_STATE_OR_WAIT) {
+    /* remove from waiting-circuit list. */
+    if (circuits_pending_or_conns)
+      smartlist_remove(circuits_pending_or_conns, circ);
+  }
+  if (state == CIRCUIT_STATE_OR_WAIT) {
+    /* add to waiting-circuit list. */
+    if (!circuits_pending_or_conns)
+      circuits_pending_or_conns = smartlist_create();
+    smartlist_add(circuits_pending_or_conns, circ);
+  }
+  circ->state = state;
 }
 
 /** Add <b>circ</b> to the global list of circuits. This is called only from
@@ -185,7 +213,7 @@ circuit_state_to_string(int state)
   switch (state) {
     case CIRCUIT_STATE_BUILDING: return "doing handshakes";
     case CIRCUIT_STATE_ONIONSKIN_PENDING: return "processing the onion";
-    case CIRCUIT_STATE_OR_WAIT: return "connecting to firsthop";
+    case CIRCUIT_STATE_OR_WAIT: return "connecting to server";
     case CIRCUIT_STATE_OPEN: return "open";
     default:
       warn(LD_BUG, "Bug: unknown circuit state %d", state);
@@ -210,8 +238,6 @@ circuit_new(uint16_t p_circ_id, connection_t *p_conn)
 
   circ->timestamp_created = time(NULL);
 
-  circ->state = CIRCUIT_STATE_ONIONSKIN_PENDING;
-
   /* CircIDs */
   if (p_conn) {
     circuit_set_circid_orconn(circ, p_circ_id, p_conn, P_CONN_CHANGED);
@@ -300,6 +326,9 @@ circuit_free_all(void)
     circuit_free(global_circuitlist);
     global_circuitlist = next;
   }
+  smartlist_free(circuits_pending_or_conns);
+  circuits_pending_or_conns = NULL;
+  HT_CLEAR(orconn_circid_map, &orconn_circid_circuit_map);
 }
 
 /** Deallocate space associated with the cpath node <b>victim</b>. */
@@ -360,7 +389,7 @@ circuit_get_by_circid_orconn_impl(uint16_t circ_id, connection_t *conn)
   } else {
     search.circ_id = circ_id;
     search.or_conn = conn;
-    found = RB_FIND(orconn_circid_tree, &orconn_circid_circuit_map, &search);
+    found = HT_FIND(orconn_circid_map, &orconn_circid_circuit_map, &search);
     _last_circid_orconn_ent = found;
   }
   if (found && found->circuit)
@@ -373,11 +402,11 @@ circuit_get_by_circid_orconn_impl(uint16_t circ_id, connection_t *conn)
     circuit_t *circ;
     for (circ=global_circuitlist;circ;circ = circ->next) {
       if (circ->p_conn == conn && circ->p_circ_id == circ_id) {
-        warn(LD_BUG, "circuit matches p_conn, but not in tree (Bug!)");
+        warn(LD_BUG, "circuit matches p_conn, but not in hash table (Bug!)");
         return circ;
       }
       if (circ->n_conn == conn && circ->n_circ_id == circ_id) {
-        warn(LD_BUG, "circuit matches n_conn, but not in tree (Bug!)");
+        warn(LD_BUG, "circuit matches n_conn, but not in hash table (Bug!)");
         return circ;
       }
     }
@@ -422,69 +451,32 @@ circuit_t *
 circuit_get_by_edge_conn(connection_t *conn)
 {
   circuit_t *circ;
-#if 0
-  connection_t *tmpconn;
-#endif
   tor_assert(CONN_IS_EDGE(conn));
 
-  if (! conn->on_circuit) {
-    /* return NULL; */
-    circ = circuit_get_by_conn(conn);
-    if (circ) {
-      warn(LD_BUG, "BUG: conn->on_circuit==NULL, but there was in fact a circuit there.");
-    }
-    return circ;
-  }
-
   circ = conn->on_circuit;
-  tor_assert(circ->magic == CIRCUIT_MAGIC);
-#if 0
-  /* All this stuff here is sanity-checking. */
-  for (tmpconn = circ->p_streams; tmpconn; tmpconn=tmpconn->next_stream)
-    if (tmpconn == conn)
-      return circ;
-  for (tmpconn = circ->n_streams; tmpconn; tmpconn=tmpconn->next_stream)
-    if (tmpconn == conn)
-      return circ;
-  for (tmpconn = circ->resolving_streams; tmpconn; tmpconn=tmpconn->next_stream)
-    if (tmpconn == conn)
-      return circ;
+  tor_assert(!circ || circ->magic == CIRCUIT_MAGIC);
 
-  tor_assert(0);
-#endif
   return circ;
 }
 
-/** Return a circ such that circ is attached to <b>conn</b>, either as
- * p_conn, n_conn, or in p_streams or n_streams or resolving_streams.
- *
- * Return NULL if no such circuit exists.
+/** For each circuits that have <b>conn</b> as n_conn or p_conn, unlink the
+ * circuit from the orconn,circid map, and mark it for close if it hasn't
+ * been marked already.
  */
-circuit_t *
-circuit_get_by_conn(connection_t *conn)
+void
+circuit_unlink_all_from_or_conn(connection_t *conn)
 {
   circuit_t *circ;
-  connection_t *tmpconn;
-
-  for (circ=global_circuitlist;circ;circ = circ->next) {
-    if (circ->marked_for_close)
-      continue;
-
-    if (circ->p_conn == conn)
-      return circ;
-    if (circ->n_conn == conn)
-      return circ;
-    for (tmpconn = circ->p_streams; tmpconn; tmpconn=tmpconn->next_stream)
-      if (tmpconn == conn)
-        return circ;
-    for (tmpconn = circ->n_streams; tmpconn; tmpconn=tmpconn->next_stream)
-      if (tmpconn == conn)
-        return circ;
-    for (tmpconn = circ->resolving_streams; tmpconn; tmpconn=tmpconn->next_stream)
-      if (tmpconn == conn)
-        return circ;
+  for (circ = global_circuitlist; circ; circ = circ->next) {
+    if (circ->n_conn == conn || circ->p_conn == conn) {
+      if (circ->n_conn == conn)
+        circuit_set_circid_orconn(circ, 0, NULL, N_CONN_CHANGED);
+      if (circ->p_conn == conn)
+        circuit_set_circid_orconn(circ, 0, NULL, P_CONN_CHANGED);
+      if (!circ->marked_for_close)
+        circuit_mark_for_close(circ);
+    }
   }
-  return NULL;
 }
 
 /** Return a circ such that:
@@ -554,10 +546,10 @@ circuit_get_rendezvous(const char *cookie)
  * if required, and if info is defined, does not already use info
  * as any of its hops; or NULL if no circuit fits this description.
  *
- * Avoid returning need_uptime circuits if not necessary.
+ * Return need_uptime circuits if that is requested; and if it's not
+ * requested, return non-uptime circuits if possible, else either.
  *
- * FFFF As a more important goal, not yet implemented, avoid returning
- * internal circuits if not necessary.
+ * Only return internal circuits if that is requested.
  */
 circuit_t *
 circuit_find_to_cannibalize(uint8_t purpose, extend_info_t *info,
@@ -567,7 +559,8 @@ circuit_find_to_cannibalize(uint8_t purpose, extend_info_t *info,
   circuit_t *circ;
   circuit_t *best=NULL;
 
-  debug(LD_CIRC,"Hunting for a circ to cannibalize: purpose %d, uptime %d, capacity %d, internal %d", purpose, need_uptime, need_capacity, internal);
+  debug(LD_CIRC,"Hunting for a circ to cannibalize: purpose %d, uptime %d, capacity %d, internal %d",
+         purpose, need_uptime, need_capacity, internal);
 
   for (circ=global_circuitlist; circ; circ = circ->next) {
     if (CIRCUIT_IS_ORIGIN(circ) &&
@@ -577,7 +570,7 @@ circuit_find_to_cannibalize(uint8_t purpose, extend_info_t *info,
         !circ->timestamp_dirty &&
         (!need_uptime || circ->build_state->need_uptime) &&
         (!need_capacity || circ->build_state->need_capacity) &&
-        (!internal || circ->build_state->is_internal)) {
+        (internal == circ->build_state->is_internal)) {
       if (info) {
         /* need to make sure we don't duplicate hops */
         crypt_path_t *hop = circ->cpath;
@@ -677,6 +670,10 @@ _circuit_mark_for_close(circuit_t *circ, int line, const char *file)
     }
     circuit_rep_hist_note_result(circ);
   }
+  if (circ->state == CIRCUIT_STATE_OR_WAIT) {
+    if (circuits_pending_or_conns)
+      smartlist_remove(circuits_pending_or_conns, circ);
+  }
   if (CIRCUIT_IS_ORIGIN(circ)) {
     control_event_circuit_status(circ,
      (circ->state == CIRCUIT_STATE_OPEN)?CIRC_EVENT_CLOSED:CIRC_EVENT_FAILED);
@@ -686,8 +683,8 @@ _circuit_mark_for_close(circuit_t *circ, int line, const char *file)
     tor_assert(circ->build_state->chosen_exit);
     /* treat this like getting a nack from it */
     info(LD_REND,"Failed intro circ %s to %s (awaiting ack). Removing from descriptor.",
-           safe_str(circ->rend_query),
-           safe_str(build_state_get_exit_nickname(circ->build_state)));
+         safe_str(circ->rend_query),
+         safe_str(build_state_get_exit_nickname(circ->build_state)));
     rend_client_remove_intro_point(circ->build_state->chosen_exit,
                                    circ->rend_query);
   }
@@ -823,6 +820,13 @@ assert_circuit_ok(const circuit_t *c)
       tor_assert(c->p_digest);
     }
   }
+  if (c->state == CIRCUIT_STATE_OR_WAIT && !c->marked_for_close) {
+    tor_assert(circuits_pending_or_conns &&
+               smartlist_isin(circuits_pending_or_conns, c));
+  } else {
+    tor_assert(!circuits_pending_or_conns ||
+               !smartlist_isin(circuits_pending_or_conns, c));
+  }
   if (c->cpath) {
     assert_cpath_ok(c->cpath);
   }
