@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuituse.c,v 1.89 2005/11/15 03:05:18 nickm Exp $ */
-const char circuituse_c_id[] = "$Id: circuituse.c,v 1.89 2005/11/15 03:05:18 nickm Exp $";
+/* $Id: circuituse.c,v 1.102 2005/12/10 09:36:25 arma Exp $ */
+const char circuituse_c_id[] = "$Id: circuituse.c,v 1.102 2005/12/10 09:36:25 arma Exp $";
 
 /**
  * \file circuituse.c
@@ -29,10 +29,9 @@ static void circuit_increment_failure_count(void);
  * Else return 0.
  */
 static int
-circuit_is_acceptable(circuit_t *circ,
-                      connection_t *conn,
-                      int must_be_open,
-                      uint8_t purpose,
+circuit_is_acceptable(circuit_t *circ, connection_t *conn,
+                      int must_be_open, uint8_t purpose,
+                      int need_uptime, int need_internal,
                       time_t now)
 {
   routerinfo_t *exitrouter;
@@ -76,9 +75,9 @@ circuit_is_acceptable(circuit_t *circ,
    */
   exitrouter = build_state_get_exit_router(circ->build_state);
 
-  if (!circ->build_state->need_uptime &&
-      smartlist_string_num_isin(get_options()->LongLivedPorts,
-                                conn->socks_request->port))
+  if (need_uptime && !circ->build_state->need_uptime)
+    return 0;
+  if (need_internal != circ->build_state->is_internal)
     return 0;
 
   if (purpose == CIRCUIT_PURPOSE_C_GENERAL) {
@@ -153,7 +152,8 @@ circuit_is_better(circuit_t *a, circuit_t *b, uint8_t purpose)
  * closest introduce-purposed circuit that you can find.
  */
 static circuit_t *
-circuit_get_best(connection_t *conn, int must_be_open, uint8_t purpose)
+circuit_get_best(connection_t *conn, int must_be_open, uint8_t purpose,
+                 int need_uptime, int need_internal)
 {
   circuit_t *circ, *best=NULL;
   time_t now = time(NULL);
@@ -165,7 +165,8 @@ circuit_get_best(connection_t *conn, int must_be_open, uint8_t purpose)
              purpose == CIRCUIT_PURPOSE_C_REND_JOINED);
 
   for (circ=global_circuitlist;circ;circ = circ->next) {
-    if (!circuit_is_acceptable(circ,conn,must_be_open,purpose,now))
+    if (!circuit_is_acceptable(circ,conn,must_be_open,purpose,
+                               need_uptime,need_internal,now))
       continue;
 
     /* now this is an acceptable circ to hand back. but that doesn't
@@ -190,16 +191,15 @@ void
 circuit_expire_building(time_t now)
 {
   circuit_t *victim, *circ = global_circuitlist;
+  time_t cutoff = now - MIN_SECONDS_BEFORE_EXPIRING_CIRC;
 
   while (circ) {
     victim = circ;
     circ = circ->next;
-    if (!CIRCUIT_IS_ORIGIN(victim))
-      continue; /* didn't originate here */
-    if (victim->marked_for_close)
-      continue; /* don't mess with marked circs */
-    if (victim->timestamp_created + MIN_SECONDS_BEFORE_EXPIRING_CIRC > now)
-      continue; /* it's young still, don't mess with it */
+    if (!CIRCUIT_IS_ORIGIN(victim) || /* didn't originate here */
+        victim->timestamp_created > cutoff || /* Not old enough to expire */
+        victim->marked_for_close) /* don't mess with marked circs */
+      continue;
 
 #if 0
     /* some debug logs, to help track bugs */
@@ -221,38 +221,46 @@ circuit_expire_building(time_t now)
 
     /* if circ is !open, or if it's open but purpose is a non-finished
      * intro or rend, then mark it for close */
-    if (victim->state != CIRCUIT_STATE_OPEN ||
-        victim->purpose == CIRCUIT_PURPOSE_C_ESTABLISH_REND ||
-        victim->purpose == CIRCUIT_PURPOSE_C_INTRODUCING ||
-        victim->purpose == CIRCUIT_PURPOSE_S_ESTABLISH_INTRO ||
-
-        /* it's a rend_ready circ, but it's already picked a query */
-        (victim->purpose == CIRCUIT_PURPOSE_C_REND_READY &&
-         victim->rend_query[0]) ||
-
-        /* c_rend_ready circs measure age since timestamp_dirty,
-         * because that's set when they switch purposes
-         */
-        /* rend and intro circs become dirty each time they
-         * make an introduction attempt. so timestamp_dirty
-         * will reflect the time since the last attempt.
-         */
-        ((victim->purpose == CIRCUIT_PURPOSE_C_REND_READY ||
-          victim->purpose == CIRCUIT_PURPOSE_C_REND_READY_INTRO_ACKED ||
-          victim->purpose == CIRCUIT_PURPOSE_C_INTRODUCE_ACK_WAIT) &&
-         victim->timestamp_dirty + MIN_SECONDS_BEFORE_EXPIRING_CIRC > now)) {
-      if (victim->n_conn)
-        info(LD_CIRC,"Abandoning circ %s:%d:%d (state %d:%s, purpose %d)",
-               victim->n_conn->address, victim->n_port, victim->n_circ_id,
-               victim->state, circuit_state_to_string(victim->state), victim->purpose);
-      else
-        info(LD_CIRC,"Abandoning circ %d (state %d:%s, purpose %d)",
-             victim->n_circ_id, victim->state,
-             circuit_state_to_string(victim->state), victim->purpose);
-
-      circuit_log_path(LOG_INFO,LD_CIRC,victim);
-      circuit_mark_for_close(victim);
+    if (victim->state == CIRCUIT_STATE_OPEN) {
+      switch (victim->purpose) {
+        default: /* most open circuits can be left alone. */
+          continue; /* yes, continue inside a switch refers to the nearest
+                     * enclosing loop. C is smart. */
+        case CIRCUIT_PURPOSE_C_ESTABLISH_REND:
+        case CIRCUIT_PURPOSE_C_INTRODUCING:
+        case CIRCUIT_PURPOSE_S_ESTABLISH_INTRO:
+          break; /* too old, need to die */
+        case CIRCUIT_PURPOSE_C_REND_READY:
+          /* it's a rend_ready circ -- has it already picked a query? */
+          /* c_rend_ready circs measure age since timestamp_dirty,
+           * because that's set when they switch purposes
+           */
+          if (!victim->rend_query[0] || victim->timestamp_dirty > cutoff)
+            continue;
+          break;
+        case CIRCUIT_PURPOSE_C_REND_READY_INTRO_ACKED:
+        case CIRCUIT_PURPOSE_C_INTRODUCE_ACK_WAIT:
+          /* rend and intro circs become dirty each time they
+           * make an introduction attempt. so timestamp_dirty
+           * will reflect the time since the last attempt.
+           */
+          if (victim->timestamp_dirty > cutoff)
+            continue;
+          break;
+      }
     }
+
+    if (victim->n_conn)
+      info(LD_CIRC,"Abandoning circ %s:%d:%d (state %d:%s, purpose %d)",
+           victim->n_conn->address, victim->n_port, victim->n_circ_id,
+           victim->state, circuit_state_to_string(victim->state), victim->purpose);
+    else
+      info(LD_CIRC,"Abandoning circ %d (state %d:%s, purpose %d)",
+           victim->n_circ_id, victim->state,
+           circuit_state_to_string(victim->state), victim->purpose);
+
+    circuit_log_path(LOG_INFO,LD_CIRC,victim);
+    circuit_mark_for_close(victim);
   }
 }
 
@@ -278,8 +286,9 @@ circuit_remove_handled_ports(smartlist_t *needed_ports)
   }
 }
 
-/** Return 1 if at least <b>min</b> general-purpose circuits will have
- * an acceptable exit node for conn if conn is defined, else for "*:port".
+/** Return 1 if at least <b>min</b> general-purpose non-internal circuits
+ * will have an acceptable exit node for exit stream <b>conn</b> if it
+ * is defined, else for "*:port".
  * Else return 0.
  */
 int
@@ -296,8 +305,9 @@ circuit_stream_is_being_handled(connection_t *conn, uint16_t port, int min)
     if (CIRCUIT_IS_ORIGIN(circ) &&
         !circ->marked_for_close &&
         circ->purpose == CIRCUIT_PURPOSE_C_GENERAL &&
+        !circ->build_state->is_internal &&
         (!circ->timestamp_dirty ||
-         circ->timestamp_dirty + get_options()->MaxCircuitDirtiness < now)) {
+         circ->timestamp_dirty + get_options()->MaxCircuitDirtiness > now)) {
       exitrouter = build_state_get_exit_router(circ->build_state);
       if (exitrouter &&
           (!need_uptime || circ->build_state->need_uptime)) {
@@ -320,7 +330,7 @@ circuit_stream_is_being_handled(connection_t *conn, uint16_t port, int min)
 }
 
 /** Don't keep more than 10 unused open circuits around. */
-#define MAX_UNUSED_OPEN_CIRCUITS 10
+#define MAX_UNUSED_OPEN_CIRCUITS 12
 
 /** Figure out how many circuits we have open that are clean. Make
  * sure it's enough for all the upcoming behaviors we predict we'll have.
@@ -362,7 +372,7 @@ circuit_predict_and_launch_new(void)
   if (!circuit_all_predicted_ports_handled(now, &port_needs_uptime,
                                            &port_needs_capacity)) {
     info(LD_CIRC,"Have %d clean circs (%d internal), need another exit circ.",
-      num, num_internal);
+         num, num_internal);
     circuit_launch_by_router(CIRCUIT_PURPOSE_C_GENERAL, NULL,
                              port_needs_uptime, port_needs_capacity, 0);
     return;
@@ -371,19 +381,19 @@ circuit_predict_and_launch_new(void)
   /* Third, see if we need any more hidden service (server) circuits. */
   if (num_rend_services() && num_uptime_internal < 3) {
     info(LD_CIRC,"Have %d clean circs (%d internal), need another internal circ for my hidden service.",
-           num, num_internal);
+         num, num_internal);
     circuit_launch_by_router(CIRCUIT_PURPOSE_C_GENERAL, NULL,
                              1, 1, 1);
     return;
   }
 
   /* Fourth, see if we need any more hidden service (client) circuits. */
-  if (rep_hist_get_predicted_hidserv(now, &hidserv_needs_uptime,
-                                     &hidserv_needs_capacity) &&
+  if (rep_hist_get_predicted_internal(now, &hidserv_needs_uptime,
+                                      &hidserv_needs_capacity) &&
       ((num_uptime_internal<2 && hidserv_needs_uptime) ||
         num_internal<2)) {
     info(LD_CIRC,"Have %d clean circs (%d uptime-internal, %d internal),"
-      " need another hidserv circ.", num, num_uptime_internal, num_internal);
+         " need another hidserv circ.", num, num_uptime_internal, num_internal);
     circuit_launch_by_router(CIRCUIT_PURPOSE_C_GENERAL, NULL,
                              hidserv_needs_uptime, hidserv_needs_capacity, 1);
     return;
@@ -503,26 +513,17 @@ circuit_about_to_close_connection(connection_t *conn)
   /* currently, we assume it's too late to flush conn's buf here.
    * down the road, maybe we'll consider that eof doesn't mean can't-write
    */
-  circuit_t *circ;
-
   switch (conn->type) {
-    case CONN_TYPE_OR:
+    case CONN_TYPE_OR: {
       /* Inform any pending (not attached) circs that they should give up. */
       circuit_n_conn_done(conn, 0);
       /* Now close all the attached circuits on it. */
-      while ((circ = circuit_get_by_conn(conn))) {
-        if (circ->n_conn == conn)
-          /* it's closing in front of us */
-          circuit_set_circid_orconn(circ, 0, NULL, P_CONN_CHANGED);
-        if (circ->p_conn == conn)
-          /* it's closing behind us */
-          circuit_set_circid_orconn(circ, 0, NULL, N_CONN_CHANGED);
-        circuit_mark_for_close(circ);
-      }
+      circuit_unlink_all_from_or_conn(conn);
       return;
+    }
     case CONN_TYPE_AP:
-    case CONN_TYPE_EXIT:
-
+    case CONN_TYPE_EXIT: {
+      circuit_t *circ;
       /* It's an edge conn. Need to remove it from the linked list of
        * conn's for this circuit. Confirm that 'end' relay command has
        * been sent. But don't kill the circuit.
@@ -533,7 +534,7 @@ circuit_about_to_close_connection(connection_t *conn)
         return;
 
       circuit_detach_stream(circ, conn);
-
+    }
   } /* end switch */
 }
 
@@ -568,7 +569,7 @@ circuit_expire_old_circuits(void)
 #define CIRCUIT_UNUSED_CIRC_TIMEOUT 3600 /* an hour */
       if (circ->timestamp_created + CIRCUIT_UNUSED_CIRC_TIMEOUT < now) {
         debug(LD_CIRC,"Closing circuit that has been unused for %d seconds.",
-               (int)(now - circ->timestamp_created));
+              (int)(now - circ->timestamp_created));
         circuit_mark_for_close(circ);
       }
     }
@@ -654,7 +655,17 @@ circuit_build_failed(circuit_t *circ)
   if (circ->cpath &&
       circ->cpath->prev->state != CPATH_STATE_OPEN &&
       circ->cpath->prev->prev->state == CPATH_STATE_OPEN) {
-      failed_at_last_hop = 1;
+    failed_at_last_hop = 1;
+  }
+  if (circ->cpath &&
+      circ->cpath->state != CPATH_STATE_OPEN) {
+    /* We failed at the first hop. If there's an OR connection
+       to blame, blame it. */
+    if (circ->n_conn) {
+      info(LD_OR, "Our circuit failed to get a response from the first hop (%s:%d). I'm going to try to rotate to a better connection.",
+           circ->n_conn->address, circ->n_conn->port);
+      circ->n_conn->is_obsolete = 1;
+    }
   }
 
   switch (circ->purpose) {
@@ -760,7 +771,7 @@ circuit_launch_by_extend_info(uint8_t purpose, extend_info_t *extend_info,
                                        need_uptime, need_capacity, internal);
     if (circ) {
       info(LD_CIRC,"Cannibalizing circ '%s' for purpose %d",
-             build_state_get_exit_nickname(circ->build_state), purpose);
+           build_state_get_exit_nickname(circ->build_state), purpose);
       circ->purpose = purpose;
       /* reset the birth date of this circ, else expire_building
        * will see it and think it's been trying to build since it
@@ -857,14 +868,19 @@ circuit_get_open_circ_or_launch(connection_t *conn,
 {
   circuit_t *circ;
   int is_resolve;
-  int need_uptime;
+  int need_uptime, need_internal;
 
   tor_assert(conn);
   tor_assert(circp);
   tor_assert(conn->state == AP_CONN_STATE_CIRCUIT_WAIT);
   is_resolve = conn->socks_request->command == SOCKS_COMMAND_RESOLVE;
 
-  circ = circuit_get_best(conn, 1, desired_circuit_purpose);
+  need_uptime = smartlist_string_num_isin(get_options()->LongLivedPorts,
+                                          conn->socks_request->port);
+  need_internal = desired_circuit_purpose != CIRCUIT_PURPOSE_C_GENERAL;
+
+  circ = circuit_get_best(conn, 1, desired_circuit_purpose,
+                          need_uptime, need_internal);
 
   if (circ) {
     *circp = circ;
@@ -888,9 +904,6 @@ circuit_get_open_circ_or_launch(connection_t *conn,
     return 0;
   }
 
-  need_uptime = smartlist_string_num_isin(get_options()->LongLivedPorts,
-                                          conn->socks_request->port);
-
   /* Do we need to check exit policy? */
   if (!is_resolve && !connection_edge_is_rendezvous_stream(conn)) {
     struct in_addr in;
@@ -899,19 +912,18 @@ circuit_get_open_circ_or_launch(connection_t *conn,
       addr = ntohl(in.s_addr);
     if (router_exit_policy_all_routers_reject(addr, conn->socks_request->port,
                                               need_uptime)) {
-      /* LD_GENERAL? LD_APP? ???? NM */
-      notice(LD_CIRC,"No Tor server exists that allows exit to %s:%d. Rejecting.",
+      notice(LD_APP,"No Tor server exists that allows exit to %s:%d. Rejecting.",
              safe_str(conn->socks_request->address), conn->socks_request->port);
       return -1;
     }
   }
 
   /* is one already on the way? */
-  circ = circuit_get_best(conn, 0, desired_circuit_purpose);
+  circ = circuit_get_best(conn, 0, desired_circuit_purpose,
+                          need_uptime, need_internal);
   if (!circ) {
     extend_info_t *extend_info=NULL;
     uint8_t new_circ_purpose;
-    int is_internal;
 
     if (desired_circuit_purpose == CIRCUIT_PURPOSE_C_INTRODUCE_ACK_WAIT) {
       /* need to pick an intro point */
@@ -923,7 +935,7 @@ circuit_get_open_circ_or_launch(connection_t *conn,
         conn->state = AP_CONN_STATE_RENDDESC_WAIT;
         return 0;
       }
-      info(LD_REND,"Chose %s as intro point for %s.",
+      info(LD_REND,"Chose '%s' as intro point for '%s'.",
            extend_info->nickname, safe_str(conn->rend_query));
     }
 
@@ -950,15 +962,14 @@ circuit_get_open_circ_or_launch(connection_t *conn,
     else
       new_circ_purpose = desired_circuit_purpose;
 
-    is_internal = (new_circ_purpose != CIRCUIT_PURPOSE_C_GENERAL || is_resolve);
     circ = circuit_launch_by_extend_info(
-              new_circ_purpose, extend_info, need_uptime, 1, is_internal);
+              new_circ_purpose, extend_info, need_uptime, 1, need_internal);
     if (extend_info)
       extend_info_free(extend_info);
 
     if (desired_circuit_purpose != CIRCUIT_PURPOSE_C_GENERAL) {
       /* help predict this next time */
-      rep_hist_note_used_hidserv(time(NULL), need_uptime, 1);
+      rep_hist_note_used_internal(time(NULL), need_uptime, 1);
       if (circ) {
         /* write the service_id into circ */
         strlcpy(circ->rend_query, conn->rend_query, sizeof(circ->rend_query));
@@ -999,7 +1010,7 @@ link_apconn_to_circ(connection_t *apconn, circuit_t *circ)
 }
 
 /** If an exit wasn't specifically chosen, save the history for future
- * use */
+ * use. */
 static void
 consider_recording_trackhost(connection_t *conn, circuit_t *circ)
 {
@@ -1008,6 +1019,7 @@ consider_recording_trackhost(connection_t *conn, circuit_t *circ)
   or_options_t *options = get_options();
   size_t len;
   char *new_address;
+  char fp[HEX_DIGEST_LEN+1];
 
   /* Search the addressmap for this conn's destination. */
   /* If he's not in the address map.. */
@@ -1032,15 +1044,19 @@ consider_recording_trackhost(connection_t *conn, circuit_t *circ)
   if (!found_needle || !circ->build_state->chosen_exit)
     return;
 
+  /* write down the fingerprint of the chosen exit, not the nickname,
+   * because the chosen exit might not be verified. */
+  base16_encode(fp, sizeof(fp),
+                circ->build_state->chosen_exit->identity_digest, DIGEST_LEN);
+
   /* Add this exit/hostname pair to the addressmap. */
   len = strlen(conn->socks_request->address) + 1 /* '.' */ +
-        strlen(circ->build_state->chosen_exit->nickname) + 1 /* '.' */ +
+        strlen(fp) + 1 /* '.' */ +
         strlen("exit") + 1 /* '\0' */;
   new_address = tor_malloc(len);
 
   tor_snprintf(new_address, len, "%s.%s.exit",
-               conn->socks_request->address,
-               circ->build_state->chosen_exit->nickname);
+               conn->socks_request->address, fp);
 
   addressmap_register(conn->socks_request->address, new_address,
                       time(NULL) + options->TrackHostExitsExpire);
@@ -1113,12 +1129,12 @@ connection_ap_handshake_attach_circuit(connection_t *conn)
       routerinfo_t *router = router_get_by_nickname(conn->chosen_exit_name, 1);
       if (!router) {
         warn(LD_APP,"Requested exit point '%s' is not known. Closing.",
-               conn->chosen_exit_name);
+             conn->chosen_exit_name);
         return -1;
       }
       if (!connection_ap_can_use_exit(conn, router)) {
         warn(LD_APP, "Requested exit point '%s' would refuse request. Closing.",
-               conn->chosen_exit_name);
+             conn->chosen_exit_name);
         return -1;
       }
     }
@@ -1129,7 +1145,7 @@ connection_ap_handshake_attach_circuit(connection_t *conn)
       return retval;
 
     debug(LD_APP|LD_CIRC,"Attaching apconn to circ %d (stream %d sec old).",
-           circ->n_circ_id, conn_age);
+          circ->n_circ_id, conn_age);
     /* here, print the circ's path. so people can figure out which circs are sucking. */
     circuit_log_path(LOG_INFO,LD_APP|LD_CIRC,circ);
 
@@ -1165,7 +1181,10 @@ connection_ap_handshake_attach_circuit(connection_t *conn)
     }
 
     if (rendcirc && rendcirc->purpose == CIRCUIT_PURPOSE_C_REND_READY_INTRO_ACKED) {
-      info(LD_REND,"pending-join circ %d already here, with intro ack. Stalling. (stream %d sec old)", rendcirc->n_circ_id, conn_age);
+      info(LD_REND,
+           "pending-join circ %d already here, with intro ack. "
+           "Stalling. (stream %d sec old)",
+            rendcirc->n_circ_id, conn_age);
       return 0;
     }
 
@@ -1186,12 +1205,12 @@ connection_ap_handshake_attach_circuit(connection_t *conn)
 
     if (rendcirc && introcirc && rendcirc->purpose == CIRCUIT_PURPOSE_C_REND_READY) {
       info(LD_REND,"ready rend circ %d already here (no intro-ack yet on intro %d). (stream %d sec old)",
-             rendcirc->n_circ_id, introcirc->n_circ_id, conn_age);
+           rendcirc->n_circ_id, introcirc->n_circ_id, conn_age);
 
       tor_assert(introcirc->purpose == CIRCUIT_PURPOSE_C_INTRODUCING);
       if (introcirc->state == CIRCUIT_STATE_OPEN) {
         info(LD_REND,"found open intro circ %d (rend %d); sending introduction. (stream %d sec old)",
-               introcirc->n_circ_id, rendcirc->n_circ_id, conn_age);
+             introcirc->n_circ_id, rendcirc->n_circ_id, conn_age);
         if (rend_client_send_introduction(introcirc, rendcirc) < 0) {
           return -1;
         }
