@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: command.c,v 1.99 2005/10/29 19:13:48 arma Exp $ */
-const char command_c_id[] = "$Id: command.c,v 1.99 2005/10/29 19:13:48 arma Exp $";
+/* $Id: command.c,v 1.104 2005/12/10 09:36:25 arma Exp $ */
+const char command_c_id[] = "$Id: command.c,v 1.104 2005/12/10 09:36:25 arma Exp $";
 
 /**
  * \file command.c
@@ -82,10 +82,10 @@ command_process_cell(cell_t *cell, connection_t *conn)
   if (now > current_second) { /* the second has rolled over */
     /* print stats */
     info(LD_OR,"At end of second: %d creates (%d ms), %d createds (%d ms), %d relays (%d ms), %d destroys (%d ms)",
-      num_create, create_time/1000,
-      num_created, created_time/1000,
-      num_relay, relay_time/1000,
-      num_destroy, destroy_time/1000);
+         num_create, create_time/1000,
+         num_created, created_time/1000,
+         num_relay, relay_time/1000,
+         num_destroy, destroy_time/1000);
 
     /* zero out stats */
     num_create = num_created = num_relay = num_destroy = 0;
@@ -174,11 +174,11 @@ command_process_create_cell(cell_t *cell, connection_t *conn)
    */
   if ((cell->circ_id & (1<<15)) && conn->circ_id_type == CIRC_ID_TYPE_HIGHER) {
     info(LD_OR, "Got a high circuit ID from %s (%d); switching to low circuit IDs.",
-           conn->nickname ? conn->nickname : "client", conn->s);
+         conn->nickname ? conn->nickname : "client", conn->s);
     conn->circ_id_type = CIRC_ID_TYPE_LOWER;
   } else if (!(cell->circ_id & (1<<15)) && conn->circ_id_type == CIRC_ID_TYPE_LOWER) {
     info(LD_OR, "Got a low circuit ID from %s (%d); switching to high circuit IDs.",
-           conn->nickname ? conn->nickname : "client", conn->s);
+         conn->nickname ? conn->nickname : "client", conn->s);
     conn->circ_id_type = CIRC_ID_TYPE_HIGHER;
   }
 
@@ -198,7 +198,7 @@ command_process_create_cell(cell_t *cell, connection_t *conn)
 
   circ = circuit_new(cell->circ_id, conn);
   circ->purpose = CIRCUIT_PURPOSE_OR;
-  circ->state = CIRCUIT_STATE_ONIONSKIN_PENDING;
+  circuit_set_state(circ, CIRCUIT_STATE_ONIONSKIN_PENDING);
   if (cell->command == CELL_CREATE) {
     circ->onionskin = tor_malloc(ONIONSKIN_CHALLENGE_LEN);
     memcpy(circ->onionskin, cell->payload, ONIONSKIN_CHALLENGE_LEN);
@@ -211,6 +211,8 @@ command_process_create_cell(cell_t *cell, connection_t *conn)
     }
     debug(LD_OR,"success: handed off onionskin.");
   } else {
+    /* This is a CREATE_FAST cell; we can handle it immediately without using
+     * a CPU worker.*/
     char keys[CPATH_KEY_MATERIAL_LEN];
     char reply[DIGEST_LEN*2];
     tor_assert(cell->command == CELL_CREATE_FAST);
@@ -227,7 +229,8 @@ command_process_create_cell(cell_t *cell, connection_t *conn)
   }
 }
 
-/** Process a 'created' <b>cell</b> that just arrived from <b>conn</b>. Find the circuit
+/** Process a 'created' <b>cell</b> that just arrived from <b>conn</b>.
+ * Find the circuit
  * that it's intended for. If we're not the origin of the circuit, package
  * the 'created' cell in an 'extended' relay cell and pass it back. If we
  * are the origin of the circuit, send it to circuit_finish_handshake() to
@@ -333,14 +336,10 @@ command_process_destroy_cell(cell_t *cell, connection_t *conn)
 
   if (!circ) {
     info(LD_OR,"unknown circuit %d on connection from %s:%d. Dropping.",
-           cell->circ_id, conn->address, conn->port);
+         cell->circ_id, conn->address, conn->port);
     return;
   }
-
   debug(LD_OR,"Received for circID %d.",cell->circ_id);
-  if (circ->state == CIRCUIT_STATE_ONIONSKIN_PENDING) {
-    onion_pending_remove(circ);
-  }
 
   if (cell->circ_id == circ->p_circ_id) {
     /* the destroy came from behind */
