@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection.c,v 1.417 2005/11/11 17:03:35 arma Exp $ */
-const char connection_c_id[] = "$Id: connection.c,v 1.417 2005/11/11 17:03:35 arma Exp $";
+/* $Id: connection.c,v 1.426 2005/12/10 09:36:25 arma Exp $ */
+const char connection_c_id[] = "$Id: connection.c,v 1.426 2005/12/10 09:36:25 arma Exp $";
 
 /**
  * \file connection.c
@@ -237,6 +237,11 @@ _connection_free(connection_t *conn)
     tor_close_socket(conn->s);
   }
 
+  if (conn->type == CONN_TYPE_OR && !tor_digest_is_zero(conn->identity_digest)) {
+    warn(LD_BUG, "called on OR conn with non-zeroed idenity_digest");
+    connection_or_remove_from_identity_map(conn);
+  }
+
   memset(conn, 0xAA, sizeof(connection_t)); /* poison memory */
   tor_free(conn);
 }
@@ -252,6 +257,9 @@ connection_free(connection_t *conn)
   if (connection_speaks_cells(conn)) {
     if (conn->state == OR_CONN_STATE_OPEN)
       directory_set_dirty();
+    if (!tor_digest_is_zero(conn->identity_digest)) {
+      connection_or_remove_from_identity_map(conn);
+    }
   }
   if (conn->type == CONN_TYPE_CONTROL) {
     conn->event_mask = 0;
@@ -283,6 +291,9 @@ connection_free_all(void)
       carray[i]->event_mask = 0;
   control_update_global_event_mask();
 
+  /* Unlink everything from the identity map. */
+  connection_or_clear_identity_map();
+
   for (i=0;i<n;i++)
     _connection_free(carray[i]);
 
@@ -311,7 +322,9 @@ connection_about_to_close_connection(connection_t *conn)
 
   if (CONN_IS_EDGE(conn)) {
     if (!conn->has_sent_end) {
-      warn(LD_BUG,"Harmless bug: Edge connection (marked at %s:%d) hasn't sent end yet?", conn->marked_for_close_file, conn->marked_for_close);
+      warn(LD_BUG,
+           "Harmless bug: Edge connection (marked at %s:%d) hasn't sent end yet?",
+           conn->marked_for_close_file, conn->marked_for_close);
       tor_fragile_assert();
     }
   }
@@ -356,7 +369,7 @@ connection_about_to_close_connection(connection_t *conn)
         /* since conn gets removed right after this function finishes,
          * there's no point trying to send back a reply at this point. */
         warn(LD_BUG,"Bug: Closing stream (marked at %s:%d) without sending back a socks reply.",
-               conn->marked_for_close_file, conn->marked_for_close);
+             conn->marked_for_close_file, conn->marked_for_close);
       } else {
         control_event_stream_status(conn, STREAM_EVENT_CLOSED);
       }
@@ -392,9 +405,9 @@ connection_close_immediate(connection_t *conn)
   }
   if (conn->outbuf_flushlen) {
     info(LD_NET,"fd %d, type %s, state %s, %d bytes on outbuf.",
-           conn->s, conn_type_to_string(conn->type),
-           conn_state_to_string(conn->type, conn->state),
-           (int)conn->outbuf_flushlen);
+         conn->s, conn_type_to_string(conn->type),
+         conn_state_to_string(conn->type, conn->state),
+         (int)conn->outbuf_flushlen);
   }
 
   connection_unregister(conn);
@@ -529,13 +542,13 @@ connection_create_listener(const char *listenaddress, uint16_t listenport,
 
   if (bind(s,(struct sockaddr *)&listenaddr,sizeof(listenaddr)) < 0) {
     warn(LD_NET, "Could not bind to %s:%u: %s", address, usePort,
-           tor_socket_strerror(tor_socket_errno(s)));
+         tor_socket_strerror(tor_socket_errno(s)));
     goto err;
   }
 
   if (listen(s,SOMAXCONN) < 0) {
     warn(LD_NET, "Could not listen on %s:%u: %s", address, usePort,
-           tor_socket_strerror(tor_socket_errno(s)));
+         tor_socket_strerror(tor_socket_errno(s)));
     goto err;
   }
 
@@ -627,7 +640,7 @@ connection_handle_listener_read(connection_t *conn, int new_type)
     }
     /* else there was a real error. */
     warn(LD_NET,"accept() failed: %s. Closing listener.",
-           tor_socket_strerror(e));
+         tor_socket_strerror(e));
     connection_mark_for_close(conn);
     return -1;
   }
@@ -737,12 +750,12 @@ connection_connect(connection_t *conn, char *address,
   s = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
   if (s < 0) {
     warn(LD_NET,"Error creating network socket: %s",
-           tor_socket_strerror(tor_socket_errno(-1)));
+         tor_socket_strerror(tor_socket_errno(-1)));
     return -1;
   } else if (!SOCKET_IS_POLLABLE(s)) {
     warn(LD_NET,
-      "Too many connections; can't create pollable connection to %s",
-      safe_str(address));
+         "Too many connections; can't create pollable connection to %s",
+         safe_str(address));
     tor_close_socket(s);
     return -1;
   }
@@ -755,11 +768,11 @@ connection_connect(connection_t *conn, char *address,
     ext_addr.sin_port = 0;
     if (!tor_inet_aton(options->OutboundBindAddress, &ext_addr.sin_addr)) {
       warn(LD_CONFIG,"Outbound bind address '%s' didn't parse. Ignoring.",
-             options->OutboundBindAddress);
+           options->OutboundBindAddress);
     } else {
       if (bind(s, (struct sockaddr*)&ext_addr, sizeof(ext_addr)) < 0) {
         warn(LD_NET,"Error binding network socket: %s",
-               tor_socket_strerror(tor_socket_errno(s)));
+             tor_socket_strerror(tor_socket_errno(s)));
         return -1;
       }
     }
@@ -779,7 +792,7 @@ connection_connect(connection_t *conn, char *address,
     if (!ERRNO_IS_CONN_EINPROGRESS(e)) {
       /* yuck. kill it. */
       info(LD_NET,"connect() to %s:%u failed: %s",safe_str(address),port,
-             tor_socket_strerror(e));
+           tor_socket_strerror(e));
       tor_close_socket(s);
       return -1;
     } else {
@@ -889,8 +902,8 @@ retry_listeners(int type, config_line_t *cfg,
       }
     } else {
       /* It's configured; we don't need to launch it. */
-      debug(LD_NET, "Already have %s on %s:%d",
-             conn_type_to_string(type), conn->address, conn->port);
+//      debug(LD_NET, "Already have %s on %s:%d",
+//             conn_type_to_string(type), conn->address, conn->port);
       smartlist_remove(launch, line);
       if (free_launch_elts)
         config_free_lines(line);
@@ -1221,7 +1234,7 @@ connection_read_to_buf(connection_t *conn, int *max_to_read)
     }
 
     debug(LD_NET,"%d: starting, inbuf_datalen %d (%d pending in tls object). at_most %d.",
-           conn->s,(int)buf_datalen(conn->inbuf),tor_tls_get_pending_bytes(conn->tls), at_most);
+          conn->s,(int)buf_datalen(conn->inbuf),tor_tls_get_pending_bytes(conn->tls), at_most);
 
     /* else open, or closing */
     result = read_to_buf_tls(conn->tls, at_most, conn->inbuf);
@@ -1229,11 +1242,11 @@ connection_read_to_buf(connection_t *conn, int *max_to_read)
     switch (result) {
       case TOR_TLS_CLOSE:
         info(LD_NET,"TLS connection closed on read. Closing. (Nickname %s, address %s",
-               conn->nickname ? conn->nickname : "not set", conn->address);
+             conn->nickname ? conn->nickname : "not set", conn->address);
         return -1;
       case TOR_TLS_ERROR:
         info(LD_NET,"tls error. breaking (nickname %s, address %s).",
-               conn->nickname ? conn->nickname : "not set", conn->address);
+             conn->nickname ? conn->nickname : "not set", conn->address);
         return -1;
       case TOR_TLS_WANTWRITE:
         connection_start_writing(conn);
@@ -1402,7 +1415,7 @@ connection_handle_write(connection_t *conn)
       case TOR_TLS_ERROR:
       case TOR_TLS_CLOSE:
         info(LD_NET,result==TOR_TLS_ERROR?
-               "tls error. breaking.":"TLS connection closed on flush");
+             "tls error. breaking.":"TLS connection closed on flush");
         connection_close_immediate(conn); /* Don't flush; connection is dead. */
         connection_mark_for_close(conn);
         return -1;
@@ -1547,8 +1560,8 @@ connection_or_exact_get_by_addr_port(uint32_t addr, uint16_t port)
   return best;
 }
 
-/** Return a connection with give type, address, port, and purpose or NULL if
- * no such connection exists. */
+/** Return a connection with given type, address, port, and purpose;
+ * or NULL if no such connection exists. */
 connection_t *
 connection_get_by_type_addr_port_purpose(int type, uint32_t addr, uint16_t port,
                                          int purpose)
@@ -1570,26 +1583,6 @@ connection_get_by_type_addr_port_purpose(int type, uint32_t addr, uint16_t port,
   return NULL;
 }
 
-connection_t *
-connection_get_by_identity_digest(const char *digest, int type)
-{
-  int i, n;
-  connection_t *conn, *best=NULL;
-  connection_t **carray;
-
-  get_connection_array(&carray,&n);
-  for (i=0;i<n;i++) {
-    conn = carray[i];
-    if (conn->type != type)
-      continue;
-    if (!memcmp(conn->identity_digest, digest, DIGEST_LEN) &&
-        !conn->marked_for_close &&
-        (!best || best->timestamp_created < conn->timestamp_created))
-      best = conn;
-  }
-  return best;
-}
-
 /** Return the connection with id <b>id</b> if it is not already
  * marked for close.
  */
