@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_edge.c,v 1.361 2005/11/14 04:13:43 arma Exp $ */
-const char connection_edge_c_id[] = "$Id: connection_edge.c,v 1.361 2005/11/14 04:13:43 arma Exp $";
+/* $Id: connection_edge.c,v 1.375 2005/12/10 09:36:25 arma Exp $ */
+const char connection_edge_c_id[] = "$Id: connection_edge.c,v 1.375 2005/12/10 09:36:25 arma Exp $";
 
 /**
  * \file connection_edge.c
@@ -11,7 +11,6 @@ const char connection_edge_c_id[] = "$Id: connection_edge.c,v 1.361 2005/11/14 0
  **/
 
 #include "or.h"
-#include "tree.h"
 
 static addr_policy_t *socks_policy = NULL;
 /* List of exit_redirect_t */
@@ -127,7 +126,7 @@ connection_edge_process_inbuf(connection_t *conn, int package_partial)
     case AP_CONN_STATE_RESOLVE_WAIT:
     case AP_CONN_STATE_CONTROLLER_WAIT:
       info(LD_EDGE,"data from edge while in '%s' state. Leaving it on buffer.",
-             conn_state_to_string(conn->type, conn->state));
+           conn_state_to_string(conn->type, conn->state));
       return 0;
   }
   warn(LD_BUG,"Bug: Got unexpected state %d. Closing.",conn->state);
@@ -184,7 +183,7 @@ connection_edge_end(connection_t *conn, char reason, crypt_path_t *cpath_layer)
 
   if (conn->marked_for_close) {
     warn(LD_BUG,"Bug: called on conn that's already marked for close at %s:%d.",
-           conn->marked_for_close_file, conn->marked_for_close);
+         conn->marked_for_close_file, conn->marked_for_close);
     return 0;
   }
 
@@ -275,7 +274,7 @@ connection_edge_finished_connecting(connection_t *conn)
   in.s_addr = htonl(conn->addr);
   tor_inet_ntoa(&in,valbuf,sizeof(valbuf));
   info(LD_EXIT,"Exit connection to %s:%u (%s) established.",
-         safe_str(conn->address),conn->port,safe_str(valbuf));
+       safe_str(conn->address),conn->port,safe_str(valbuf));
 
   conn->state = EXIT_CONN_STATE_OPEN;
   connection_watch_events(conn, EV_READ); /* stop writing, continue reading */
@@ -315,6 +314,7 @@ connection_ap_expire_beginning(void)
   connection_t **carray;
   connection_t *conn;
   circuit_t *circ;
+  const char *nickname;
   int n, i;
   time_t now = time(NULL);
   or_options_t *options = get_options();
@@ -341,13 +341,13 @@ connection_ap_expire_beginning(void)
     circ = circuit_get_by_edge_conn(conn);
     if (!circ) { /* it's vanished? */
       info(LD_APP,"Conn is waiting (address %s), but lost its circ.",
-             safe_str(conn->socks_request->address));
+           safe_str(conn->socks_request->address));
       connection_mark_unattached_ap(conn, END_STREAM_REASON_TIMEOUT);
       continue;
     }
     if (circ->purpose == CIRCUIT_PURPOSE_C_REND_JOINED) {
       if (now - conn->timestamp_lastread > 45) {
-        notice(LD_REND,"Rend stream is %d seconds late. Giving up on address '%s'.",
+        notice(LD_REND,"Rend stream is %d seconds late. Giving up on address '%s.onion'.",
                (int)(now - conn->timestamp_lastread),
                safe_str(conn->socks_request->address));
         connection_edge_end(conn, END_STREAM_REASON_TIMEOUT, conn->cpath_layer);
@@ -356,9 +356,11 @@ connection_ap_expire_beginning(void)
       continue;
     }
     tor_assert(circ->purpose == CIRCUIT_PURPOSE_C_GENERAL);
-    notice(LD_APP,"We tried for %d seconds to connect to '%s'. Retrying on a new circuit.",
+    nickname = build_state_get_exit_nickname(circ->build_state);
+    notice(LD_APP,"We tried for %d seconds to connect to '%s' using exit '%s'. Retrying on a new circuit.",
            (int)(now - conn->timestamp_lastread),
-           safe_str(conn->socks_request->address));
+           safe_str(conn->socks_request->address),
+           nickname ? nickname : "*unnamed*");
     /* send an end down the circuit */
     connection_edge_end(conn, END_STREAM_REASON_TIMEOUT, conn->cpath_layer);
     /* un-mark it as ending, since we're going to reuse it */
@@ -425,8 +427,8 @@ connection_ap_detach_retriable(connection_t *conn, circuit_t *circ)
 }
 
 /** A client-side struct to remember requests to rewrite addresses
- * to new addresses. These structs make up a tree, with addressmap
- * below as its root.
+ * to new addresses. These structs are stored in the hash table
+ * "addressmap" below.
  *
  * There are 5 ways to set an address mapping:
  * - A MapAddress command from the controller [permanent]
@@ -456,10 +458,10 @@ typedef struct {
   char *hostname_address;
 } virtaddress_entry_t;
 
-/** The tree of client-side address rewrite instructions. */
+/** A hash table to store client-side address rewrite instructions. */
 static strmap_t *addressmap=NULL;
 /**
- * Tree mapping addresses to which virtual address, if any, we
+ * Table mapping addresses to which virtual address, if any, we
  * assigned them to.
  *
  * We maintain the following invariant: if [A,B] is in
@@ -586,7 +588,7 @@ addressmap_rewrite(char *address, size_t maxlen)
     strlcpy(address, ent->new_address, maxlen);
   }
   warn(LD_CONFIG,"Loop detected: we've rewritten '%s' 16 times! Using it as-is.",
-         safe_str(address));
+       safe_str(address));
   /* it's fine to rewrite a rewrite, but don't loop forever */
 }
 
@@ -666,7 +668,7 @@ client_dns_incr_failures(const char *address)
   }
   ++ent->num_resolve_failures;
   info(LD_APP, "Address %s now has %d resolve failures.",
-         safe_str(address), ent->num_resolve_failures);
+       safe_str(address), ent->num_resolve_failures);
   return ent->num_resolve_failures;
 }
 
@@ -890,8 +892,8 @@ addressmap_get_mappings(smartlist_t *sl, time_t min_expires, time_t max_expires)
      val = _val;
      if (val->expires >= min_expires && val->expires <= max_expires) {
        if (!sl) {
-         addressmap_ent_remove(key, val);
          iter = strmap_iter_next_rmv(addressmap,iter);
+         addressmap_ent_remove(key, val);
          continue;
        } else if (val->new_address) {
          size_t len = strlen(key)+strlen(val->new_address)+2;
@@ -923,7 +925,8 @@ connection_ap_handshake_process_socks(connection_t *conn)
   socks_request_t *socks;
   int sockshere;
   hostname_type_t addresstype;
-  int tor_should_handle_stream = !get_options()->LeaveStreamsUnattached;
+  or_options_t *options = get_options();
+  int tor_should_handle_stream = !options->LeaveStreamsUnattached;
 
   tor_assert(conn);
   tor_assert(conn->type == CONN_TYPE_AP);
@@ -933,7 +936,7 @@ connection_ap_handshake_process_socks(connection_t *conn)
 
   debug(LD_APP,"entered.");
 
-  sockshere = fetch_from_buf_socks(conn->inbuf, socks);
+  sockshere = fetch_from_buf_socks(conn->inbuf, socks, options->TestSocks);
   if (sockshere == 0) {
     if (socks->replylen) {
       connection_write_to_buf(socks->reply, socks->replylen, conn);
@@ -957,7 +960,7 @@ connection_ap_handshake_process_socks(connection_t *conn)
 
   tor_strlower(socks->address); /* normalize it */
   debug(LD_APP,"Client asked for %s:%d", safe_str(socks->address),
-         socks->port);
+        socks->port);
 
   /* For address map controls, remap the address */
   addressmap_rewrite(socks->address, sizeof(socks->address));
@@ -970,7 +973,7 @@ connection_ap_handshake_process_socks(connection_t *conn)
      * information.
      */
     warn(LD_APP,"Missing mapping for virtual address '%s'. Refusing.",
-           socks->address); /* don't safe_str() this yet. */
+         socks->address); /* don't safe_str() this yet. */
     connection_mark_unattached_ap(conn, END_STREAM_REASON_INTERNAL);
     return -1;
   }
@@ -1002,9 +1005,11 @@ connection_ap_handshake_process_socks(connection_t *conn)
       }
     } else {
       struct in_addr in;
-      routerinfo_t *r = router_get_by_nickname(socks->address, 1);
+      routerinfo_t *r;
+      conn->chosen_exit_name = tor_strdup(socks->address);
+      r = router_get_by_nickname(conn->chosen_exit_name, 1);
+      *socks->address = 0;
       if (r) {
-        conn->chosen_exit_name = tor_strdup(socks->address);
         /* XXXX Should this use server->address instead? */
         in.s_addr = htonl(r->addr);
         strlcpy(socks->address, inet_ntoa(in), sizeof(socks->address));
@@ -1024,7 +1029,7 @@ connection_ap_handshake_process_socks(connection_t *conn)
     if (tor_should_handle_stream &&
         address_is_invalid_destination(socks->address)) {
       warn(LD_APP,"Destination '%s' seems to be an invalid hostname. Failing.",
-             safe_str(socks->address));
+           safe_str(socks->address));
       connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
       return -1;
     }
@@ -1061,7 +1066,7 @@ connection_ap_handshake_process_socks(connection_t *conn)
           router_find_exact_exit_enclave(socks->address, socks->port);
         if (r) {
           info(LD_APP,"Redirecting address %s to exit at enclave router %s",
-                 safe_str(socks->address), r->nickname);
+               safe_str(socks->address), r->nickname);
           /* use the hex digest, not nickname, in case there are two
              routers with this nickname */
           conn->chosen_exit_name =
@@ -1072,7 +1077,7 @@ connection_ap_handshake_process_socks(connection_t *conn)
       rep_hist_note_used_port(socks->port, time(NULL)); /* help predict this next time */
       control_event_stream_status(conn, STREAM_EVENT_NEW);
     }
-    if (get_options()->LeaveStreamsUnattached) {
+    if (!tor_should_handle_stream) {
       conn->state = AP_CONN_STATE_CONTROLLER_WAIT;
     } else {
       conn->state = AP_CONN_STATE_CIRCUIT_WAIT;
@@ -1098,23 +1103,21 @@ connection_ap_handshake_process_socks(connection_t *conn)
 
     strlcpy(conn->rend_query, socks->address, sizeof(conn->rend_query));
     info(LD_REND,"Got a hidden service request for ID '%s'",
-           safe_str(conn->rend_query));
+         safe_str(conn->rend_query));
     /* see if we already have it cached */
     r = rend_cache_lookup_entry(conn->rend_query, -1, &entry);
     if (r<0) {
       warn(LD_REND,"Invalid service descriptor %s",
-             safe_str(conn->rend_query));
+           safe_str(conn->rend_query));
       connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
       return -1;
     }
     if (r==0) {
       conn->state = AP_CONN_STATE_RENDDESC_WAIT;
       info(LD_REND, "Unknown descriptor %s. Fetching.",
-             safe_str(conn->rend_query));
+           safe_str(conn->rend_query));
       rend_client_refetch_renddesc(conn->rend_query);
-      return 0;
-    }
-    if (r>0) {
+    } else { /* r > 0 */
 #define NUM_SECONDS_BEFORE_REFETCH (60*15)
       if (time(NULL) - entry->received < NUM_SECONDS_BEFORE_REFETCH) {
         conn->state = AP_CONN_STATE_CIRCUIT_WAIT;
@@ -1123,15 +1126,15 @@ connection_ap_handshake_process_socks(connection_t *conn)
           connection_mark_unattached_ap(conn, END_STREAM_REASON_CANT_ATTACH);
           return -1;
         }
-        return 0;
       } else {
         conn->state = AP_CONN_STATE_RENDDESC_WAIT;
         info(LD_REND, "Stale descriptor %s. Refetching.",
              safe_str(conn->rend_query));
         rend_client_refetch_renddesc(conn->rend_query);
-        return 0;
       }
     }
+    control_event_stream_status(conn, STREAM_EVENT_NEW);
+    return 0;
   }
   return 0; /* unreached but keeps the compiler happy */
 }
@@ -1240,7 +1243,7 @@ connection_ap_handshake_send_resolve(connection_t *ap_conn, circuit_t *circ)
 
   ap_conn->state = AP_CONN_STATE_RESOLVE_WAIT;
   info(LD_APP,"Address sent for resolve, ap socket %d, n_circ_id %d",
-         ap_conn->s, circ->n_circ_id);
+       ap_conn->s, circ->n_circ_id);
   control_event_stream_status(ap_conn, STREAM_EVENT_SENT_RESOLVE);
   return 0;
 }
@@ -1262,7 +1265,7 @@ connection_ap_make_bridge(char *address, uint16_t port)
 
   if ((err = tor_socketpair(AF_UNIX, SOCK_STREAM, 0, fd)) < 0) {
     warn(LD_NET,"Couldn't construct socketpair (%s). Network down? Delaying.",
-        tor_socket_strerror(-err));
+         tor_socket_strerror(-err));
     return -1;
   }
 
@@ -1604,7 +1607,7 @@ connection_exit_connect(connection_t *conn)
   if (!connection_edge_is_rendezvous_stream(conn) &&
       router_compare_to_my_exit_policy(conn)) {
     info(LD_EXIT,"%s:%d failed exit policy. Closing.",
-           safe_str(conn->address), conn->port);
+         safe_str(conn->address), conn->port);
     connection_edge_end(conn, END_STREAM_REASON_EXITPOLICY, conn->cpath_layer);
     circuit_detach_stream(circuit_get_by_edge_conn(conn), conn);
     connection_free(conn);
@@ -1626,7 +1629,7 @@ connection_exit_connect(connection_t *conn)
           in.s_addr = htonl(addr);
           tor_inet_ntoa(&in, tmpbuf, sizeof(tmpbuf));
           debug(LD_EXIT, "Redirecting connection from %s:%d to %s:%d",
-                 safe_str(conn->address), conn->port, safe_str(tmpbuf), port);
+                safe_str(conn->address), conn->port, safe_str(tmpbuf), port);
         }
         break;
       }
@@ -1713,7 +1716,7 @@ connection_ap_can_use_exit(connection_t *conn, routerinfo_t *exit)
     if (router_get_by_nickname(conn->chosen_exit_name, 1) != exit) {
       /* doesn't match */
       debug(LD_APP,"Requested node '%s', considering node '%s'. No.",
-             conn->chosen_exit_name, exit->nickname);
+            conn->chosen_exit_name, exit->nickname);
       return 0;
     }
   }
