@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_or.c,v 1.196 2005/11/05 20:15:26 nickm Exp $ */
-const char connection_or_c_id[] = "$Id: connection_or.c,v 1.196 2005/11/05 20:15:26 nickm Exp $";
+/* $Id: connection_or.c,v 1.201 2005/12/10 09:36:25 arma Exp $ */
+const char connection_or_c_id[] = "$Id: connection_or.c,v 1.201 2005/12/10 09:36:25 arma Exp $";
 
 /**
  * \file connection_or.c
@@ -22,6 +22,92 @@ static int connection_or_process_cells_from_inbuf(connection_t *conn);
 
 /**************************************************************/
 
+/** Map from identity digest of connected OR or desired OR to a connection_t
+ * with that identity digest.  If there is more than one such connection_t,
+ * they form a linked list, with next_with_same_id as the next pointer.*/
+static digestmap_t *orconn_identity_map = NULL;
+
+/** If conn is listed in orconn_identity_map, remove it, and clear
+ * conn->identity_digest. */
+void
+connection_or_remove_from_identity_map(connection_t *conn)
+{
+  connection_t *tmp;
+  tor_assert(conn);
+  tor_assert(conn->type == CONN_TYPE_OR);
+  if (!orconn_identity_map)
+    return;
+  tmp = digestmap_get(orconn_identity_map, conn->identity_digest);
+  if (!tmp)
+    return;
+  if (conn == tmp) {
+    if (conn->next_with_same_id)
+      digestmap_set(orconn_identity_map, conn->identity_digest,
+                    conn->next_with_same_id);
+    else
+      digestmap_remove(orconn_identity_map, conn->identity_digest);
+  } else {
+    while (tmp->next_with_same_id) {
+      if (tmp->next_with_same_id == conn) {
+        tmp->next_with_same_id = conn->next_with_same_id;
+        break;
+      }
+      tmp = tmp->next_with_same_id;
+    }
+  }
+  memset(conn->identity_digest, 0, DIGEST_LEN);
+  conn->next_with_same_id = NULL;
+}
+
+/** Remove all entries from the identity-to-orconn map, and clear
+ * all identities in OR conns.*/
+void
+connection_or_clear_identity_map(void)
+{
+  int i, n;
+  connection_t **carray;
+
+  get_connection_array(&carray,&n);
+  for (i = 0; i < n; ++i) {
+    connection_t* conn = carray[i];
+    if (conn->type == CONN_TYPE_OR) {
+      memset(conn->identity_digest, 0, DIGEST_LEN);
+      conn->next_with_same_id = NULL;
+    }
+  }
+
+  digestmap_free(orconn_identity_map, NULL);
+  orconn_identity_map = NULL;
+}
+
+/** Change conn->identity_digest to digest, and add conn into
+ * orconn_digest_map. */
+static void
+connection_or_set_identity_digest(connection_t *conn, const char *digest)
+{
+  connection_t *tmp;
+  tor_assert(conn);
+  tor_assert(conn->type == CONN_TYPE_OR);
+  tor_assert(digest);
+
+  if (!orconn_identity_map)
+    orconn_identity_map = digestmap_new();
+  if (!memcmp(conn->identity_digest, digest, DIGEST_LEN))
+    return;
+  if (tor_digest_is_zero(conn->identity_digest))
+    connection_or_remove_from_identity_map(conn);
+
+  memcpy(conn->identity_digest, digest, DIGEST_LEN);
+  tmp = digestmap_set(orconn_identity_map, digest, conn);
+  conn->next_with_same_id = tmp;
+
+  /* Checking code; remove once I'm sure this works. XXXX*/
+  for (; tmp; tmp = tmp->next_with_same_id) {
+    tor_assert(!memcmp(tmp->identity_digest, digest, DIGEST_LEN));
+    tor_assert(tmp != conn);
+  }
+}
+
 /** Pack the cell_t host-order structure <b>src</b> into network-order
  * in the buffer <b>dest</b>. See tor-spec.txt for details about the
  * wire format.
@@ -103,8 +189,8 @@ connection_or_read_proxy_response(connection_t *conn)
   }
   /* else, bad news on the status code */
   warn(LD_OR,
-      "The https proxy sent back an unexpected status code %d (\"%s\"). Closing.",
-      status_code, reason);
+       "The https proxy sent back an unexpected status code %d (\"%s\"). Closing.",
+       status_code, reason);
   tor_free(reason);
   connection_mark_for_close(conn);
   return -1;
@@ -174,7 +260,7 @@ connection_or_finished_connecting(connection_t *conn)
   tor_assert(conn->state == OR_CONN_STATE_CONNECTING);
 
   debug(LD_OR,"OR connect() to router at %s:%u finished.",
-         conn->address,conn->port);
+        conn->address,conn->port);
 
   if (get_options()->HttpsProxy) {
     char buf[1024];
@@ -227,7 +313,7 @@ connection_or_init_conn_from_router(connection_t *conn, routerinfo_t *router)
   conn->port = router->or_port;
   conn->receiver_bucket = conn->bandwidth = (int)options->BandwidthBurst;
   conn->identity_pkey = crypto_pk_dup_key(router->identity_pkey);
-  crypto_pk_get_digest(conn->identity_pkey, conn->identity_digest);
+  connection_or_set_identity_digest(conn, router->cache_info.identity_digest);
   conn->nickname = tor_strdup(router->nickname);
   tor_free(conn->address);
   conn->address = tor_strdup(router->address);
@@ -252,7 +338,7 @@ connection_or_init_conn_from_address(connection_t *conn,
   conn->port = port;
   /* This next part isn't really right, but it's good enough for now. */
   conn->receiver_bucket = conn->bandwidth = (int)options->BandwidthBurst;
-  memcpy(conn->identity_digest, id_digest, DIGEST_LEN);
+  connection_or_set_identity_digest(conn, id_digest);
   /* If we're an authoritative directory server, we may know a
    * nickname for this router. */
   n = dirserv_get_nickname_by_digest(id_digest);
@@ -268,38 +354,48 @@ connection_or_init_conn_from_address(connection_t *conn,
   conn->address = tor_dup_addr(addr);
 }
 
-/** "update an OR connection nickname on the fly"
- * Actually, nobody calls this. Should we remove it? */
-void
-connection_or_update_nickname(connection_t *conn)
+/** Return the best connection of type OR with the
+ * digest <b>digest</b> that we have, or NULL if we have none.
+ *
+ * 1) Don't return it if it's marked for close.
+ * 2) If there are any open conns, ignore non-open conns.
+ * 3) If there are any non-obsolete conns, ignore obsolete conns.
+ * 4) Then if there are any non-empty conns, ignore empty conns.
+ * 5) Of the remaining conns, prefer newer conns.
+ */
+connection_t *
+connection_or_get_by_identity_digest(const char *digest)
 {
-  routerinfo_t *r;
-  const char *n;
+  int newer;
+  connection_t *conn, *best=NULL;
 
-  tor_assert(conn);
-  tor_assert(conn->type == CONN_TYPE_OR);
-  n = dirserv_get_nickname_by_digest(conn->identity_digest);
-  if (n) {
-    if (!conn->nickname || strcmp(conn->nickname, n)) {
-      tor_free(conn->nickname);
-      conn->nickname = tor_strdup(n);
+  if (!orconn_identity_map)
+    return NULL;
+
+  conn = digestmap_get(orconn_identity_map, digest);
+
+  for (; conn; conn = conn->next_with_same_id) {
+    tor_assert(conn->magic == CONNECTION_MAGIC);
+    tor_assert(conn->type == CONN_TYPE_OR);
+    tor_assert(!memcmp(conn->identity_digest, digest, DIGEST_LEN));
+    if (conn->marked_for_close)
+      continue;
+    if (!best) {
+      best = conn; /* whatever it is, it's better than nothing. */
+      continue;
     }
-    return;
-  }
-  r = router_get_by_digest(conn->identity_digest);
-  if (r && r->is_verified) {
-    if (!conn->nickname || strcmp(conn->nickname, r->nickname)) {
-      tor_free(conn->nickname);
-      conn->nickname = tor_strdup(r->nickname);
-    }
-    return;
-  }
-  if (conn->nickname[0] != '$') {
-    tor_free(conn->nickname);
-    conn->nickname = tor_malloc(HEX_DIGEST_LEN+1);
-    base16_encode(conn->nickname, HEX_DIGEST_LEN+1,
-                  conn->identity_digest, DIGEST_LEN);
+    if (best->state == OR_CONN_STATE_OPEN &&
+        conn->state != OR_CONN_STATE_OPEN)
+      continue; /* avoid non-open conns if we can */
+    newer = best->timestamp_created < conn->timestamp_created;
+    if (conn->is_obsolete && (!best->is_obsolete || !newer))
+      continue; /* we have something, and it's better than this. */
+    if (best->n_circuits && !conn->n_circuits)
+      continue; /* prefer conns with circuits on them */
+    if (newer)
+      best = conn; /* lastly, prefer newer conns */
   }
+  return best;
 }
 
 /** Launch a new OR connection to <b>addr</b>:<b>port</b> and expect to
@@ -419,8 +515,6 @@ connection_tls_continue_handshake(connection_t *conn)
   return 0;
 }
 
-static char ZERO_DIGEST[] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
-
 /** Return 1 if we initiated this connection, or 0 if it started
  * out as an incoming connection.
  *
@@ -431,10 +525,9 @@ static char ZERO_DIGEST[] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
 int
 connection_or_nonopen_was_started_here(connection_t *conn)
 {
-  tor_assert(sizeof(ZERO_DIGEST) == DIGEST_LEN);
   tor_assert(conn->type == CONN_TYPE_OR);
 
-  if (!memcmp(ZERO_DIGEST, conn->identity_digest, DIGEST_LEN))
+  if (tor_digest_is_zero(conn->identity_digest))
     return 0;
   else
     return 1;
@@ -479,7 +572,7 @@ connection_or_check_valid_handshake(connection_t *conn, char *digest_rcvd)
   }
   check_no_tls_errors();
   debug(LD_OR, "Other side (%s:%d) claims to be router '%s'",
-         conn->address, conn->port, nickname);
+        conn->address, conn->port, nickname);
 
   if (tor_tls_verify(severity, conn->tls, &identity_rcvd) < 0) {
     log_fn(severity,LD_OR,"Other side, which claims to be router '%s' (%s:%d), has a cert but it's invalid. Closing.",
@@ -558,12 +651,14 @@ connection_tls_finish_handshake(connection_t *conn)
     return -1;
 
   if (!connection_or_nonopen_was_started_here(conn)) {
+#if 0
     connection_t *c;
-    if ((c=connection_get_by_identity_digest(digest_rcvd, CONN_TYPE_OR))) {
+    if ((c=connection_or_get_by_identity_digest(digest_rcvd))) {
       debug(LD_OR,"Router '%s' is already connected on fd %d. Dropping fd %d.",
-             c->nickname, c->s, conn->s);
+            c->nickname, c->s, conn->s);
       return -1;
     }
+#endif
     connection_or_init_conn_from_address(conn,conn->addr,conn->port,digest_rcvd);
   }
 
@@ -615,7 +710,7 @@ connection_or_write_cell_to_buf(const cell_t *cell, connection_t *conn)
       if (!conn->marked_for_close) {
         /* this connection is broken. remove it. */
         warn(LD_BUG,"Bug: unhandled error on write for OR conn (fd %d); removing",
-               conn->s);
+             conn->s);
         tor_fragile_assert();
         conn->has_sent_end = 1; /* otherwise we cry wolf about duplicate close */
         /* XXX do we need a close-immediate here, so we don't try to flush? */
@@ -645,7 +740,7 @@ connection_or_process_cells_from_inbuf(connection_t *conn)
 
 loop:
   debug(LD_OR,"%d: starting, inbuf_datalen %d (%d pending in tls object).",
-         conn->s,(int)buf_datalen(conn->inbuf),tor_tls_get_pending_bytes(conn->tls));
+        conn->s,(int)buf_datalen(conn->inbuf),tor_tls_get_pending_bytes(conn->tls));
   if (buf_datalen(conn->inbuf) < CELL_NETWORK_SIZE) /* entire response available? */
     return 0; /* not yet */
 
