@@ -1,7 +1,7 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: control.c,v 1.147 2005/11/05 20:15:26 nickm Exp $ */
-const char control_c_id[] = "$Id: control.c,v 1.147 2005/11/05 20:15:26 nickm Exp $";
+/* $Id: control.c,v 1.156 2005/12/10 09:36:25 arma Exp $ */
+const char control_c_id[] = "$Id: control.c,v 1.156 2005/12/10 09:36:25 arma Exp $";
 
 /**
  * \file control.c
@@ -73,7 +73,8 @@ const char control_c_id[] = "$Id: control.c,v 1.147 2005/11/05 20:15:26 nickm Ex
 #define EVENT_ERR_MSG         0x000B
 #define LAST_V0_EVENT         0x000B
 #define EVENT_ADDRMAP         0x000C
-#define _EVENT_MAX            0x000C
+#define EVENT_AUTHDIR_NEWDESCS 0x000D
+#define _EVENT_MAX            0x000D
 
 /** Array mapping from message type codes to human-readable message
  * type names. Used for compatibility with version 0 of the control
@@ -712,26 +713,26 @@ control_setconf_helper(connection_t *conn, uint32_t len, char *body,
     switch (r) {
       case -1:
         v0_err = ERR_UNRECOGNIZED_CONFIG_KEY;
-        msg = "Unrecognized option";
+        msg = "552 Unrecognized option";
         break;
       case -2:
         v0_err = ERR_INVALID_CONFIG_VALUE;
-        msg = "Unrecognized option value";
+        msg = "513 Unrecognized option value";
         break;
       case -3:
         v0_err = ERR_INVALID_CONFIG_VALUE;
-        msg = "Transition not allowed";
+        msg = "553 Transition not allowed";
         break;
       case -4:
       default:
         v0_err = ERR_INVALID_CONFIG_VALUE;
-        msg = "Unable to set option";
+        msg = "553 Unable to set option";
         break;
     }
     if (v0) {
       send_control0_error(conn, v0_err, msg);
     } else {
-      connection_printf_to_buf(conn, "552 %s\r\n", msg);
+      connection_printf_to_buf(conn, "%s\r\n", msg);
     }
     config_free_lines(lines);
     return 0;
@@ -916,6 +917,8 @@ handle_control_setevents(connection_t *conn, uint32_t len, const char *body)
           event_code = EVENT_NEW_DESC;
         else if (!strcasecmp(ev, "ADDRMAP"))
           event_code = EVENT_ADDRMAP;
+        else if (!strcasecmp(ev, "AUTHDIR_NEWDESCS"))
+          event_code = EVENT_AUTHDIR_NEWDESCS;
         else {
           connection_printf_to_buf(conn, "552 Unrecognized event \"%s\"\r\n",
                                    ev);
@@ -982,7 +985,10 @@ handle_control_authenticate(connection_t *conn, uint32_t len, const char *body)
         ++i;
       password = tor_malloc(i/2 + 1);
       if (base16_decode(password, i/2+1, body, i)<0) {
-        connection_write_str_to_buf("551 Invalid hexadecimal encoding.  Maybe you tried a plain text password?  If so, the standard requires you put it in double quotes.\r\n", conn);
+        connection_write_str_to_buf(
+            "551 Invalid hexadecimal encoding.  Maybe you tried a plain text "
+            "password?  If so, the standard requires you put it in double "
+            "quotes.\r\n", conn);
         tor_free(password);
         return 0;
       }
@@ -1028,7 +1034,9 @@ handle_control_authenticate(connection_t *conn, uint32_t len, const char *body)
     if (used_quoted_string)
       connection_write_str_to_buf("515 Authentication failed\r\n", conn);
     else
-      connection_write_str_to_buf("515 Authentication failed.  Maybe you tried a plain text password?  If so, the standard requires you put it in double quotes.\r\n", conn);
+      connection_write_str_to_buf(
+         "515 Authentication failed.  Maybe you tried a plain text password?  "
+         "If so, the standard requires you put it in double quotes.\r\n",conn);
   }
   return 0;
  ok:
@@ -1331,7 +1339,7 @@ handle_getinfo_helper(const char *question, char **answer)
           state = "SUCCEEDED"; break;
         default:
           warn(LD_BUG, "Asked for stream in unknown state %d",
-                 conns[i]->state);
+               conns[i]->state);
           continue;
         }
       circ = circuit_get_by_edge_conn(conns[i]);
@@ -1452,7 +1460,7 @@ handle_control_getinfo(connection_t *conn, uint32_t len, const char *body)
     tor_assert(msg_len > 0); /* it will at least be terminated */
     send_control0_message(conn, CONTROL0_CMD_INFOVALUE,
                           msg_len, msg);
-  } else if (smartlist_len(answers)) {
+  } else {
     int i;
     for (i = 0; i < smartlist_len(answers); i += 2) {
       char *k = smartlist_get(answers, i);
@@ -1591,7 +1599,7 @@ handle_control_extendcircuit(connection_t *conn, uint32_t len,
     }
   } else {
     if (circ->state == CIRCUIT_STATE_OPEN) {
-      circ->state = CIRCUIT_STATE_BUILDING;
+      circuit_set_state(circ, CIRCUIT_STATE_BUILDING);
       if (circuit_send_next_onion_skin(circ) < 0) {
         info(LD_CONTROL,"send_next_onion_skin failed; circuit marked for closing.");
         circuit_mark_for_close(circ);
@@ -1756,7 +1764,7 @@ handle_control_postdescriptor(connection_t *conn, uint32_t len,
 }
 
 /** Called when we receive a REDIRECTSTERAM command.  Try to change the target
- * adderess of the named AP steream, and report success or failure. */
+ * address of the named AP stream, and report success or failure. */
 static int
 handle_control_redirectstream(connection_t *conn, uint32_t len,
                               const char *body)
@@ -1764,6 +1772,7 @@ handle_control_redirectstream(connection_t *conn, uint32_t len,
   connection_t *ap_conn = NULL;
   uint32_t conn_id;
   char *new_addr = NULL;
+  uint16_t new_port = 0;
   if (STATE_IS_V0(conn->state)) {
     if (len < 6) {
       send_control0_error(conn, ERR_SYNTAX, "redirectstream message too short");
@@ -1784,14 +1793,24 @@ handle_control_redirectstream(connection_t *conn, uint32_t len,
     args = smartlist_create();
     smartlist_split_string(args, body, " ",
                            SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 0);
-    if (smartlist_len(args)<2)
+    if (smartlist_len(args) < 2)
       connection_printf_to_buf(conn,"512 Missing argument to REDIRECTSTREAM\r\n");
     else if (!(ap_conn = get_stream(smartlist_get(args, 0)))
              || !ap_conn->socks_request) {
       connection_printf_to_buf(conn, "552 Unknown stream \"%s\"\r\n",
                                (char*)smartlist_get(args, 0));
     } else {
-      new_addr = tor_strdup(smartlist_get(args, 1));
+      int ok;
+      if (smartlist_len(args) > 2) { /* they included a port too */
+        new_port = (uint16_t) tor_parse_ulong(smartlist_get(args, 2),
+                                     10, 1, 65535, &ok, NULL);
+      }
+      if (!ok) {
+        connection_printf_to_buf(conn, "512 Cannot parse port \"%s\"\r\n",
+                                 (char*)smartlist_get(args, 2));
+      } else {
+        new_addr = tor_strdup(smartlist_get(args, 1));
+      }
     }
 
     SMARTLIST_FOREACH(args, char *, cp, tor_free(cp));
@@ -1802,6 +1821,8 @@ handle_control_redirectstream(connection_t *conn, uint32_t len,
 
   strlcpy(ap_conn->socks_request->address, new_addr,
           sizeof(ap_conn->socks_request->address));
+  if (new_port)
+    ap_conn->socks_request->port = new_port;
   tor_free(new_addr);
   send_control_done(conn);
   return 0;
@@ -2260,7 +2281,7 @@ connection_control_process_inbuf_v0(connection_t *conn)
     case CONTROL0_CMD_EVENT:
     case CONTROL0_CMD_INFOVALUE:
       warn(LD_CONTROL, "Received client-only '%s' command; ignoring.",
-             control_cmd_to_string(command_type));
+           control_cmd_to_string(command_type));
       send_control0_error(conn, ERR_UNRECOGNIZED_TYPE,
                          "Command type only valid from server to tor client");
       break;
@@ -2270,7 +2291,7 @@ connection_control_process_inbuf_v0(connection_t *conn)
       send_control0_error(conn, ERR_SYNTAX, "Bad fragmentation on command.");
     default:
       warn(LD_CONTROL, "Received unrecognized command type %d; ignoring.",
-             (int)command_type);
+           (int)command_type);
       send_control0_error(conn, ERR_UNRECOGNIZED_TYPE,
                          "Unrecognized command type");
       break;
@@ -2349,9 +2370,11 @@ write_stream_target_to_buf(connection_t *conn, char *buf, size_t len)
   if (conn->chosen_exit_name)
     if (tor_snprintf(buf2, sizeof(buf2), ".%s.exit", conn->chosen_exit_name)<0)
       return -1;
-  if (tor_snprintf(buf, len, "%s%s:%d",
+  if (tor_snprintf(buf, len, "%s%s%s:%d",
                    conn->socks_request->address,
                    conn->chosen_exit_name ? buf2 : "",
+                   !conn->chosen_exit_name &&
+                     connection_edge_is_rendezvous_stream(conn) ? ".onion" : "",
                    conn->socks_request->port)<0)
     return -1;
   return 0;
@@ -2596,6 +2619,44 @@ control_event_address_mapped(const char *from, const char *to, time_t expires)
   return 0;
 }
 
+/** The authoritative dirserver has received a new descriptor that
+ * has passed basic syntax checks and is properly self-signed.
+ *
+ * Notify any interested party of the new descriptor and what has
+ * been done with it, and also optionally give an explanation/reason. */
+int
+control_event_or_authdir_new_descriptor(const char *action, const char *descriptor, const char *msg)
+
+{
+  char firstline[1024];
+  char *buf;
+  int totallen;
+  char *esc = NULL;
+  size_t esclen;
+
+  if (!EVENT_IS_INTERESTING(EVENT_AUTHDIR_NEWDESCS))
+    return 0;
+
+  tor_snprintf(firstline, sizeof(firstline),
+               "650+AUTHDIR_NEWDESC=\r\n%s\r\n%s\r\n",
+               action,
+               msg ? msg : "");
+
+  /* Escape the server descriptor properly */
+  esclen = write_escaped_data(descriptor, strlen(descriptor), 1, &esc);
+
+  totallen = strlen(firstline) + esclen + 1;
+  buf = tor_malloc(totallen);
+  strlcpy(buf, firstline, totallen);
+  strlcpy(buf+strlen(firstline), esc, totallen);
+  send_control1_event_string(EVENT_AUTHDIR_NEWDESCS, buf);
+
+  tor_free(esc);
+  tor_free(buf);
+
+  return 0;
+}
+
 /** Choose a random authentication cookie and write it to disk.
  * Anybody who can read the cookie from disk will be considered
  * authorized to use the control connection. */
