@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c,v 1.319 2005/11/08 22:30:17 nickm Exp $ */
-const char directory_c_id[] = "$Id: directory.c,v 1.319 2005/11/08 22:30:17 nickm Exp $";
+/* $Id: directory.c,v 1.324 2005/12/10 09:36:25 arma Exp $ */
+const char directory_c_id[] = "$Id: directory.c,v 1.324 2005/12/10 09:36:25 arma Exp $";
 
 #include "or.h"
 
@@ -133,6 +133,7 @@ directory_post_to_dirservers(uint8_t purpose, const char *payload,
                              size_t payload_len)
 {
   smartlist_t *dirservers;
+  int post_via_tor;
 
   router_get_trusted_dir_servers(&dirservers);
   tor_assert(dirservers);
@@ -141,23 +142,15 @@ directory_post_to_dirservers(uint8_t purpose, const char *payload,
    */
   SMARTLIST_FOREACH(dirservers, trusted_dir_server_t *, ds,
     {
-      /* Pay attention to fascistfirewall when we're uploading a
-       * router descriptor, but not when uploading a service
-       * descriptor -- those use Tor. */
-      if (purpose == DIR_PURPOSE_UPLOAD_DIR && !get_options()->HttpProxy) {
-        if (!fascist_firewall_allows_address(ds->addr,ds->dir_port))
-          continue;
-      }
-      directory_initiate_command_trusted_dir(ds, purpose,
-                                             purpose_is_private(purpose),
+      post_via_tor = purpose_is_private(purpose) ||
+                     !fascist_firewall_allows_address(ds->addr,ds->dir_port);
+      directory_initiate_command_trusted_dir(ds, purpose, post_via_tor,
                                              NULL, payload, payload_len);
     });
 }
 
 /** Start a connection to a random running directory server, using
- * connection purpose 'purpose' requesting 'resource'.  The purpose
- * should be one of 'DIR_PURPOSE_FETCH_DIR',
- * 'DIR_PURPOSE_FETCH_RENDDESC', 'DIR_PURPOSE_FETCH_RUNNING_LIST.'
+ * connection purpose 'purpose' and requesting 'resource'.
  * If <b>retry_if_no_servers</b>, then if all the possible servers seem
  * down, mark them up and try again.
  */
@@ -167,7 +160,6 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
 {
   routerinfo_t *r = NULL;
   trusted_dir_server_t *ds = NULL;
-  int fascistfirewall = firewall_is_fascist();
   or_options_t *options = get_options();
   int fetch_fresh_first = server_mode(options) && options->DirPort != 0;
   int directconn = !purpose_is_private(purpose);
@@ -187,12 +179,12 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
     }
     if (!ds && fetch_fresh_first) {
       /* only ask authdirservers, and don't ask myself */
-      ds = router_pick_trusteddirserver(need_v1_support, 1, fascistfirewall,
+      ds = router_pick_trusteddirserver(need_v1_support, 1, 1,
                                         retry_if_no_servers);
     }
     if (!ds) {
       /* anybody with a non-zero dirport will do */
-      r = router_pick_directory_server(1, fascistfirewall, need_v2_support,
+      r = router_pick_directory_server(1, 1, need_v2_support,
                                        retry_if_no_servers);
       if (!r) {
         const char *which;
@@ -205,15 +197,24 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
         else // if (purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS)
           which = "server descriptors";
         info(LD_DIR,
-             "No router found for %s; falling back to dirserver list",which);
-        ds = router_pick_trusteddirserver(1, 1, fascistfirewall,
+             "No router found for %s; falling back to dirserver list", which);
+        ds = router_pick_trusteddirserver(1, 1, 1,
                                           retry_if_no_servers);
+        if (!ds)
+          directconn = 0; /* last resort: try routing it via Tor */
       }
     }
-  } else { // (purpose == DIR_PURPOSE_FETCH_RENDDESC)
-    /* only ask authdirservers, any of them will do */
+  }
+  if (!directconn) {
     /* Never use fascistfirewall; we're going via Tor. */
-    ds = router_pick_trusteddirserver(0, 0, 0, retry_if_no_servers);
+    if (purpose == DIR_PURPOSE_FETCH_RENDDESC) {
+      /* only ask authdirservers, any of them will do */
+      ds = router_pick_trusteddirserver(0, 0, 0, retry_if_no_servers);
+    } else {
+      /* anybody with a non-zero dirport will do. Disregard firewalls. */
+      r = router_pick_directory_server(1, 0, need_v2_support,
+                                       retry_if_no_servers);
+    }
   }
 
   if (r)
@@ -226,7 +227,7 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
     notice(LD_DIR,
            "No running dirservers known. Will try again later. (purpose %d)",
            purpose);
-    if (directconn) {
+    if (!purpose_is_private(purpose)) {
       /* remember we tried them all and failed. */
       directory_all_unreachable(time(NULL));
     }
@@ -281,16 +282,16 @@ connection_dir_request_failed(connection_t *conn)
   if (conn->purpose == DIR_PURPOSE_FETCH_DIR ||
       conn->purpose == DIR_PURPOSE_FETCH_RUNNING_LIST) {
     info(LD_DIR, "Giving up on directory server at '%s:%d'; retrying",
-           conn->address, conn->port);
+         conn->address, conn->port);
     directory_get_from_dirserver(conn->purpose, NULL,
                                  0 /* don't retry_if_no_servers */);
   } else if (conn->purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS) {
     info(LD_DIR, "Giving up on directory server at '%s'; retrying",
-           conn->address);
+         conn->address);
     connection_dir_download_networkstatus_failed(conn);
   } else if (conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC) {
     info(LD_DIR, "Giving up on directory server at '%s'; retrying",
-           conn->address);
+         conn->address);
     connection_dir_download_routerdesc_failed(conn);
   }
 }
@@ -500,7 +501,7 @@ directory_send_command(connection_t *conn, const char *platform,
       tor_assert(!resource);
       tor_assert(!payload);
       debug(LD_DIR, "Asking for compressed directory from server running %s",
-             platform?platform:"<unknown version>");
+            platform?platform:"<unknown version>");
       httpcommand = "GET";
       url = tor_strdup("/tor/dir.z");
       break;
@@ -814,7 +815,8 @@ connection_dir_client_reached_eof(connection_t *conn)
                               &body, &body_len, MAX_DIR_SIZE,
                               allow_partial)) {
     case -1: /* overflow */
-      warn(LD_PROTOCOL,"'fetch' response too large (server '%s:%d'). Closing.", conn->address, conn->port);
+      warn(LD_PROTOCOL,"'fetch' response too large (server '%s:%d'). Closing.",
+           conn->address, conn->port);
       return -1;
     case 0:
       info(LD_HTTP,"'fetch' response not all here, but we're at eof. Closing.");
@@ -824,7 +826,8 @@ connection_dir_client_reached_eof(connection_t *conn)
 
   if (parse_http_response(headers, &status_code, &date_header,
                           &compression, &reason) < 0) {
-    warn(LD_HTTP,"Unparseable headers (server '%s:%d'). Closing.", conn->address, conn->port);
+    warn(LD_HTTP,"Unparseable headers (server '%s:%d'). Closing.",
+         conn->address, conn->port);
     tor_free(body); tor_free(headers);
     return -1;
   }
@@ -876,9 +879,9 @@ connection_dir_client_reached_eof(connection_t *conn)
         description2 = "uncompressed";
 
       info(LD_HTTP, "HTTP body from server '%s:%d' was labeled %s, "
-             "but it seems to be %s.%s",
-             conn->address, conn->port, description1, description2,
-             (compression>0 && guessed>0)?"  Trying both.":"");
+           "but it seems to be %s.%s",
+           conn->address, conn->port, description1, description2,
+           (compression>0 && guessed>0)?"  Trying both.":"");
     }
     /* Try declared compression first if we can. */
     if (compression > 0)
@@ -907,16 +910,16 @@ connection_dir_client_reached_eof(connection_t *conn)
   if (conn->purpose == DIR_PURPOSE_FETCH_DIR) {
     /* fetch/process the directory to learn about new routers. */
     info(LD_DIR,"Received directory (size %d) from server '%s:%d'",
-           (int)body_len, conn->address, conn->port);
+         (int)body_len, conn->address, conn->port);
     if (status_code == 503 || body_len == 0) {
       info(LD_DIR,"Empty directory; status %d (\"%s\") Ignoring.",
-             status_code, reason);
+           status_code, reason);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
     if (status_code != 200) {
       warn(LD_DIR,"Received http status code %d (\"%s\") from server '%s:%d'. I'll try again soon.",
-             status_code, reason, conn->address, conn->port);
+           status_code, reason, conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
@@ -930,13 +933,13 @@ connection_dir_client_reached_eof(connection_t *conn)
     info(LD_DIR,"Received running-routers list (size %d)", (int)body_len);
     if (status_code != 200) {
       warn(LD_DIR,"Received http status code %d (\"%s\") from server '%s:%d'. I'll try again soon.",
-             status_code, reason, conn->address, conn->port);
+           status_code, reason, conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
     if (router_parse_runningrouters(body)<0) {
       warn(LD_DIR,"Bad running-routers from server '%s:%d'. I'll try again soon.",
-             conn->address, conn->port);
+           conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
@@ -948,8 +951,8 @@ connection_dir_client_reached_eof(connection_t *conn)
     info(LD_DIR,"Received networkstatus objects (size %d) from server '%s:%d'",(int) body_len, conn->address, conn->port);
     if (status_code != 200) {
       warn(LD_DIR,"Received http status code %d (\"%s\") from server '%s:%d' while fetching \"/tor/status/%s\". I'll try again soon.",
-             status_code, reason, conn->address, conn->port,
-             conn->requested_resource);
+           status_code, reason, conn->address, conn->port,
+           conn->requested_resource);
       tor_free(body); tor_free(headers); tor_free(reason);
       connection_dir_download_networkstatus_failed(conn);
       return -1;
@@ -990,7 +993,7 @@ connection_dir_client_reached_eof(connection_t *conn)
     smartlist_t *which = NULL;
     int n_asked_for = 0;
     info(LD_DIR,"Received server info (size %d) from server '%s:%d'",
-           (int)body_len, conn->address, conn->port);
+         (int)body_len, conn->address, conn->port);
     if (conn->requested_resource &&
         !strcmpstart(conn->requested_resource,"fp/")) {
       which = smartlist_create();
@@ -1030,8 +1033,8 @@ connection_dir_client_reached_eof(connection_t *conn)
     }
     if (which) { /* mark remaining ones as failed */
       info(LD_DIR, "Received %d/%d routers requested from %s:%d",
-             n_asked_for-smartlist_len(which), n_asked_for,
-             conn->address, (int)conn->port);
+           n_asked_for-smartlist_len(which), n_asked_for,
+           conn->address, (int)conn->port);
       if (smartlist_len(which)) {
         dir_routerdesc_download_failed(which);
       }
@@ -1059,10 +1062,15 @@ connection_dir_client_reached_eof(connection_t *conn)
         warn(LD_GENERAL,"http status 400 (\"%s\") response from dirserver '%s:%d'. Please correct.", reason, conn->address, conn->port);
         break;
       case 403:
-        warn(LD_GENERAL,"http status 403 (\"%s\") response from dirserver '%s:%d'. Is your clock skewed? Have you mailed us your key fingerprint? Are you using the right key? Are you using a private IP address? See http://tor.eff.org/doc/tor-doc-server.html", reason, conn->address, conn->port);
+        warn(LD_GENERAL,"http status 403 (\"%s\") response from dirserver "
+             "'%s:%d'. Is your clock skewed? Have you mailed us your key "
+             "fingerprint? Are you using the right key? Are you using a "
+             "private IP address? See http://tor.eff.org/doc/"
+             "tor-doc-server.html", reason, conn->address, conn->port);
         break;
       default:
-        warn(LD_GENERAL,"http status %d (\"%s\") reason unexpected (server '%s:%d').", status_code, reason, conn->address, conn->port);
+        warn(LD_GENERAL,"http status %d (\"%s\") reason unexpected (server '%s:%d').",
+             status_code, reason, conn->address, conn->port);
         break;
     }
     /* return 0 in all cases, since we don't want to mark any
@@ -1071,7 +1079,7 @@ connection_dir_client_reached_eof(connection_t *conn)
 
   if (conn->purpose == DIR_PURPOSE_FETCH_RENDDESC) {
     info(LD_REND,"Received rendezvous descriptor (size %d, status %d (\"%s\"))",
-           (int)body_len, status_code, reason);
+         (int)body_len, status_code, reason);
     switch (status_code) {
       case 200:
         if (rend_cache_store(body, body_len) < 0) {
@@ -1092,7 +1100,8 @@ connection_dir_client_reached_eof(connection_t *conn)
         warn(LD_REND,"http status 400 (\"%s\"). Dirserver didn't like our rendezvous query?", reason);
         break;
       default:
-        warn(LD_REND,"http status %d (\"%s\") response unexpected (server '%s:%d').", status_code, reason, conn->address, conn->port);
+        warn(LD_REND,"http status %d (\"%s\") response unexpected (server '%s:%d').",
+             status_code, reason, conn->address, conn->port);
         break;
     }
   }
@@ -1103,10 +1112,12 @@ connection_dir_client_reached_eof(connection_t *conn)
         info(LD_REND,"Uploading rendezvous descriptor: finished with status 200 (\"%s\")", reason);
         break;
       case 400:
-        warn(LD_REND,"http status 400 (\"%s\") response from dirserver '%s:%d'. Malformed rendezvous descriptor?", reason, conn->address, conn->port);
+        warn(LD_REND,"http status 400 (\"%s\") response from dirserver '%s:%d'. Malformed rendezvous descriptor?",
+             reason, conn->address, conn->port);
         break;
       default:
-        warn(LD_REND,"http status %d (\"%s\") response unexpected (server '%s:%d').", status_code, reason, conn->address, conn->port);
+        warn(LD_REND,"http status %d (\"%s\") response unexpected (server '%s:%d').",
+             status_code, reason, conn->address, conn->port);
         break;
     }
   }
@@ -1125,7 +1136,9 @@ connection_dir_reached_eof(connection_t *conn)
      * parsing partial serverdesc responses. */
     if (conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC &&
         buf_datalen(conn->inbuf)>=(24*1024)) {
-      notice(LD_DIR, "Directory connection closed early after downloading %d bytes of descriptors.  If this happens often, please file a bug report.",
+      notice(LD_DIR,
+             "Directory connection closed early after downloading %d bytes "
+             "of descriptors. If this happens often, please file a bug report.",
              (int)buf_datalen(conn->inbuf));
     }
     connection_close_immediate(conn); /* it was an error; give up on flushing */
@@ -1252,11 +1265,13 @@ directory_handle_command_get(connection_t *conn, char *headers,
     debug(LD_DIRSERV,"Dumping %sdirectory to client.",
           deflated?"deflated ":"");
     format_rfc1123_time(date, time(NULL));
-    tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: %s\r\nContent-Encoding: %s\r\n\r\n",
-             date,
-             (int)dlen,
-             deflated?"application/octet-stream":"text/plain",
-             deflated?"deflate":"identity");
+    tor_snprintf(tmp, sizeof(tmp),
+                 "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\n"
+                 "Content-Type: %s\r\nContent-Encoding: %s\r\n\r\n",
+                 date,
+                 (int)dlen,
+                 deflated?"application/octet-stream":"text/plain",
+                 deflated?"deflate":"identity");
     connection_write_to_buf(tmp, strlen(tmp), conn);
     connection_write_to_buf(cp, dlen, conn);
     return 0;
@@ -1276,7 +1291,9 @@ directory_handle_command_get(connection_t *conn, char *headers,
     }
 
     format_rfc1123_time(date, time(NULL));
-    tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: %s\r\nContent-Encoding: %s\r\n\r\n",
+    tor_snprintf(tmp, sizeof(tmp),
+                 "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\n"
+                 "Content-Type: %s\r\nContent-Encoding: %s\r\n\r\n",
                  date,
                  (int)dlen,
                  deflated?"application/octet-stream":"text/plain",
@@ -1308,7 +1325,9 @@ directory_handle_command_get(connection_t *conn, char *headers,
     SMARTLIST_FOREACH(dir_objs, cached_dir_t *, d,
                       dlen += deflated?d->dir_z_len:d->dir_len);
     format_rfc1123_time(date, time(NULL));
-    tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: %s\r\nContent-Encoding: %s\r\n\r\n",
+    tor_snprintf(tmp, sizeof(tmp),
+                 "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\n"
+                 "Content-Type: %s\r\nContent-Encoding: %s\r\n\r\n",
                  date,
                  (int)dlen,
                  deflated?"application/octet-stream":"text/plain",
@@ -1364,14 +1383,19 @@ directory_handle_command_get(connection_t *conn, char *headers,
           return -1;
         }
         tor_free(inp);
-        tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: application/octet-stream\r\nContent-Encoding: deflate\r\n\r\n",
+        tor_snprintf(tmp, sizeof(tmp),
+                     "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\n"
+                     "Content-Type: application/octet-stream\r\n"
+                     "Content-Encoding: deflate\r\n\r\n",
                      date,
                      (int)compressed_len);
         connection_write_to_buf(tmp, strlen(tmp), conn);
         connection_write_to_buf(compressed, compressed_len, conn);
         tor_free(compressed);
       } else {
-        tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: text/plain\r\n\r\n",
+        tor_snprintf(tmp, sizeof(tmp),
+                     "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\n"
+                     "Content-Type: text/plain\r\n\r\n",
                      date,
                      (int)len);
         connection_write_to_buf(tmp, strlen(tmp), conn);
@@ -1406,10 +1430,13 @@ directory_handle_command_get(connection_t *conn, char *headers,
     switch (rend_cache_lookup_desc(query, versioned?-1:0, &descp, &desc_len)) {
       case 1: /* valid */
         format_rfc1123_time(date, time(NULL));
-        tor_snprintf(tmp, sizeof(tmp), "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\nContent-Type: application/octet-stream\r\n\r\n",
-                 date,
-                 (int)desc_len); /* can't include descp here, because it's got nuls */
+        tor_snprintf(tmp, sizeof(tmp),
+                     "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\n"
+                     "Content-Type: application/octet-stream\r\n\r\n",
+                     date,
+                     (int)desc_len);
         connection_write_to_buf(tmp, strlen(tmp), conn);
+        /* need to send descp separately, because it may include nuls */
         connection_write_to_buf(descp, desc_len, conn);
         break;
       case 0: /* well-formed but not present */
@@ -1644,13 +1671,13 @@ dir_routerdesc_download_failed(smartlist_t *failed)
     }
     if (rs->next_attempt_at == 0)
       debug(LD_DIR, "%s failed %d time(s); I'll try again immediately.",
-             cp, (int)rs->n_download_failures);
+            cp, (int)rs->n_download_failures);
     else if (rs->next_attempt_at < TIME_MAX)
       debug(LD_DIR, "%s failed %d time(s); I'll try again in %d seconds.",
-             cp, (int)rs->n_download_failures, (int)(rs->next_attempt_at-now));
+            cp, (int)rs->n_download_failures, (int)(rs->next_attempt_at-now));
     else
       debug(LD_DIR, "%s failed %d time(s); Giving up for a while.",
-             cp, (int)rs->n_download_failures);
+            cp, (int)rs->n_download_failures);
   });
 
   /* update_router_descriptor_downloads(time(NULL)); */
