@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dirserv.c,v 1.266 2005/11/05 20:15:27 nickm Exp $ */
-const char dirserv_c_id[] = "$Id: dirserv.c,v 1.266 2005/11/05 20:15:27 nickm Exp $";
+/* $Id: dirserv.c,v 1.277 2005/12/10 09:36:25 arma Exp $ */
+const char dirserv_c_id[] = "$Id: dirserv.c,v 1.277 2005/12/10 09:36:25 arma Exp $";
 
 #include "or.h"
 
@@ -39,6 +39,12 @@ int add_fingerprint_to_dir(const char *nickname, const char *fp, smartlist_t *li
 static int router_is_general_exit(routerinfo_t *ri);
 static router_status_t dirserv_router_get_status(const routerinfo_t *router,
                                                  const char **msg);
+static router_status_t
+dirserv_get_status_impl(const char *fp, const char *nickname,
+                        const char *address,
+                        uint32_t addr, uint16_t or_port,
+                        const char *platform, const char *contact,
+                        const char **msg, int should_log);
 static int dirserv_thinks_router_is_reachable(routerinfo_t *router,
                                               time_t now);
 
@@ -198,8 +204,8 @@ dirserv_parse_fingerprint_file(const char *fname)
       /* If you approved an OR called "client", then clients who use
        * the default nickname could all be rejected.  That's no good. */
       notice(LD_CONFIG,
-          "Authorizing a nickname '%s' would break many clients; skipping.",
-          DEFAULT_CLIENT_NICKNAME);
+             "Authorizing a nickname '%s' would break many clients; skipping.",
+             DEFAULT_CLIENT_NICKNAME);
       continue;
     }
     if (add_fingerprint_to_dir(nickname, fingerprint, fingerprint_list_new) != 0)
@@ -219,28 +225,67 @@ dirserv_parse_fingerprint_file(const char *fname)
  * according to our configuration.  Return the appropriate router status.
  *
  * If the status is 'FP_REJECT' and <b>msg</b> is provided, set
- * *<b>msg</b> to an explanation of why.
- */
+ * *<b>msg</b> to an explanation of why. */
 static router_status_t
 dirserv_router_get_status(const routerinfo_t *router, const char **msg)
 {
-  fingerprint_entry_t *nn_ent = NULL, *fp_ent = NULL;
-  char fp[FINGERPRINT_LEN+1];
+  char fingerprint[FINGERPRINT_LEN+1];
 
-  if (!fingerprint_list)
-    fingerprint_list = smartlist_create();
-
-  if (crypto_pk_get_fingerprint(router->identity_pkey, fp, 0)) {
+  if (crypto_pk_get_fingerprint(router->identity_pkey, fingerprint, 0)) {
     warn(LD_BUG,"Error computing fingerprint");
     return -1;
   }
 
-  debug(LD_DIRSERV, "%d fingerprints known.", smartlist_len(fingerprint_list));
+  return dirserv_get_status_impl(fingerprint, router->nickname,
+                                 router->address,
+                                 router->addr, router->or_port,
+                                 router->platform, router->contact_info,
+                                 msg, 1);
+}
+
+/** Return true if there is no point in downloading the router described by
+ * <b>rs</b> because this directory would reject it. */
+int
+dirserv_would_reject_router(routerstatus_t *rs)
+{
+  char fp[FINGERPRINT_LEN+1];
+  router_status_t res;
+  base16_encode(fp, sizeof(fp), rs->identity_digest, DIGEST_LEN);
+
+  res = dirserv_get_status_impl(fp, rs->nickname,
+                                "", /* address is only used in logs */
+                                rs->addr, rs->or_port,
+                                NULL, NULL,
+                                NULL, 0);
+
+  return (res == FP_REJECT);
+}
+
+/** Helper: As dirserv_get_router_status, but takes the router fingerprint
+ * (hex, no spaces), nickname, address (used for logging only), IP address, OR
+ * port, platform (logging only) and contact info (logging only) as arguments.
+ *
+ * If should_log is false, do not log messages.  (There's not much point in
+ * logging that we're rejecting servers we'll not download.)
+ */
+static router_status_t
+dirserv_get_status_impl(const char *fp, const char *nickname,
+                        const char *address,
+                        uint32_t addr, uint16_t or_port,
+                        const char *platform, const char *contact,
+                        const char **msg, int should_log)
+{
+  fingerprint_entry_t *nn_ent = NULL, *fp_ent = NULL;
+  if (!fingerprint_list)
+    fingerprint_list = smartlist_create();
+
+  if (should_log)
+    debug(LD_DIRSERV, "%d fingerprints known.", smartlist_len(fingerprint_list));
   SMARTLIST_FOREACH(fingerprint_list, fingerprint_entry_t *, ent,
   {
     if (!strcasecmp(fp,ent->fingerprint))
       fp_ent = ent;
-    if (!strcasecmp(router->nickname,ent->nickname))
+    if (!strcasecmp(nickname,ent->nickname))
       nn_ent = ent;
   });
 
@@ -258,37 +303,44 @@ dirserv_router_get_status(const routerinfo_t *router, const char **msg)
 
   if (!nn_ent) { /* No such server known with that nickname */
     addr_policy_result_t rej = router_compare_addr_to_addr_policy(
-                       router->addr, router->or_port, authdir_reject_policy);
+                       addr, or_port, authdir_reject_policy);
     addr_policy_result_t inv = router_compare_addr_to_addr_policy(
-                       router->addr, router->or_port, authdir_invalid_policy);
+                       addr, or_port, authdir_invalid_policy);
 
     if (rej == ADDR_POLICY_PROBABLY_REJECTED || rej == ADDR_POLICY_REJECTED) {
-      info(LD_DIRSERV, "Rejecting '%s' because of address %s",
-           router->nickname, router->address);
+      if (should_log)
+        info(LD_DIRSERV, "Rejecting '%s' because of address %s",
+             nickname, address);
       if (msg)
         *msg = "Authdir is rejecting routers in this range.";
       return FP_REJECT;
     }
     if (inv == ADDR_POLICY_PROBABLY_REJECTED || inv == ADDR_POLICY_REJECTED) {
-      info(LD_DIRSERV, "Not marking '%s' valid because of address %s",
-           router->nickname, router->address);
+      if (should_log)
+        info(LD_DIRSERV, "Not marking '%s' valid because of address %s",
+             nickname, address);
       return FP_INVALID;
     }
-    if (tor_version_as_new_as(router->platform,"0.1.0.2-rc"))
+    if (!platform || tor_version_as_new_as(platform,"0.1.0.2-rc"))
       return FP_VALID;
     else
       return FP_INVALID;
-    info(LD_DIRSERV,"No fingerprint found for '%s'",router->nickname);
+    if (should_log)
+      info(LD_DIRSERV,"No fingerprint found for '%s'",nickname);
     return 0;
   }
   if (0==strcasecmp(nn_ent->fingerprint, fp)) {
-    debug(LD_DIRSERV,"Good fingerprint for '%s'",router->nickname);
+    if (should_log)
+      debug(LD_DIRSERV,"Good fingerprint for '%s'",nickname);
     return FP_NAMED; /* Right fingerprint. */
   } else {
-    warn(LD_DIRSERV,"Mismatched fingerprint for '%s': expected '%s' got '%s'. ContactInfo '%s', platform '%s'.)",
-         router->nickname, nn_ent->fingerprint, fp,
-         router->contact_info ? router->contact_info : "",
-         router->platform ? router->platform : "");
+    if (should_log)
+      warn(LD_DIRSERV,
+           "Mismatched fingerprint for '%s': expected '%s' got '%s'. "
+           "ContactInfo '%s', platform '%s'.)",
+           nickname, nn_ent->fingerprint, fp,
+           contact ? contact : "",
+           platform ? platform : "");
     if (msg)
       *msg = "Rejected: There is already a verified server with this nickname and a different fingerprint.";
     return FP_REJECT; /* Wrong fingerprint. */
@@ -349,7 +401,7 @@ dirserv_router_has_valid_address(routerinfo_t *ri)
          ri->nickname, ri->address);
     return -1;
   }
-  if (is_internal_IP(ntohl(iaddr.s_addr))) {
+  if (is_internal_IP(ntohl(iaddr.s_addr), 0)) {
     info(LD_DIRSERV,
          "Router '%s' published internal IP address '%s'. Refusing.",
          ri->nickname, ri->address);
@@ -377,7 +429,9 @@ authdir_wants_to_reject_router(routerinfo_t *ri,
   /* Is there too much clock skew? */
   now = time(NULL);
   if (ri->cache_info.published_on > now+ROUTER_ALLOW_SKEW) {
-    notice(LD_DIRSERV, "Publication time for nickname '%s' is too far (%d minutes) in the future; possible clock skew. Not adding (ContactInfo '%s', platform '%s').",
+    notice(LD_DIRSERV, "Publication time for nickname '%s' is too far "
+           "(%d minutes) in the future; possible clock skew. Not adding "
+           "(ContactInfo '%s', platform '%s').",
            ri->nickname, (int)((ri->cache_info.published_on-now)/60),
            ri->contact_info ? ri->contact_info : "",
            ri->platform ? ri->platform : "");
@@ -385,7 +439,9 @@ authdir_wants_to_reject_router(routerinfo_t *ri,
     return -1;
   }
   if (ri->cache_info.published_on < now-ROUTER_MAX_AGE) {
-    notice(LD_DIRSERV, "Publication time for router with nickname '%s' is too far (%d minutes) in the past. Not adding (ContactInfo '%s', platform '%s').",
+    notice(LD_DIRSERV,
+           "Publication time for router with nickname '%s' is too far "
+           "(%d minutes) in the past. Not adding (ContactInfo '%s', platform '%s').",
            ri->nickname, (int)((now-ri->cache_info.published_on)/60),
            ri->contact_info ? ri->contact_info : "",
            ri->platform ? ri->platform : "");
@@ -456,12 +512,18 @@ dirserv_add_descriptor(const char *desc, const char **msg)
          ri->nickname);
     *msg = "Not replacing router descriptor; no information has changed since the last one with this identity.";
     routerinfo_free(ri);
+    control_event_or_authdir_new_descriptor("DROPPED", desc, *msg);
     return 0;
   }
   if ((r = router_add_to_routerlist(ri, msg, 0))<0) {
+    if (r < -1) /* unless the routerinfo was fine, just out-of-date */
+      control_event_or_authdir_new_descriptor("REJECTED", desc, *msg);
     return r == -1 ? 0 : -1;
   } else {
-    smartlist_t *changed = smartlist_create();
+    smartlist_t *changed;
+    control_event_or_authdir_new_descriptor("ACCEPTED", desc, *msg);
+
+    changed = smartlist_create();
     smartlist_add(changed, ri);
     control_event_descriptors_changed(changed);
     smartlist_free(changed);
@@ -491,7 +553,7 @@ directory_remove_invalid(void)
     switch (r) {
       case FP_REJECT:
         info(LD_DIRSERV, "Router '%s' is now rejected: %s",
-            ent->nickname, msg?msg:"");
+             ent->nickname, msg?msg:"");
         routerlist_remove(rl, ent, i--, 0);
         changed = 1;
         break;
@@ -616,8 +678,7 @@ dirserv_thinks_router_is_reachable(routerinfo_t *router, time_t now)
   connection_t *conn;
   if (router_is_me(router) && !we_are_hibernating())
     return 1;
-  conn = connection_get_by_identity_digest(router->cache_info.identity_digest,
-                                           CONN_TYPE_OR);
+  conn = connection_or_get_by_identity_digest(router->cache_info.identity_digest);
   if (conn && conn->state == OR_CONN_STATE_OPEN)
     return get_options()->AssumeReachable ||
            now < router->last_reachable + REACHABLE_TIMEOUT;
@@ -633,8 +694,7 @@ dirserv_thinks_router_is_blatantly_unreachable(routerinfo_t *router, time_t now)
   connection_t *conn;
   if (router->is_hibernating)
     return 0;
-  conn = connection_get_by_identity_digest(router->cache_info.identity_digest,
-                                           CONN_TYPE_OR);
+  conn = connection_or_get_by_identity_digest(router->cache_info.identity_digest);
   if (conn && conn->state == OR_CONN_STATE_OPEN &&
       now >= router->last_reachable + 2*REACHABLE_TIMEOUT &&
       router->testing_since &&
@@ -771,13 +831,13 @@ dirserv_dump_directory_to_string(char **dir_out,
      overruns.*/
 
   tor_snprintf(buf, buf_len,
-           "signed-directory\n"
-           "published %s\n"
-           "recommended-software %s\n"
-           "router-status %s\n"
-           "dir-signing-key\n%s\n",
-           published, recommended_versions, router_status,
-           identity_pkey);
+               "signed-directory\n"
+               "published %s\n"
+               "recommended-software %s\n"
+               "router-status %s\n"
+               "dir-signing-key\n%s\n",
+               published, recommended_versions, router_status,
+               identity_pkey);
 
   tor_free(recommended_versions);
   tor_free(router_status);
@@ -1135,6 +1195,14 @@ router_is_general_exit(routerinfo_t *ri)
 /** For authoritative directories: the current (v2) network status */
 static cached_dir_t the_v2_networkstatus = { NULL, NULL, 0, 0, 0 };
 
+static int
+should_generate_v2_networkstatus(void)
+{
+  return get_options()->AuthoritativeDir &&
+    the_v2_networkstatus_is_dirty &&
+    the_v2_networkstatus_is_dirty + DIR_REGEN_SLACK_TIME < time(NULL);
+}
+
 /** For authoritative directories only: replace the contents of
  * <b>the_v2_networkstatus</b> with a newly generated network status
  * object. */
@@ -1332,7 +1400,10 @@ dirserv_get_networkstatus_v2(smartlist_t *result,
         warn(LD_BUG,"Unable to generate an authoritative network status.");
     }
   } else if (!strcmp(key, "all")) {
-    digestmap_iter_t *iter = digestmap_iter_init(cached_v2_networkstatus);
+    digestmap_iter_t *iter;
+    if (should_generate_v2_networkstatus())
+      generate_v2_networkstatus();
+    iter = digestmap_iter_init(cached_v2_networkstatus);
     while (!digestmap_iter_done(iter)) {
       const char *ident;
       void *val;
@@ -1349,10 +1420,7 @@ dirserv_get_networkstatus_v2(smartlist_t *result,
     SMARTLIST_FOREACH(digests, char *, cp,
         {
           cached_dir_t *cached;
-          if (router_digest_is_me(cp) &&
-              get_options()->AuthoritativeDir &&
-              the_v2_networkstatus_is_dirty &&
-              the_v2_networkstatus_is_dirty + DIR_REGEN_SLACK_TIME < time(NULL))
+          if (router_digest_is_me(cp) && should_generate_v2_networkstatus())
             generate_v2_networkstatus();
           cached = digestmap_get(cached_v2_networkstatus, cp);
           if (cached) {
