@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dns.c,v 1.173 2005/10/25 19:01:48 nickm Exp $ */
-const char dns_c_id[] = "$Id: dns.c,v 1.173 2005/10/25 19:01:48 nickm Exp $";
+/* $Id: dns.c,v 1.177 2005/12/10 09:36:25 arma Exp $ */
+const char dns_c_id[] = "$Id: dns.c,v 1.177 2005/12/10 09:36:25 arma Exp $";
 
 /**
  * \file dns.c
@@ -18,7 +18,7 @@ const char dns_c_id[] = "$Id: dns.c,v 1.173 2005/10/25 19:01:48 nickm Exp $";
  */
 
 #include "or.h"
-#include "tree.h"
+#include "../common/ht.h"
 
 /** Longest hostname we're willing to resolve. */
 #define MAX_ADDRESSLEN 256
@@ -51,11 +51,11 @@ typedef struct pending_connection_t {
 } pending_connection_t;
 
 /** A DNS request: possibly completed, possibly pending; cached_resolve
- * structs are stored at the OR side in a splay tree, and as a linked
+ * structs are stored at the OR side in a hash table, and as a linked
  * list from oldest to newest.
  */
 typedef struct cached_resolve_t {
-  SPLAY_ENTRY(cached_resolve_t) node;
+  HT_ENTRY(cached_resolve_t) node;
   char address[MAX_ADDRESSLEN]; /**< The hostname to be resolved. */
   uint32_t addr; /**< IPv4 addr for <b>address</b>. */
   char state; /**< 0 is pending; 1 means answer is valid; 2 means resolve failed. */
@@ -76,34 +76,41 @@ static int spawn_dnsworker(void);
 static int spawn_enough_dnsworkers(void);
 static void send_resolved_cell(connection_t *conn, uint8_t answer_type);
 
-/** Splay tree of cached_resolve objects. */
-static SPLAY_HEAD(cache_tree, cached_resolve_t) cache_root;
+/** Hash table of cached_resolve objects. */
+static HT_HEAD(cache_map, cached_resolve_t) cache_root;
 
 /** Function to compare hashed resolves on their addresses; used to
- * implement splay trees. */
-static int
-compare_cached_resolves(cached_resolve_t *a,
-                        cached_resolve_t *b)
+ * implement hash tables. */
+static INLINE int
+cached_resolves_eq(cached_resolve_t *a, cached_resolve_t *b)
 {
   /* make this smarter one day? */
-  return strncmp(a->address, b->address, MAX_ADDRESSLEN);
+  return !strncmp(a->address, b->address, MAX_ADDRESSLEN);
 }
 
-SPLAY_PROTOTYPE(cache_tree, cached_resolve_t, node, compare_cached_resolves);
-SPLAY_GENERATE(cache_tree, cached_resolve_t, node, compare_cached_resolves);
+static INLINE unsigned int
+cached_resolve_hash(cached_resolve_t *a)
+{
+  return ht_string_hash(a->address);
+}
+
+HT_PROTOTYPE(cache_map, cached_resolve_t, node, cached_resolve_hash,
+             cached_resolves_eq);
+HT_GENERATE(cache_map, cached_resolve_t, node, cached_resolve_hash,
+            cached_resolves_eq, 0.6, malloc, realloc, free);
 
 /** Initialize the DNS cache. */
 static void
-init_cache_tree(void)
+init_cache_map(void)
 {
-  SPLAY_INIT(&cache_root);
+  HT_INIT(&cache_root);
 }
 
 /** Initialize the DNS subsystem; called by the OR process. */
 void
 dns_init(void)
 {
-  init_cache_tree();
+  init_cache_map();
   dnsworkers_rotate();
 }
 
@@ -123,12 +130,13 @@ _free_cached_resolve(cached_resolve_t *r)
 void
 dns_free_all(void)
 {
-  cached_resolve_t *ptr, *next;
-  for (ptr = SPLAY_MIN(cache_tree, &cache_root); ptr != NULL; ptr = next) {
-    next = SPLAY_NEXT(cache_tree, &cache_root, ptr);
-    SPLAY_REMOVE(cache_tree, &cache_root, ptr);
-    _free_cached_resolve(ptr);
+  cached_resolve_t **ptr, **next, *item;
+  for (ptr = HT_START(cache_map, &cache_root); ptr != NULL; ptr = next) {
+    item = *ptr;
+    next = HT_NEXT_RMV(cache_map, &cache_root, ptr);
+    _free_cached_resolve(item);
   }
+  HT_CLEAR(cache_map, &cache_root);
 }
 
 /** Linked list of resolved addresses, oldest to newest. */
@@ -150,7 +158,7 @@ purge_expired_resolves(uint32_t now)
   while (oldest_cached_resolve && (oldest_cached_resolve->expire < now)) {
     resolve = oldest_cached_resolve;
     debug(LD_EXIT,"Forgetting old cached resolve (address %s, expires %lu)",
-        safe_str(resolve->address), (unsigned long)resolve->expire);
+          safe_str(resolve->address), (unsigned long)resolve->expire);
     if (resolve->state == CACHE_STATE_PENDING) {
       debug(LD_EXIT,"Bug: Expiring a dns resolve ('%s') that's still pending. Forgot to cull it?", safe_str(resolve->address));
       tor_fragile_assert();
@@ -174,7 +182,7 @@ purge_expired_resolves(uint32_t now)
     oldest_cached_resolve = resolve->next;
     if (!oldest_cached_resolve) /* if there are no more, */
       newest_cached_resolve = NULL; /* then make sure the list's tail knows that too */
-    SPLAY_REMOVE(cache_tree, &cache_root, resolve);
+    HT_REMOVE(cache_map, &cache_root, resolve);
     tor_free(resolve);
   }
 }
@@ -217,8 +225,8 @@ send_resolved_cell(connection_t *conn, uint8_t answer_type)
                                conn->cpath_layer);
 }
 
-/** Link <b>r</b> into the tree of address-to-result mappings, and add it to
- * the linked list of resolves-by-age. */
+/** Link <b>r</b> into the hash table of address-to-result mappings, and add it
+ * to the linked list of resolves-by-age. */
 static void
 insert_resolve(cached_resolve_t *r)
 {
@@ -230,7 +238,7 @@ insert_resolve(cached_resolve_t *r)
   }
   newest_cached_resolve = r;
 
-  SPLAY_INSERT(cache_tree, &cache_root, r);
+  HT_INSERT(cache_map, &cache_root, r);
 }
 
 /** See if we have a cache entry for <b>exitconn</b>-\>address. if so,
@@ -265,15 +273,15 @@ dns_resolve(connection_t *exitconn)
   }
 
   /* then take this opportunity to see if there are any expired
-   * resolves in the tree. */
+   * resolves in the hash table. */
   purge_expired_resolves(now);
 
   /* lower-case exitconn->address, so it's in canonical form */
   tor_strlower(exitconn->address);
 
-  /* now check the tree to see if 'address' is already there. */
+  /* now check the hash table to see if 'address' is already there. */
   strlcpy(search.address, exitconn->address, sizeof(search.address));
-  resolve = SPLAY_FIND(cache_tree, &cache_root, &search);
+  resolve = HT_FIND(cache_map, &cache_root, &search);
   if (resolve) { /* already there */
     switch (resolve->state) {
       case CACHE_STATE_PENDING:
@@ -284,19 +292,19 @@ dns_resolve(connection_t *exitconn)
         pending_connection->next = resolve->pending_connections;
         resolve->pending_connections = pending_connection;
         debug(LD_EXIT,"Connection (fd %d) waiting for pending DNS resolve of '%s'",
-               exitconn->s, safe_str(exitconn->address));
+              exitconn->s, safe_str(exitconn->address));
         exitconn->state = EXIT_CONN_STATE_RESOLVING;
         return 0;
       case CACHE_STATE_VALID:
         exitconn->addr = resolve->addr;
         debug(LD_EXIT,"Connection (fd %d) found cached answer for '%s'",
-               exitconn->s, safe_str(exitconn->address));
+              exitconn->s, safe_str(exitconn->address));
         if (exitconn->purpose == EXIT_PURPOSE_RESOLVE)
           send_resolved_cell(exitconn, RESOLVED_TYPE_IPV4);
         return 1;
       case CACHE_STATE_FAILED:
         debug(LD_EXIT,"Connection (fd %d) found cached error for '%s'",
-               exitconn->s, safe_str(exitconn->address));
+              exitconn->s, safe_str(exitconn->address));
         if (exitconn->purpose == EXIT_PURPOSE_RESOLVE)
           send_resolved_cell(exitconn, RESOLVED_TYPE_ERROR);
         circ = circuit_get_by_edge_conn(exitconn);
@@ -351,7 +359,8 @@ assign_to_dnsworker(connection_t *exitconn)
   }
 
   debug(LD_EXIT,
-    "Connection (fd %d) needs to resolve '%s'; assigning to DNSWorker (fd %d)",
+        "Connection (fd %d) needs to resolve '%s'; assigning "
+        "to DNSWorker (fd %d)",
         exitconn->s, safe_str(exitconn->address), dnsconn->s);
 
   tor_free(dnsconn->address);
@@ -383,7 +392,7 @@ connection_dns_remove(connection_t *conn)
 
   strlcpy(search.address, conn->address, sizeof(search.address));
 
-  resolve = SPLAY_FIND(cache_tree, &cache_root, &search);
+  resolve = HT_FIND(cache_map, &cache_root, &search);
   if (!resolve) {
     /* XXXX RD This *is* a bug, right? -NM */
     notice(LD_BUG,"Address '%s' is not pending. Dropping.", safe_str(conn->address));
@@ -399,7 +408,7 @@ connection_dns_remove(connection_t *conn)
     resolve->pending_connections = pend->next;
     tor_free(pend);
     debug(LD_EXIT, "First connection (fd %d) no longer waiting for resolve of '%s'",
-           conn->s, safe_str(conn->address));
+          conn->s, safe_str(conn->address));
     return;
   } else {
     for ( ; pend->next; pend = pend->next) {
@@ -408,7 +417,7 @@ connection_dns_remove(connection_t *conn)
         pend->next = victim->next;
         tor_free(victim);
         debug(LD_EXIT, "Connection (fd %d) no longer waiting for resolve of '%s'",
-               conn->s, safe_str(conn->address));
+              conn->s, safe_str(conn->address));
         return; /* more are pending */
       }
     }
@@ -422,10 +431,10 @@ void
 assert_connection_edge_not_dns_pending(connection_t *conn)
 {
   pending_connection_t *pend;
-  cached_resolve_t *resolve;
+  cached_resolve_t **resolve;
 
-  SPLAY_FOREACH(resolve, cache_tree, &cache_root) {
-    for (pend = resolve->pending_connections;
+  HT_FOREACH(resolve, cache_map, &cache_root) {
+    for (pend = (*resolve)->pending_connections;
          pend;
          pend = pend->next) {
       tor_assert(pend->conn != conn);
@@ -439,10 +448,10 @@ void
 assert_all_pending_dns_resolves_ok(void)
 {
   pending_connection_t *pend;
-  cached_resolve_t *resolve;
+  cached_resolve_t **resolve;
 
-  SPLAY_FOREACH(resolve, cache_tree, &cache_root) {
-    for (pend = resolve->pending_connections;
+  HT_FOREACH(resolve, cache_map, &cache_root) {
+    for (pend = (*resolve)->pending_connections;
          pend;
          pend = pend->next) {
       assert_connection_ok(pend->conn, 0);
@@ -467,7 +476,7 @@ dns_cancel_pending_resolve(char *address)
 
   strlcpy(search.address, address, sizeof(search.address));
 
-  resolve = SPLAY_FIND(cache_tree, &cache_root, &search);
+  resolve = HT_FIND(cache_map, &cache_root, &search);
   if (!resolve) {
     /* XXXX RD This *is* a bug, right? -NM */
     notice(LD_BUG,"Address '%s' is not pending. Dropping.", safe_str(address));
@@ -477,7 +486,7 @@ dns_cancel_pending_resolve(char *address)
   if (!resolve->pending_connections) {
     /* XXX this should never trigger, but sometimes it does */
     warn(LD_BUG,"Bug: Address '%s' is pending but has no pending connections!",
-           safe_str(address));
+         safe_str(address));
     tor_fragile_assert();
     return;
   }
@@ -528,8 +537,8 @@ dns_purge_resolve(cached_resolve_t *resolve)
       newest_cached_resolve = tmp;
   }
 
-  /* remove resolve from the tree */
-  SPLAY_REMOVE(cache_tree, &cache_root, resolve);
+  /* remove resolve from the map */
+  HT_REMOVE(cache_map, &cache_root, resolve);
 
   tor_free(resolve);
 }
@@ -552,7 +561,7 @@ dns_found_answer(char *address, uint32_t addr, char outcome)
 
   strlcpy(search.address, address, sizeof(search.address));
 
-  resolve = SPLAY_FIND(cache_tree, &cache_root, &search);
+  resolve = HT_FIND(cache_map, &cache_root, &search);
   if (!resolve) {
     info(LD_EXIT,"Resolved unasked address '%s'; caching anyway.",
          safe_str(address));
@@ -696,7 +705,7 @@ connection_dns_process_inbuf(connection_t *conn)
       connection_fetch_from_buf(&success,1,conn);
       connection_fetch_from_buf((char *)&addr,sizeof(uint32_t),conn);
       warn(LD_EXIT,"Discarding idle dns answer (success %d, addr %d.)",
-             success, addr); // XXX safe_str
+           success, addr); // XXX safe_str
     }
     return 0;
   }
@@ -934,7 +943,7 @@ spawn_enough_dnsworkers(void)
     /* too many idle? */
     /* cull excess workers */
     info(LD_EXIT,"%d of %d dnsworkers are idle. Killing one.",
-           num_dnsworkers-num_dnsworkers_busy, num_dnsworkers);
+         num_dnsworkers-num_dnsworkers_busy, num_dnsworkers);
     dnsconn = connection_get_by_type_state(CONN_TYPE_DNSWORKER, DNSWORKER_STATE_IDLE);
     tor_assert(dnsconn);
     connection_mark_for_close(dnsconn);
