@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: main.c,v 1.589 2005/11/15 03:05:19 nickm Exp $ */
-const char main_c_id[] = "$Id: main.c,v 1.589 2005/11/15 03:05:19 nickm Exp $";
+/* $Id: main.c,v 1.600 2005/12/10 09:36:25 arma Exp $ */
+const char main_c_id[] = "$Id: main.c,v 1.600 2005/12/10 09:36:25 arma Exp $";
 
 /**
  * \file main.c
@@ -94,11 +94,10 @@ static char* nt_strerror(uint32_t errnum);
 
 #define FORCE_REGENERATE_DESCRIPTOR_INTERVAL 18*60*60 /* 18 hours */
 #define CHECK_DESCRIPTOR_INTERVAL 60 /* one minute */
-#define CHECK_IPADDRESS_INTERVAL (5*60) /* five minutes */
+#define CHECK_IPADDRESS_INTERVAL (15*60) /* 15 minutes */
 #define BUF_SHRINK_INTERVAL 60 /* one minute */
 #define DESCRIPTOR_RETRY_INTERVAL 10
 #define DESCRIPTOR_FAILURE_RESET_INTERVAL 60*60
-#define TIMEOUT_UNTIL_UNREACHABILITY_COMPLAINT (20*60) /* 20 minutes */
 #define ENTROPY_INTERVAL 60*60
 
 /********* END VARIABLES ************/
@@ -140,7 +139,7 @@ connection_add(connection_t *conn)
   nfds++;
 
   debug(LD_NET,"new conn type %s, socket %d, nfds %d.",
-      conn_type_to_string(conn->type), conn->s, nfds);
+        conn_type_to_string(conn->type), conn->s, nfds);
 
   return 0;
 }
@@ -198,6 +197,9 @@ connection_unlink(connection_t *conn, int remove)
   if (conn->type == CONN_TYPE_EXIT) {
     assert_connection_edge_not_dns_pending(conn);
   }
+  if (conn->type == CONN_TYPE_OR && !tor_digest_is_zero(conn->identity_digest)) {
+    connection_or_remove_from_identity_map(conn);
+  }
   connection_free(conn);
 }
 
@@ -261,8 +263,9 @@ connection_watch_events(connection_t *conn, short events)
 
   if (r<0)
     warn(LD_NET,
-         "Error from libevent setting read event state for %d to %swatched.",
-         conn->s, (events & EV_READ)?"":"un");
+         "Error from libevent setting read event state for %d to %swatched: %s",
+         conn->s, (events & EV_READ)?"":"un",
+         tor_socket_strerror(tor_socket_errno(conn->s)));
 
   if (events & EV_WRITE) {
     r = event_add(conn->write_event, NULL);
@@ -272,8 +275,9 @@ connection_watch_events(connection_t *conn, short events)
 
   if (r<0)
     warn(LD_NET,
-         "Error from libevent setting read event state for %d to %swatched.",
-         conn->s, (events & EV_WRITE)?"":"un");
+         "Error from libevent setting read event state for %d to %swatched: %s",
+         conn->s, (events & EV_WRITE)?"":"un",
+         tor_socket_strerror(tor_socket_errno(conn->s)));
 }
 
 /** Return true iff <b>conn</b> is listening for read events. */
@@ -328,7 +332,7 @@ connection_stop_writing(connection_t *conn)
 
   if (event_del(conn->write_event))
     warn(LD_NET, "Error from libevent setting write event state for %d to unwatched.",
-           conn->s);
+         conn->s);
 
 }
 
@@ -341,7 +345,7 @@ connection_start_writing(connection_t *conn)
 
   if (event_add(conn->write_event, NULL))
     warn(LD_NET, "Error from libevent setting write event state for %d to watched.",
-           conn->s);
+         conn->s);
 }
 
 /** Close all connections that have been scheduled to get closed */
@@ -375,7 +379,7 @@ conn_read_callback(int fd, short event, void *_conn)
     if (!conn->marked_for_close) {
 #ifndef MS_WINDOWS
       warn(LD_BUG,"Bug: unhandled error on read for %s connection (fd %d); removing",
-             conn_type_to_string(conn->type), conn->s);
+           conn_type_to_string(conn->type), conn->s);
       tor_fragile_assert();
 #endif
       if (CONN_IS_EDGE(conn))
@@ -444,10 +448,11 @@ conn_close_if_marked(int i)
     int sz = connection_bucket_write_limit(conn);
     if (!conn->hold_open_until_flushed)
       info(LD_NET,
-        "Conn (addr %s, fd %d, type %s, state %d) marked, but wants to flush %d bytes. "
-        "(Marked at %s:%d)",
-        conn->address, conn->s, conn_type_to_string(conn->type), conn->state,
-        (int)conn->outbuf_flushlen, conn->marked_for_close_file, conn->marked_for_close);
+           "Conn (addr %s, fd %d, type %s, state %d) marked, but wants "
+           "to flush %d bytes. (Marked at %s:%d)",
+           conn->address, conn->s, conn_type_to_string(conn->type), conn->state,
+           (int)conn->outbuf_flushlen,
+            conn->marked_for_close_file, conn->marked_for_close);
     if (connection_speaks_cells(conn)) {
       if (conn->state == OR_CONN_STATE_OPEN) {
         retval = flush_buf_tls(conn->tls, conn->outbuf, sz, &conn->outbuf_flushlen);
@@ -471,7 +476,9 @@ conn_close_if_marked(int i)
         severity = LOG_INFO;
       else
         severity = LOG_NOTICE;
-      log_fn(severity, LD_NET, "Something wrong with your network connection? Conn (addr %s, fd %d, type %s, state %d) tried to write %d bytes but timed out. (Marked at %s:%d)",
+      log_fn(severity, LD_NET, "Something wrong with your network connection? "
+             "Conn (addr %s, fd %d, type %s, state %d) tried to write %d "
+             "bytes but timed out. (Marked at %s:%d)",
              safe_str(conn->address), conn->s, conn_type_to_string(conn->type),
              conn->state,
              (int)buf_datalen(conn->outbuf), conn->marked_for_close_file,
@@ -584,7 +591,7 @@ run_connection_housekeeping(int i, time_t now)
       !conn->marked_for_close &&
       conn->timestamp_lastwritten + 5*60 < now) {
     info(LD_DIR,"Expiring wedged directory conn (fd %d, purpose %d)",
-           conn->s, conn->purpose);
+         conn->s, conn->purpose);
     /* This check is temporary; it's to let us know whether we should consider
      * parsing partial serverdesc responses. */
     if (conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC &&
@@ -597,26 +604,47 @@ run_connection_housekeeping(int i, time_t now)
     return;
   }
 
+  if (!connection_speaks_cells(conn))
+    return; /* we're all done here, the rest is just for OR conns */
+
+#define TIME_BEFORE_OR_CONN_IS_OBSOLETE (60*60*24*7) /* a week */
+  if (!conn->is_obsolete &&
+      conn->timestamp_created + TIME_BEFORE_OR_CONN_IS_OBSOLETE < now) {
+    info(LD_OR, "Marking OR conn to %s:%d obsolete (fd %d, %d secs old).",
+         conn->address, conn->port, conn->s,
+         (int)(now - conn->timestamp_created));
+    conn->is_obsolete = 1;
+  }
+
+  if (conn->is_obsolete && conn->n_circuits == 0) {
+    /* no unmarked circs -- mark it now */
+    info(LD_OR,"Expiring non-used OR connection to fd %d (%s:%d) [Obsolete].",
+         conn->s,conn->address, conn->port);
+    connection_mark_for_close(conn);
+    conn->hold_open_until_flushed = 1;
+    return;
+  }
+
   /* If we haven't written to an OR connection for a while, then either nuke
      the connection or send a keepalive, depending. */
-  if (connection_speaks_cells(conn) &&
-      now >= conn->timestamp_lastwritten + options->KeepalivePeriod) {
+  if (now >= conn->timestamp_lastwritten + options->KeepalivePeriod) {
     routerinfo_t *router = router_get_by_digest(conn->identity_digest);
     if (!connection_state_is_open(conn)) {
       info(LD_OR,"Expiring non-open OR connection to fd %d (%s:%d).",
            conn->s,conn->address, conn->port);
       connection_mark_for_close(conn);
       conn->hold_open_until_flushed = 1;
-    } else if (we_are_hibernating() && !circuit_get_by_conn(conn) &&
+    } else if (we_are_hibernating() && conn->n_circuits == 0 &&
                !buf_datalen(conn->outbuf)) {
       info(LD_OR,"Expiring non-used OR connection to fd %d (%s:%d) [Hibernating or exiting].",
              conn->s,conn->address, conn->port);
       connection_mark_for_close(conn);
       conn->hold_open_until_flushed = 1;
-    } else if (!clique_mode(options) && !circuit_get_by_conn(conn) &&
-               (!router || !server_mode(options) || !router_is_clique_mode(router))) {
+    } else if (!clique_mode(options) && conn->n_circuits &&
+               (!router || !server_mode(options) ||
+                !router_is_clique_mode(router))) {
       info(LD_OR,"Expiring non-used OR connection to fd %d (%s:%d) [Not in clique mode].",
-             conn->s,conn->address, conn->port);
+           conn->s,conn->address, conn->port);
       connection_mark_for_close(conn);
       conn->hold_open_until_flushed = 1;
     } else if (
@@ -896,11 +924,15 @@ second_elapsed_callback(int fd, short event, void *args)
     /* every 20 minutes, check and complain if necessary */
     routerinfo_t *me = router_get_my_routerinfo();
     if (me && !check_whether_orport_reachable())
-      warn(LD_CONFIG,"Your server (%s:%d) has not managed to confirm that its ORPort is reachable. Please check your firewalls, ports, address, /etc/hosts file, etc.",
-          me->address, me->or_port);
+      warn(LD_CONFIG,"Your server (%s:%d) has not managed to confirm that "
+           "its ORPort is reachable. Please check your firewalls, ports, "
+           "address, /etc/hosts file, etc.",
+           me->address, me->or_port);
     if (me && !check_whether_dirport_reachable())
-      warn(LD_CONFIG,"Your server (%s:%d) has not managed to confirm that its DirPort is reachable. Please check your firewalls, ports, address, /etc/hosts file, etc.",
-          me->address, me->dir_port);
+      warn(LD_CONFIG,"Your server (%s:%d) has not managed to confirm that its "
+           "DirPort is reachable. Please check your firewalls, ports, "
+           "address, /etc/hosts file, etc.",
+           me->address, me->dir_port);
   }
 
   /* if more than 100s have elapsed, probably the clock jumped: doesn't count. */
@@ -1010,7 +1042,7 @@ do_main_loop(void)
 {
   int loop_result;
 
-  dns_init(); /* initialize dns resolve tree, spawn workers if needed */
+  dns_init(); /* initialize dns resolve map, spawn workers if needed */
 
   handle_signals(1);
 
@@ -1071,10 +1103,10 @@ do_main_loop(void)
       if (e != EINTR && !ERRNO_IS_EINPROGRESS(e)) {
 #ifdef HAVE_EVENT_GET_METHOD
         err(LD_NET,"libevent poll with %s failed: %s [%d]",
-               event_get_method(), tor_socket_strerror(e), e);
+            event_get_method(), tor_socket_strerror(e), e);
 #else
         err(LD_NET,"libevent poll failed: %s [%d]",
-               tor_socket_strerror(e), e);
+            tor_socket_strerror(e), e);
 #endif
         return -1;
 #ifndef MS_WINDOWS
@@ -1200,6 +1232,7 @@ dumpmemusage(int severity)
       nfds);
   log(severity, LD_GENERAL, "In rephist: "U64_FORMAT" used by %d Tors.",
       U64_PRINTF_ARG(rephist_total_alloc), rephist_total_num);
+  dump_routerlist_mem_usage(severity);
 }
 
 /** Write all statistics to the log, with log level 'severity'.  Called
@@ -1217,44 +1250,44 @@ dumpstats(int severity)
   for (i=0;i<nfds;i++) {
     conn = connection_array[i];
     log(severity, LD_GENERAL, "Conn %d (socket %d) type %d (%s), state %d (%s), created %d secs ago",
-      i, conn->s, conn->type, conn_type_to_string(conn->type),
+        i, conn->s, conn->type, conn_type_to_string(conn->type),
         conn->state, conn_state_to_string(conn->type, conn->state), (int)(now - conn->timestamp_created));
     if (!connection_is_listener(conn)) {
       log(severity,LD_GENERAL,"Conn %d is to '%s:%d'.",i,safe_str(conn->address), conn->port);
       log(severity,LD_GENERAL, "Conn %d: %d bytes waiting on inbuf (len %d, last read %d secs ago)",i,
-             (int)buf_datalen(conn->inbuf),
-             (int)buf_capacity(conn->inbuf),
-             (int)(now - conn->timestamp_lastread));
+          (int)buf_datalen(conn->inbuf),
+          (int)buf_capacity(conn->inbuf),
+          (int)(now - conn->timestamp_lastread));
       log(severity,LD_GENERAL, "Conn %d: %d bytes waiting on outbuf (len %d, last written %d secs ago)",i,
-             (int)buf_datalen(conn->outbuf),
-             (int)buf_capacity(conn->outbuf),
-             (int)(now - conn->timestamp_lastwritten));
+          (int)buf_datalen(conn->outbuf),
+          (int)buf_capacity(conn->outbuf),
+          (int)(now - conn->timestamp_lastwritten));
     }
     circuit_dump_by_conn(conn, severity); /* dump info about all the circuits using this conn */
   }
   log(severity, LD_NET,
-         "Cells processed: %10lu padding\n"
-         "                 %10lu create\n"
-         "                 %10lu created\n"
-         "                 %10lu relay\n"
-         "                        (%10lu relayed)\n"
-         "                        (%10lu delivered)\n"
-         "                 %10lu destroy",
-         stats_n_padding_cells_processed,
-         stats_n_create_cells_processed,
-         stats_n_created_cells_processed,
-         stats_n_relay_cells_processed,
-         stats_n_relay_cells_relayed,
-         stats_n_relay_cells_delivered,
-         stats_n_destroy_cells_processed);
+      "Cells processed: %10lu padding\n"
+      "                 %10lu create\n"
+      "                 %10lu created\n"
+      "                 %10lu relay\n"
+      "                        (%10lu relayed)\n"
+      "                        (%10lu delivered)\n"
+      "                 %10lu destroy",
+      stats_n_padding_cells_processed,
+      stats_n_create_cells_processed,
+      stats_n_created_cells_processed,
+      stats_n_relay_cells_processed,
+      stats_n_relay_cells_relayed,
+      stats_n_relay_cells_delivered,
+      stats_n_destroy_cells_processed);
   if (stats_n_data_cells_packaged)
     log(severity,LD_NET,"Average packaged cell fullness: %2.3f%%",
-           100*(((double)stats_n_data_bytes_packaged) /
-                (stats_n_data_cells_packaged*RELAY_PAYLOAD_SIZE)) );
+        100*(((double)stats_n_data_bytes_packaged) /
+             (stats_n_data_cells_packaged*RELAY_PAYLOAD_SIZE)) );
   if (stats_n_data_cells_received)
     log(severity,LD_NET,"Average delivered cell fullness: %2.3f%%",
-           100*(((double)stats_n_data_bytes_received) /
-                (stats_n_data_cells_received*RELAY_PAYLOAD_SIZE)) );
+        100*(((double)stats_n_data_bytes_received) /
+             (stats_n_data_cells_received*RELAY_PAYLOAD_SIZE)) );
 
   if (now - time_of_process_start >= 0)
     elapsed = now - time_of_process_start;
@@ -1318,7 +1351,7 @@ handle_signals(int is_parent)
                  (void*)(uintptr_t)signals[i]);
       if (signal_add(&signal_events[i], NULL))
         warn(LD_BUG, "Error from libevent when adding event for signal %d",
-               signals[i]);
+             signals[i]);
     }
   } else {
     struct sigaction action;
