@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: router.c,v 1.229 2005/11/05 20:15:27 nickm Exp $ */
-const char router_c_id[] = "$Id: router.c,v 1.229 2005/11/05 20:15:27 nickm Exp $";
+/* $Id: router.c,v 1.234 2005/12/10 09:36:26 arma Exp $ */
+const char router_c_id[] = "$Id: router.c,v 1.234 2005/12/10 09:36:26 arma Exp $";
 
 #include "or.h"
 
@@ -53,16 +53,6 @@ get_onion_key(void)
   return onionkey;
 }
 
-/** Return the onion key that was current before the most recent onion
- * key rotation.  If no rotation has been performed since this process
- * started, return NULL.
- */
-crypto_pk_env_t *
-get_previous_onion_key(void)
-{
-  return lastonionkey;
-}
-
 /** Store a copy of the current onion key into *<b>key</b>, and a copy
  * of the most recent onion key into *<b>last</b>.
  */
@@ -178,7 +168,7 @@ init_key_from_file_name_changed(const char *fname_old,
   /* The old filename exists, and the new one doesn't.  Rename and load. */
   if (rename(fname_old, fname_new) < 0) {
     log_fn(LOG_ERR, LD_FS, "Couldn't rename \"%s\" to \"%s\": %s",
-        fname_old, fname_new, strerror(errno));
+           fname_old, fname_new, strerror(errno));
     return NULL;
   }
   return init_key_from_file(fname_new);
@@ -476,7 +466,7 @@ router_orport_found_reachable(void)
   if (!can_reach_or_port) {
     if (!clique_mode(get_options()))
       notice(LD_OR,"Self-testing indicates your ORPort is reachable from the outside. Excellent.%s",
-          get_options()->NoPublish ? "" : " Publishing server descriptor.");
+            get_options()->NoPublish ? "" : " Publishing server descriptor.");
     can_reach_or_port = 1;
     mark_my_descriptor_dirty();
     consider_publishable_server(time(NULL), 1);
@@ -623,11 +613,10 @@ router_retry_connections(int force)
     if (!clique_mode(options) && !router_is_clique_mode(router))
       continue;
     if (force ||
-        !connection_get_by_identity_digest(router->cache_info.identity_digest,
-                                           CONN_TYPE_OR)) {
+      !connection_or_get_by_identity_digest(router->cache_info.identity_digest)) {
       debug(LD_OR,"%sconnecting to %s at %s:%u.",
-             clique_mode(options) ? "(forced) " : "",
-             router->nickname, router->address, router->or_port);
+            clique_mode(options) ? "(forced) " : "",
+            router->nickname, router->address, router->or_port);
       /* Remember when we started trying to determine reachability */
       if (!router->testing_since)
         router->testing_since = now;
@@ -831,8 +820,8 @@ router_rebuild_descriptor(int force)
        if (!member) {
          if (!smartlist_string_isin(warned_nonexistent_family, name)) {
            warn(LD_CONFIG, "I have no descriptor for the router named \"%s\" "
-                  "in my declared family; I'll use the nickname as is, but "
-                  "this may confuse clients.", name);
+                "in my declared family; I'll use the nickname as is, but "
+                "this may confuse clients.", name);
            smartlist_add(warned_nonexistent_family, tor_strdup(name));
          }
          smartlist_add(ri->declared_family, name);
@@ -910,15 +899,12 @@ check_descriptor_bandwidth_changed(time_t now)
   }
 }
 
-#define MAX_IPADDRESS_CHANGE_FREQ 60*60
 /** Check whether our own address as defined by the Address configuration
  * has changed. This is for routers that get their address from a service
  * like dyndns. If our address has changed, mark our descriptor dirty. */
 void
 check_descriptor_ipaddress_changed(time_t now)
 {
-  static time_t last_changed = 0;
-  static time_t last_warned_lastchangetime = 0;
   uint32_t prev, cur;
   or_options_t *options = get_options();
 
@@ -943,19 +929,8 @@ check_descriptor_ipaddress_changed(time_t now)
     in_cur.s_addr = htonl(cur);
     tor_inet_ntoa(&in_cur, addrbuf_cur, sizeof(addrbuf_cur));
 
-    if (last_changed+MAX_IPADDRESS_CHANGE_FREQ < now) {
-      info(LD_GENERAL,"Our IP Address has changed from %s to %s; rebuilding descriptor.", addrbuf_prev, addrbuf_cur);
-      mark_my_descriptor_dirty();
-      last_changed = now;
-      last_warned_lastchangetime = 0;
-    }
-    else
-    {
-      if (last_warned_lastchangetime != last_changed) {
-        warn(LD_GENERAL,"Our IP Address seems to be flapping.  It has changed twice within one hour (from %s to %s this time).  Ignoring for now.", addrbuf_prev, addrbuf_cur);
-        last_warned_lastchangetime = last_changed;
-      }
-    }
+    info(LD_GENERAL,"Our IP Address has changed from %s to %s; rebuilding descriptor.", addrbuf_prev, addrbuf_cur);
+    mark_my_descriptor_dirty();
   }
 }
 
