@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerparse.c,v 1.159 2005/11/05 20:15:27 nickm Exp $ */
-const char routerparse_c_id[] = "$Id: routerparse.c,v 1.159 2005/11/05 20:15:27 nickm Exp $";
+/* $Id: routerparse.c,v 1.162 2005/12/10 09:36:26 arma Exp $ */
+const char routerparse_c_id[] = "$Id: routerparse.c,v 1.162 2005/12/10 09:36:26 arma Exp $";
 
 /**
  * \file routerparse.c
@@ -135,7 +135,7 @@ static struct {
   { "dir-signing-key",     K_DIR_SIGNING_KEY,     ARGS,    OBJ_OK,
                                                                 DIR|NETSTATUS},
   { "family",              K_FAMILY,              ARGS,    NO_OBJ,  RTR },
-  { "fingerprint",         K_FINGERPRINT,         ARGS,    NO_OBJ, ANYSIGNED },
+  { "fingerprint",         K_FINGERPRINT,     CONCAT_ARGS, NO_OBJ, ANYSIGNED },
   { "hibernating",         K_HIBERNATING,         ARGS,    NO_OBJ,  RTR },
   { "read-history",        K_READ_HISTORY,        ARGS,    NO_OBJ,  RTR },
   { "write-history",       K_WRITE_HISTORY,       ARGS,    NO_OBJ,  RTR },
@@ -271,7 +271,7 @@ tor_version_is_obsolete(const char *myversion, const char *versionlist)
   vl = versionlist;
 
   debug(LD_CONFIG,"Checking whether version '%s' is in '%s'",
-         myversion, versionlist);
+        myversion, versionlist);
 
   if (tor_version_parse(myversion, &mine)) {
     err(LD_BUG,"I couldn't parse my own version (%s)", myversion);
@@ -467,7 +467,7 @@ router_parse_runningrouters(const char *str)
   }
   if ((tok = find_first_by_keyword(tokens, _UNRECOGNIZED))) {
     warn(LD_DIR, "Unrecognized keyword '%s'; can't parse running-routers",
-           tok->args[0]);
+         tok->args[0]);
     goto err;
   }
   tok = smartlist_get(tokens,0);
@@ -731,8 +731,10 @@ router_parse_entry_from_string(const char *s, const char *end)
     goto err;
   }
   if ((tok = find_first_by_keyword(tokens, _UNRECOGNIZED))) {
-    warn(LD_DIR, "Unrecognized critical keyword '%s'; skipping descriptor. (It may be from another version of Tor.)",
-           tok->args[0]);
+    warn(LD_DIR,
+         "Unrecognized critical keyword '%s'; skipping descriptor. "
+         "(It may be from another version of Tor.)",
+         tok->args[0]);
     goto err;
   }
 
@@ -830,7 +832,7 @@ router_parse_entry_from_string(const char *s, const char *end)
   }
   if (crypto_pk_keysize(tok->key) != PK_BYTES) {
     warn(LD_DIR, "Wrong size on onion key: %d bits!",
-           (int)crypto_pk_keysize(tok->key)*8);
+         (int)crypto_pk_keysize(tok->key)*8);
     goto err;
   }
   router->onion_pkey = tok->key;
@@ -851,6 +853,25 @@ router_parse_entry_from_string(const char *s, const char *end)
     warn(LD_DIR, "Couldn't calculate key digest"); goto err;
   }
 
+  if ((tok = find_first_by_keyword(tokens, K_FINGERPRINT))) {
+    /* If there's a fingerprint line, it must match the identity digest. */
+    char d[DIGEST_LEN];
+    if (tok->n_args < 1) {
+      warn(LD_DIR, "Too few arguments to fingerprint");
+      goto err;
+    }
+    tor_strstrip(tok->args[0], " ");
+    if (base16_decode(d, DIGEST_LEN, tok->args[0], strlen(tok->args[0]))) {
+      warn(LD_DIR, "Couldn't decode fingerprint '%s'", tok->args[0]);
+      goto err;
+    }
+    if (memcmp(d,router->cache_info.identity_digest, DIGEST_LEN)!=0) {
+      warn(LD_DIR, "Fingerprint '%s' does not match identity digest.",
+           tok->args[0]);
+      goto err;
+    }
+  }
+
   if ((tok = find_first_by_keyword(tokens, K_PLATFORM))) {
     router->platform = tor_strdup(tok->args[0]);
   }
@@ -975,7 +996,7 @@ routerstatus_parse_entry_from_string(const char **s, smartlist_t *tokens)
   }
   if ((tok = find_first_by_keyword(tokens, _UNRECOGNIZED))) {
     warn(LD_DIR, "Unrecognized keyword \"%s\" in router status; skipping.",
-           tok->args[0]);
+         tok->args[0]);
     goto err;
   }
   if (!(tok = find_first_by_keyword(tokens, K_R))) {
@@ -984,13 +1005,13 @@ routerstatus_parse_entry_from_string(const char **s, smartlist_t *tokens)
   }
   if (tok->n_args < 8) {
     warn(LD_DIR,
-           "Too few arguments to 'r' keywork in router status; skipping.");
+         "Too few arguments to 'r' keywork in router status; skipping.");
   }
   rs = tor_malloc_zero(sizeof(routerstatus_t));
 
   if (!is_legal_nickname(tok->args[0])) {
     warn(LD_DIR,
-           "Invalid nickname '%s' in router status; skipping.", tok->args[0]);
+         "Invalid nickname '%s' in router status; skipping.", tok->args[0]);
     goto err;
   }
   strlcpy(rs->nickname, tok->args[0], sizeof(rs->nickname));
@@ -1060,12 +1081,6 @@ _compare_routerstatus_entries(const void **_a, const void **_b)
   return memcmp(a->identity_digest, b->identity_digest, DIGEST_LEN);
 }
 
-void
-sort_routerstatus_entries(smartlist_t *sl)
-{
-  smartlist_sort(sl, _compare_routerstatus_entries);
-}
-
 /** Given a versioned (v2 or later) network-status object in <b>s</b>, try to
  * parse it and return the result.  Return NULL on failure.  Check the
  * signature of the network status, but do not (yet) check the signing key for
@@ -1095,7 +1110,7 @@ networkstatus_parse_from_string(const char *s)
   }
   if ((tok = find_first_by_keyword(tokens, _UNRECOGNIZED))) {
     warn(LD_DIR, "Unrecognized keyword '%s'; can't parse network-status",
-           tok->args[0]);
+         tok->args[0]);
     goto err;
   }
   ns = tor_malloc_zero(sizeof(networkstatus_t));
@@ -1300,23 +1315,6 @@ router_parse_addr_policy_from_string(const char *s, int assume_action)
   return r;
 }
 
-/** Given an exit policicy stored in <b>s</b>, parse it and add it to the end
- * of the exit policy of <b>router</b>.  Return 0 on success, -1 on failure.
- */
-int
-router_add_exit_policy_from_string(routerinfo_t *router, const char *s)
-{
-  addr_policy_t *newe, *tmpe;
-  newe = router_parse_addr_policy_from_string(s, -1);
-  if (!newe)
-    return -1;
-  for (tmpe = router->exit_policy; tmpe; tmpe=tmpe->next)
-    ;
-  tmpe->next = newe;
-
-  return 0;
-}
-
 /** Add an exit policy stored in the token <b>tok</b> to the router info in
  * <b>router</b>.  Return 0 on success, -1 on failure. */
 static int
@@ -1690,7 +1688,7 @@ router_get_hash_impl(const char *s, char *digest,
   }
   if (start != s && *(start-1) != '\n') {
     warn(LD_DIR, "first occurrence of \"%s\" is not at the start of a line",
-           start_str);
+         start_str);
     return -1;
   }
   end = strstr(start+strlen(start_str), end_str);
