@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: test.c,v 1.211 2005/11/05 20:15:27 nickm Exp $ */
-const char test_c_id[] = "$Id: test.c,v 1.211 2005/11/05 20:15:27 nickm Exp $";
+/* $Id: test.c,v 1.215 2005/12/08 04:03:12 nickm Exp $ */
+const char test_c_id[] = "$Id: test.c,v 1.215 2005/12/08 04:03:12 nickm Exp $";
 
 /**
  * \file test.c
@@ -972,27 +972,13 @@ test_gzip(void)
   tor_free(buf1);
 }
 
-static void *
-_squareAndRemoveK4(const char *key, void*val, void *data)
-{
-  int *ip = (int*)data;
-  intptr_t v;
-  if (strcmp(key,"K4") == 0) {
-    ++(*ip);
-    return NULL;
-  }
-  v = (intptr_t)val;
-  return (void*)(v*v);
-}
-
 static void
 test_strmap(void)
 {
   strmap_t *map;
-  strmap_iter_t *iter;
-  const char *k;
+//  strmap_iter_t *iter;
+//  const char *k;
   void *v;
-  int count;
 
   map = strmap_new();
   v = strmap_set(map, "K1", (void*)99);
@@ -1016,13 +1002,7 @@ test_strmap(void)
   strmap_set(map, "K5", (void*)104);
   strmap_set(map, "K6", (void*)105);
 
-  count = 0;
-  strmap_foreach(map, _squareAndRemoveK4, &count);
-  test_eq(count, 1);
-  test_eq_ptr(strmap_get(map, "K4"), NULL);
-  test_eq_ptr(strmap_get(map, "K1"), (void*)10000);
-  test_eq_ptr(strmap_get(map, "K6"), (void*)11025);
-
+#if 0
   iter = strmap_iter_init(map);
   strmap_iter_get(iter,&k,&v);
   test_streq(k, "K1");
@@ -1045,6 +1025,7 @@ test_strmap(void)
   /* Make sure we removed K2, but not the others. */
   test_eq_ptr(strmap_get(map, "K2"), NULL);
   test_eq_ptr(strmap_get(map, "K5"), (void*)10816);
+#endif
 
   /* Clean up after ourselves. */
   strmap_free(map, NULL);
@@ -1566,6 +1547,37 @@ test_rend_fns(void)
   crypto_free_pk_env(pk2);
 }
 
+static void
+bench_aes(void)
+{
+  int len, i;
+  char *b1, *b2;
+  crypto_cipher_env_t *c;
+  struct timeval start, end;
+  const int iters = 100000;
+  uint64_t nsec;
+  c = crypto_new_cipher_env();
+  crypto_cipher_generate_key(c);
+  crypto_cipher_encrypt_init_cipher(c);
+  for (len = 1; len <= 8192; len *= 2) {
+    b1 = tor_malloc_zero(len);
+    b2 = tor_malloc_zero(len);
+    tor_gettimeofday(&start);
+    for (i = 0; i < iters; ++i) {
+      crypto_cipher_encrypt(c, b1, b2, len);
+    }
+    tor_gettimeofday(&end);
+    tor_free(b1);
+    tor_free(b2);
+    nsec = (uint64_t) tv_udiff(&start,&end);
+    nsec *= 1000;
+    nsec /= (iters*len);
+    printf("%d bytes: "U64_FORMAT" nsec per byte\n", len,
+           U64_PRINTF_ARG(nsec));
+  }
+  crypto_free_cipher_env(c);
+}
+
 int
 main(int c, char**v)
 {
@@ -1579,6 +1591,11 @@ main(int c, char**v)
 
   crypto_seed_rng();
 
+  if (0) {
+    bench_aes();
+    return 0;
+  }
+
   rep_hist_init();
   atexit(remove_directory);
 
